/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se;

import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.Constants;
import org.objectweb.jalisto.se.api.internal.InternalFactory;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.meta.ClassDescriptionImpl;
import org.objectweb.jalisto.se.impl.meta.FieldDescriptionImpl;
import org.objectweb.jalisto.se.impl.util.JalistoUtils;

import java.io.IOException;
import java.util.Properties;

/**
 * JalistoFactory is used to create new connection to Jalisto, represented by a Session object.
 * JalistoFactory need properties file path of the base to create Session instances.
 */
public class JalistoFactory {

    private static InternalFactory internalFactory;
    private static String internalFactoryClass;

    public static synchronized InternalFactory getInternalFactory(String propertiesFilePath) {
        if (internalFactory == null) {
            try {
                Properties props = new Properties();
                JalistoUtils.loadPropertiesFileIn(propertiesFilePath, props);
                internalFactoryClass = props.getProperty(JalistoProperties.INTERNAL_FACTORY_CLASS_KEY,
                                                         Constants.INTERNALFACTORYCLASS_DIC);
            } catch (IOException ioe) {
                throw new JalistoException("error during initial loading of "+propertiesFilePath, ioe);
            }
        }
        return getInternalFactory();
    }

    public static synchronized InternalFactory getInternalFactory() {
        if (internalFactory == null) {
            try {
                internalFactory = (InternalFactory) Class.forName(internalFactoryClass).newInstance();
                internalFactory.init();
                internalFactory.setSelfInstance();
            } catch (InstantiationException e) {
                throw new JalistoException("cannot instanciate "+internalFactoryClass, e);
            } catch (IllegalAccessException e) {
                throw new JalistoException("cannot initialize "+internalFactoryClass, e);
            } catch (ClassNotFoundException e) {
                throw new JalistoException("cannot find "+internalFactoryClass, e);
            }
        }
        return internalFactory;
    }

    /**
     * Create a new instance of Session interface, working on the base corresponding to the properties file specified.
     * 
     * @param propertiesFilePath properties file path. This path can be absolute, relative to working directory, or can be the name of a file in the working classpath.
     * @return Session the new Session instance just created
     * @throws org.objectweb.jalisto.se.exception.JalistoException
     *          Thrown if a problem occured during initialisation of the session.
     */
    public static synchronized Session getSession(String propertiesFilePath) {
        return getInternalFactory(propertiesFilePath).getSession(propertiesFilePath);
    }

    public static ClassDescription createClassDescription(String fullClassName) {
        return new ClassDescriptionImpl(fullClassName);
    }

    public static FieldDescription createFieldDescription(String fieldName, MetaType type) {
        return new FieldDescriptionImpl(fieldName, type);
    }

    public static void launchJMXServer(Session session) {
        getInternalFactory().launchMBeanHtmlServer(session.getInternalSession().getProperties());
    }

    public static void launchJMXServer(String jalistoPropertiesPath) {
        JalistoProperties properties = getInternalFactory(jalistoPropertiesPath).getProperties(jalistoPropertiesPath);
        getInternalFactory().launchMBeanHtmlServer(properties);
    }
}
