/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.Constants;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.util.JalistoUtils;

import java.io.File;
import java.io.IOException;
import java.util.*;

public class JalistoPropertiesImpl implements JalistoProperties {

    private static final String NAME = "jalisto";
    private static final String IMPLEMENTATION = MONO_IMPLEMENTATION;
    private static final String DATABASE_FILES_PATHS = "";
    private static final String DATABASE_INITIAL_SIZE = "1000";
    private static final String CACHE_IMPLEMENTATION = Constants.CACHE_DIC;
    private static final String OBJECT_CACHE_SIZE = "0";
    private static final String PAGE_CACHE_SIZE = "20";
    private static final String CACHE_CLEAR_POURCENT = "20";
    private static final String OIDTABLE_SIZE = "10000";
    private static final String OBJECT_INST_PAGE_SIZE = "200";
    private static final String OBJECT_SYST_PAGE_SIZE = "10";
    private static final String OBJECT_CLASS_PAGE_SIZE = "10";
    private static final String OBJECT_OID_PAGE_SIZE = "50";
    private static final String CONCURRENCY_MODE = Constants.OPTIMISTIC_MODE;
    private static final String TIME_OUT = "100";
    private static final String ADD_SPACE = "15";
    private static final String INTERNAL_KEY_MAX_SIZE = "12";
    private static final String ALLOWS_SPECIAL_FUNCTIONNALITIES = "yes";
    private static final String MBEAN_JMX_SERVER_PORT = "8082";
    private static final String PHYSICAL_ACCESS_CLASS = Constants.PHYSICAL_FILE_ACCESS_DIC;
    private static final String INTERNAL_FACTORY_CLASS = Constants.INTERNALFACTORYCLASS_DIC;
    private static final String COMMUNICATION_FACTORY_CLASS = Constants.COMMUNICATION_FACTORY_DIC;
    private static final String QUERY_MANAGER_CLASS = Constants.QUERY_MANAGER;
    private static final String MBEAN_SERVER_CLASS = Constants.JMX_SERVER_DIC;
    private static final String KEEP_IN_MEMORY = "yes";
    private static final String CLEAN_LOG = "yes";
    private static final String LOG_FILE = "log.jalisto";
    private static final String TRACE = "";

    private File parent;
    private Properties prop;
    private String[] pathes;
    private String propertiesPath;

    static final long serialVersionUID = -7589477069041161459L;

    public JalistoPropertiesImpl(String databasePropertiesFilePath) {
        this.propertiesPath = databasePropertiesFilePath;

        try {
            prop = new Properties();
            prop.put(NAME_KEY, NAME);
            prop.put(IMPLEMENTATION_KEY, IMPLEMENTATION);
            prop.put(DATABASE_FILES_PATHS_KEY, DATABASE_FILES_PATHS);
            prop.put(DATABASE_INITIAL_SIZE_KEY, DATABASE_INITIAL_SIZE);
            prop.put(CACHE_IMPLEMENTATION_KEY, CACHE_IMPLEMENTATION);
            prop.put(OBJECT_CACHE_SIZE_KEY, OBJECT_CACHE_SIZE);
            prop.put(PAGE_CACHE_SIZE_KEY, PAGE_CACHE_SIZE);
            prop.put(CACHE_CLEAR_POURCENT_KEY, CACHE_CLEAR_POURCENT);
            prop.put(OIDTABLE_SIZE_KEY, OIDTABLE_SIZE);
            prop.put(OBJECT_INST_PAGE_SIZE_KEY, OBJECT_INST_PAGE_SIZE);
            prop.put(OBJECT_SYST_PAGE_SIZE_KEY, OBJECT_SYST_PAGE_SIZE);
            prop.put(OBJECT_CLASS_PAGE_SIZE_KEY, OBJECT_CLASS_PAGE_SIZE);
            prop.put(OBJECT_OID_PAGE_SIZE_KEY, OBJECT_OID_PAGE_SIZE);
            prop.put(CONCURRENCY_MODE_KEY, CONCURRENCY_MODE);
            prop.put(TIME_OUT_KEY, TIME_OUT);
            prop.put(ADD_SPACE_KEY, ADD_SPACE);
            prop.put(INTERNAL_KEY_MAX_SIZE_KEY, INTERNAL_KEY_MAX_SIZE);
            prop.put(ALLOWS_SPECIAL_FUNCTIONNALITIES_KEY, ALLOWS_SPECIAL_FUNCTIONNALITIES);
            prop.put(PHYSICAL_ACCESS_CLASS_KEY, PHYSICAL_ACCESS_CLASS);
            prop.put(INTERNAL_FACTORY_CLASS_KEY, INTERNAL_FACTORY_CLASS);
            prop.put(COMMUNICATION_FACTORY_CLASS_KEY, COMMUNICATION_FACTORY_CLASS);
            prop.put(QUERY_MANAGER_CLASS_KEY, QUERY_MANAGER_CLASS);
            prop.put(MBEAN_SERVER_CLASS_KEY, MBEAN_SERVER_CLASS);
            prop.put(MBEAN_JMX_SERVER_PORT_KEY, MBEAN_JMX_SERVER_PORT);
            prop.put(KEEP_IN_MEMORY_KEY, KEEP_IN_MEMORY);
            prop.put(CLEAN_LOG_KEY, CLEAN_LOG);
            prop.put(LOG_FILE_KEY, "");
            prop.put(TRACE_KEY, TRACE);

            File propfile;
            boolean isLoaded;
            if (!databasePropertiesFilePath.equalsIgnoreCase("")) {
                propfile = JalistoUtils.loadPropertiesFileIn(databasePropertiesFilePath, prop);
                parent = propfile.getAbsoluteFile().getParentFile();
                isLoaded = true;
            } else {
                parent = new File(System.getProperty("user.dir"));
                System.out.println("Use default properties values with execution path : " + parent.getAbsolutePath());
                isLoaded = true;
            }

            if (!isLoaded) {
                parent = new File(System.getProperty("user.dir"));
                System.out.println("Exception during reading of Jalisto properties. Uses only default values");
                System.out.println("Work in system user.dir directory : " + String.valueOf(parent));
            }

            JalistoUtils.truncateProperties(prop);
            loadDataFiles();
        } catch (Exception e) {
            throw new JalistoException(
                    "Error during initialisation of JalistoProperties, using " + databasePropertiesFilePath, e);
        }
    }

    public String getName() {
        return getProperty(NAME_KEY);
    }

    public int getInitialSize() {
        return Integer.parseInt(getProperty(DATABASE_INITIAL_SIZE_KEY));
    }

    public int getOidPageSize() {
        return Integer.parseInt(getProperty(OBJECT_OID_PAGE_SIZE_KEY));
    }

    public int getOidTableSize() {
        return Integer.parseInt(getProperty(OIDTABLE_SIZE_KEY));
    }

    public int getAdditionalSpace() {
        return Integer.parseInt(getProperty(ADD_SPACE_KEY));
    }

    public int getObjectCacheSize() {
        return Integer.parseInt(getProperty(OBJECT_CACHE_SIZE_KEY));
    }

    public int getPageCacheSize() {
        return Integer.parseInt(getProperty(PAGE_CACHE_SIZE_KEY));
    }

    public int getMBeanJmxPort() {
        return Integer.parseInt(getProperty(MBEAN_JMX_SERVER_PORT_KEY));
    }

    public double getCacheClearPourcent() {
        return Double.parseDouble("0." + getProperty(CACHE_CLEAR_POURCENT_KEY));
    }

    public short getClassPageSize() {
        return Short.parseShort(getProperty(OBJECT_CLASS_PAGE_SIZE_KEY));
    }

    public short getSystemPageSize() {
        return Short.parseShort(getProperty(OBJECT_SYST_PAGE_SIZE_KEY));
    }

    public String getImplementation() {
        return getProperty(IMPLEMENTATION_KEY);
    }

    public String getInternalFactoryClass() {
        return getProperty(INTERNAL_FACTORY_CLASS_KEY);
    }

    public boolean isMonoImplementation() {
        return getProperty(IMPLEMENTATION_KEY).equalsIgnoreCase(MONO_IMPLEMENTATION);
    }

    public boolean isReadOnlyImplementation() {
        return getProperty(IMPLEMENTATION_KEY).equalsIgnoreCase(READONLY_IMPLEMENTATION);
    }

    public boolean isMultiImplementation() {
        return getProperty(IMPLEMENTATION_KEY).equalsIgnoreCase(MULTI_IMPLEMENTATION);
    }

    public boolean isKeepingInMemory() {
        return getProperty(KEEP_IN_MEMORY_KEY).equalsIgnoreCase("yes");
    }

    public boolean isCleanLog() {
        return getProperty(CLEAN_LOG_KEY).equalsIgnoreCase("yes");
    }

    public boolean allowsSpecialFunctionnalities() {
        return getProperty(ALLOWS_SPECIAL_FUNCTIONNALITIES_KEY).equalsIgnoreCase("yes");
    }

    public short getInstancePageSize() {
        return Short.parseShort(getProperty(OBJECT_INST_PAGE_SIZE_KEY));
    }

    public int getKeyLength() {
        return Integer.parseInt(getProperty(INTERNAL_KEY_MAX_SIZE_KEY));
    }

    public String getCommunicationFactoryClassName() {
        return getProperty(COMMUNICATION_FACTORY_CLASS_KEY);
    }

    public Collection getKeys() {
        return prop.keySet();
    }

    public String getLogFileName() {
        String logFileName = prop.getProperty(LOG_FILE_KEY);
        if (logFileName.equals("")) {
            logFileName = getName() + "-" + LOG_FILE;
        }
        return logFileName;
    }

    public String getProperty(String key) {
        return prop.getProperty(key);
    }

    public void setProperty(String key, String property) {
        prop.setProperty(key, property);
    }

    public Collection getTraceModuleNames() {
        return getStrings(TRACE_KEY, ",");
    }

    public boolean isTraceEnable() {
        String modules = getProperty(TRACE_KEY);
        return ((modules != null) && (!modules.equals("")));
    }

    public JalistoProperties getClone() {
        return new JalistoPropertiesImpl(propertiesPath);
    }

    public void compareProperties(JalistoProperties oldProps) {
        if (oldProps == null) {
            return;
        }
        checkProperty(oldProps, NAME_KEY);
        checkProperty(oldProps, OBJECT_INST_PAGE_SIZE_KEY);
        checkProperty(oldProps, OBJECT_SYST_PAGE_SIZE_KEY);
        checkProperty(oldProps, OBJECT_CLASS_PAGE_SIZE_KEY);
        checkProperty(oldProps, OBJECT_OID_PAGE_SIZE_KEY);
        checkProperty(oldProps, INTERNAL_KEY_MAX_SIZE_KEY);
        checkProperty(oldProps, PHYSICAL_ACCESS_CLASS_KEY);
    }

    public void checkProperty(JalistoProperties oldProps, String property) {
        if (!oldProps.getProperty(property).equalsIgnoreCase(prop.getProperty(property))) {
            throw new JalistoException("Old Jalisto configuration property '" + property + "' with value '" +
                                           oldProps.getProperty(property) + "'is not equal to new value '" +
                                           prop.getProperty(property) + "'");
        }
    }

    /**
     * ********************************    PATHS    ********************************************
     */

    public void loadDataFiles() throws IOException {
        String files = getProperty(DATABASE_FILES_PATHS_KEY);
        if (files.equals("")) {
            files = getName() + ".jalisto";
        }
        StringTokenizer tokenizer = new StringTokenizer(files, ",;");
        int size = tokenizer.countTokens();
        if (size == 2) {
            throw new JalistoException("Not a valid number of path in property '" + DATABASE_FILES_PATHS_KEY + "'");
        }
        pathes = new String[size];
        int i = 0;
        while (tokenizer.hasMoreTokens()) {
            String path = tokenizer.nextToken();
            File jalistoFile = new File(path);

            if (jalistoFile.isAbsolute()) {
                pathes[i] = jalistoFile.getCanonicalPath();
            } else {
                File f = new File(parent, path);
                pathes[i] = f.getCanonicalPath();
            }
            i++;
        }
    }

    public String getPropertiesPath() {
        return propertiesPath;
    }

    public String[] getPathes() {
        return pathes;
    }

    public String getDbFileFullName() {
        return pathes[0];
    }

    public String getOidDbFilePath() {
        if (pathes.length > 1) {
            return pathes[OID_INDEX];
        } else {
            return pathes[0];
        }
    }

    public String getSystemDbFilePath() {
        if (pathes.length > 1) {
            return pathes[SYSTEM_INDEX];
        } else {
            return pathes[0];
        }
    }

    public int getInstanceDbFileNumber() {
        if (pathes.length > 1) {
            return (pathes.length - 2);
        } else {
            return 1;
        }
    }

    public String getInstanceDbFilePathAt(int index) {
        if (pathes.length > 1) {
            return pathes[index + 2];
        } else {
            return pathes[0];
        }
    }

    /*******************************  FROM INTERFACE   ************************************/

    public String getHost() {
        throw new UnsupportedOperationException();
    }

    public int getPort() {
        throw new UnsupportedOperationException();
    }

    public String getServerPropertiesPath() {
        throw new UnsupportedOperationException();
    }

    /**
     * *****************************     UTIL    ***************************************
     */

    private Collection getStrings(Object key, String separator) {
        String value = prop.getProperty((String) key);
        List result = new ArrayList();
        if (value != null) {
            StringTokenizer tokenizer = new StringTokenizer(value, separator);
            while (tokenizer.hasMoreTokens()) {
                result.add(tokenizer.nextToken().trim());
            }
        }
        return result;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("JalistoProperties[");
        Iterator keys = prop.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            sb.append(key).append("=").append(prop.getProperty(key));
            if (keys.hasNext()) {
                sb.append(",");
            }
        }
        sb.append("]");
        return sb.toString();
    }
}
