/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.cache;

import org.objectweb.jalisto.se.api.cache.JalistoCache;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.io.PrintStream;
import java.util.*;

public class GenericCache implements JalistoCache {

    public GenericCache() {
        table = new HashMap();
    }

    public void init(int size, String name, double clearPourcent) {
        this.maxSize = size;
        this.name = name;
        this.cacheClearPourcent = clearPourcent;
    }

    public void setTrace(Trace trace) {
        this.trace = trace;
    }

    public boolean isEmpty() {
        return table.isEmpty();
    }

    public void clear() {
        trace.println(Trace.CACHE, "{0} : clear()", name);
        table.clear();
        firstCell = null;
        lastCell = null;
    }

    public boolean containsKey(Object key) {
        return table.containsKey(key);
    }

    public boolean containsValue(Object value) {
        throw new UnsupportedOperationException("not used in this implementation");
    }

    public Set entrySet() {
        throw new UnsupportedOperationException("not used in this implementation");
    }

    public Object get(Object key) {
        trace.println(Trace.CACHE, "{0} : get({1})", name, key);
        Object result = table.get(key);
        if (result != null) {
            return ((CacheCell) result).getObject();
        }
        return null;
    }

    public Set keySet() {
        return table.keySet();
    }

    public Object put(Object key, Object value) {
        trace.println(Trace.CACHE, "{0} : put({1}, {2})", name, key, value);
        if (containsKey(key)) {
            ((CacheCell) table.get(key)).setObject(value);
        } else {
            trace.println(Trace.CACHE, "{0} : put -> add");
            if (table.size() >= maxSize) {
                makeSpace();
            }
            CacheCell availableCacheCell = CacheCell.newInstance(key, value);
            if (lastCell != null) {
                lastCell.setNextCell(availableCacheCell);
                availableCacheCell.setPrecedingCell(lastCell);
                lastCell = availableCacheCell;
            } else {
                lastCell = availableCacheCell;
                firstCell = availableCacheCell;
            }
            table.put(key, availableCacheCell);
        }
        return null;
    }

    public void putAll(Map map) {
        Iterator keys = map.keySet().iterator();
        while (keys.hasNext()) {
            Object key = keys.next();
            put(key, map.get(key));
        }
    }

    public Object remove(Object key) {
        trace.println(Trace.CACHE, "{0} : remove({1})", name, key);
        if (containsKey(key)) {
            synchronized (table) {
                CacheCell cell = (CacheCell) table.remove(key);
                if (cell.equals(firstCell)) {
                    firstCell = cell.getNextCell();
                }
                if (cell.equals(lastCell)) {
                    lastCell = cell.getPrecedingCell();
                }
                Object result = cell.getObject();
                cell.delink();
                if (table.isEmpty()) {
                    lastCell = null;
                    firstCell = null;
                }
                return result;
            }
        }
        return null;
    }

    public int size() {
        return table.size();
    }

    public synchronized void setMaxSize(int size) {
        maxSize = size;
        CacheCell myFirst = firstCell;
        synchronized (table) {
            while (table.size() > maxSize) {
                CacheCell mySecond = myFirst.getNextCell();
                table.remove(myFirst.getId());
                myFirst.delink();
                myFirst = mySecond;
            }
            if (table.isEmpty()) {
                lastCell = null;
                firstCell = null;
            }
            firstCell = myFirst;
        }
    }

    public int getMaxSize() {
        return maxSize;
    }

    public Collection values() {
        throw new UnsupportedOperationException("not used in this implementation");
    }


    private void makeSpace() {
        int nbrToRemove = (new Double(maxSize * cacheClearPourcent)).intValue();
        trace.println(Trace.CACHE, "{0} : makeSpace() : clear {1} objects", name, new Integer(nbrToRemove));

        CacheCell myFirst = firstCell;
        synchronized (table) {
            for (int i = 0; i < nbrToRemove; i++) {
                CacheCell mySecond = myFirst.getNextCell();
                if (mySecond == null) {
                    throw new NullPointerException();
                }
                table.remove(myFirst.getId());
                myFirst.delink();
                myFirst = mySecond;
            }
        }
        firstCell = myFirst;
    }

    public void print(PrintStream out) {
        out.println(table.toString());
    }

    public String toString() {
        return table.toString();
    }


    private CacheCell firstCell;
    private CacheCell lastCell;
    private HashMap table;
    private String name;
    private double cacheClearPourcent;
    private int maxSize;
    private Trace trace;
}
