/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.client;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.internal.multi.LockManager;
import org.objectweb.jalisto.se.api.query.QueryManager;
import org.objectweb.jalisto.se.api.remote.ClientCommunicationAgent;
import org.objectweb.jalisto.se.exception.*;
import org.objectweb.jalisto.se.impl.ExtentImpl;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.TransactionImpl;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.util.*;

public class SessionClientImpl implements SessionInternal {

    public SessionClientImpl(JalistoProperties properties) {
        this.clientProperties = properties;
        this.connexion = JalistoFactory.getInternalFactory().getClientCommunicationAgent(properties);
        this.sessionId = connexion.getSessionId();
        this.trace = JalistoFactory.getInternalFactory().getTracer(properties);
        this.cache = JalistoFactory.getInternalFactory().getCache(properties, properties.getObjectCacheSize(), "object cache");
        this.repository = JalistoFactory.getInternalFactory().getMetaRepository(properties, connexion);
        this.transaction = createTransaction();
        this.createdObjects = new HashMap();
        this.updatedObjects = new HashMap();
        this.deletedObjects = new HashMap();

        ((JalistoPropertiesClientImpl) clientProperties).setRemoteProperties(connexion.getProperties());
    }

    public void defineClass(ClassDescription classDescription) {
        trace.println(Trace.SESSION, "{0} : defineClass({1})", sessionId, classDescription);
        checkIsOpen("defineClass");
        checkValidity("defineClass", false);
        if (!isClassDefined(classDescription.getClassName())) {
            connexion.defineClass(classDescription);
            repository.defineClass(this, classDescription);
            Short clid = (Short) repository.getClidFromClassName(classDescription.getClassName());
            identityProvider.addClass(clid.shortValue());
        }
    }

    public void removeClass(String fullClassName) {
        trace.println(Trace.SESSION, "{0} : removeClass({1})", sessionId, fullClassName);
        checkIsOpen("removeClass");
        checkValidity("removeClass", false);
        if (isClassDefined(fullClassName)) {
            repository.removeClass(this, fullClassName);
            connexion.removeClass(fullClassName);
        } else {
            throw new SchemaException("Class " + fullClassName + " is not define in base");
        }
    }

    public boolean isClassDefined(String fullQualifiedClassName) {
        checkIsOpen("isClassDefined");
        try {
            return (repository.getClidFromClassName(fullQualifiedClassName) != null);
        } catch (SchemaException e) {
        }
        return false;
    }

    public String getClassNameFor(Object oid) {
        trace.println(Trace.SESSION, "{0} : getClassNameFor({1})", sessionId, oid);
        checkIsOpen("getClassNameFor");
        LogicalOid floid = getFloidFromOid(oid);
        return repository.getClassNameFromClid(new Short(floid.getClid()));
    }

    public Object makeNewFileOid(Class objectClass) {
        return makeNewFileOid(objectClass.getName());
    }

    public Object makeNewFileOid(LogicalOid floid) {
        throw new UnsupportedOperationException();
    }

    public Long reserveFloids(short clid, int number) {
        throw new UnsupportedOperationException();
    }

    public Object makeNewFileOid(String objectClassName) {
        trace.println(Trace.SESSION, "makeNewFileOid");
        checkValidity("makeNewFileOid", true);
        return identityProvider.makeNewFileOid(objectClassName);
    }

    public Object createObject(Object[] objectToCreate, Class objectClass) {
        return createObject(objectToCreate, objectClass.getName());
    }

    public Object createObject(Object[] objectToCreate, String objectClassName) {
        return createObject(makeNewFileOid(objectClassName), objectToCreate);
    }

    public Object createObject(Object oid, Object[] objectToCreate) {
        checkValidity("createObject", true);
        identityProvider.useFloid(oid);
        createdObjects.put(oid, objectToCreate);
        return oid;
    }

    public Object[] readObjectByOid(Object oid) {
        return readObjectByOid(oid, true);
    }

    public Collection readObjectsByOids(Collection oids) {
        trace.println(Trace.SESSION, "{0} : readObjectsByOids({1})", sessionId, oids);
        checkValidity("readObjectsByOids", true);
        HashMap readed = new HashMap();
        ArrayList toRead = new ArrayList();
        Iterator oidsIterator = oids.iterator();
        while (oidsIterator.hasNext()) {
            Object oid = oidsIterator.next();
            Object result = cache.get(oid);
            if (result == null) {
                result = updatedObjects.get(oid);
                if (result == null) {
                    result = createdObjects.get(oid);
                }
            }
            if (result == null) {
                toRead.add(oid);
            } else {
                readed.put(oid, result);
            }
        }
        Iterator fromServer = connexion.readObjectsByOids(oids).iterator();
        oidsIterator = oids.iterator();
        while (fromServer.hasNext() && oidsIterator.hasNext()) {
            readed.put(oidsIterator.next(), fromServer.next());
        }
        ArrayList result = new ArrayList();
        oidsIterator = oids.iterator();
        while (oidsIterator.hasNext()) {
            result.add(readed.get(oidsIterator.next()));
        }
        return result;
    }

    public boolean contains(Object oid) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    public Object updateObjectByOid(Object oid, Object[] objectToUpdate) {
        trace.println(Trace.SESSION, "{0} : readObjectByOid({1}, {2})", sessionId, oid, objectToUpdate);
        checkValidity("updateObjectByOid", true);
        if (deletedObjects.containsKey(oid)) {
            throw new IdentityException("this object has been deleted during the transaction");
        }
        cache.remove(oid);
        identityProvider.useFloid(oid);
        if (createdObjects.containsKey(oid)) {
            createdObjects.put(oid, objectToUpdate);
            return oid;
        }
        updatedObjects.put(oid, objectToUpdate);
        return oid;
    }

    public void deleteObjectByOid(Object oid) {
        trace.println(Trace.SESSION, "{0} : deleteObjectByOid({1})", sessionId, oid);
        checkValidity("deleteObjectByOid", true);
        if (deletedObjects.containsKey(oid)) {
            throw new IdentityException("this object has already been deleted during the transaction");
        }
        cache.remove(oid);
        identityProvider.useFloid(oid);
        createdObjects.remove(oid);
        updatedObjects.remove(oid);
        deletedObjects.put(oid, null);
    }

    public Extent getExtent(String fullClassName) {
        trace.println(Trace.SESSION, "getExtent(String {0})", fullClassName);
        checkValidity("getExtent", true);
        return new ExtentImpl(repository.getClidFromClassName(fullClassName), this);
    }

    public Extent getExtent(Class theClass) {
        String fullClassName = theClass.getName();
        return getExtent(fullClassName);
    }

    public boolean isNewBase() {
        return false;
    }

    public void reorganize() {
        if (clientProperties.allowsSpecialFunctionnalities()) {
            trace.println(Trace.SESSION, "{0} : reorganize()", sessionId);
            if (isOpen) {
                throw new TransactionException("Session must not be open during reorganize()");
            }
            if (transaction.isActive()) {
                throw new JalistoException("Transaction must not be active during reorganize()");
            }
            connexion.reorganize();
        }
    }

    public QueryManager getQueryManager() {
        return null;
    }

    public Transaction currentTransaction() {
        return transaction;
    }

    public MetaRepository getMetaRepository() {
        checkIsOpen("getMetaRepository");
        return repository;
    }

    public SessionInternal getInternalSession() {
        return this;
    }

    public void openSession() {
        if (isOpen) {
            throw new TransactionException("Session is already open");
        }
        connexion.openSession();
        ((InternalMetaRepositoryClientImpl)repository).init();
        identityProvider = new RemoteIdentityProvider(connexion, repository);
        isOpen = true;
    }

    public void closeSession() {
        if (!isOpen) {
            throw new TransactionException("Session is not open");
        }
        connexion.closeSession();
        isOpen = false;
    }

    public boolean isOpen() {
        return isOpen;
    }

    public void eraseStorage() {
        if (clientProperties.allowsSpecialFunctionnalities()) {
            trace.println(Trace.SESSION, "{0} : eraseStorage()", sessionId);
            if (isOpen) {
                throw new TransactionException("Session must not be open during eraseStorage()");
            }
            if (transaction.isActive()) {
                throw new JalistoException("Transaction must not be active during eraseStorage()");
            }
            connexion.eraseStorage();
        }
    }


    /**
     * *****************************    INTERNAL     ***************************************************
     */

    public void checkValidity(String message, boolean mustBeActive) {
        checkIsOpen(message);
        if (mustBeActive) {
            if (!transaction.isActive()) {
                throw new TransactionException("Transaction must be active during " + message);
            }
        } else {
            if (transaction.isActive()) {
                throw new TransactionException("Transaction must not be active during " + message);
            }
        }
    }

    public boolean isRemoteSession() {
        return true;
    }

    public void begin() {
        trace.println(Trace.SESSION, "{0} : begin()", sessionId);
        checkIsOpen("begin");
        connexion.begin();
    }

    public void clearObjectCache() {
        trace.println(Trace.SESSION, "{0} : clearObjectCache()", sessionId);
        checkIsOpen("clearObjectCache");
        cache.clear();
    }

    public void commit() {
        trace.println(Trace.SESSION, "{0} : commit()", sessionId);
        checkIsOpen("commit");
        identityProvider.commit();
        if (!createdObjects.isEmpty()) {
            connexion.createObjects(createdObjects);
            createdObjects.clear();
        }
        if (!updatedObjects.isEmpty()) {
            connexion.updateObjects(updatedObjects);
            updatedObjects.clear();
        }
        if (!deletedObjects.isEmpty()) {
            connexion.deleteObjects(deletedObjects.keySet());
            deletedObjects.clear();
        }
        connexion.commit();
    }

    public Collection getAllClassNames() {
        return repository.getAllClassNames();
    }

    public LogicalSystemPageAccess getFileAccess() {
        return null;
    }

    public LockManager getLockManager() {
        return null;
    }

    public OidTable getOidTable() {
        return null;
    }

    public JalistoProperties getProperties() {
        return clientProperties;
    }

    public JalistoProperties getClientProperties() {
        return clientProperties;
    }

    public Object getSessionId() {
        return sessionId;
    }

    public Object[] readObjectByOid(Object oid, boolean withCache) {
        trace.println(Trace.SESSION, "{0} : readObjectByOid({1})", sessionId, oid);
        checkValidity("readObjectByOid", true);
        if (deletedObjects.containsKey(oid)) {
            throw new IdentityException("this object has been deleted during the transaction");
        }
        Object result = createdObjects.get(oid);
        if (result != null) {
            return (Object[]) result;
        }
        result = updatedObjects.get(oid);
        if (result != null) {
            return (Object[]) result;
        }
        result = cache.get(oid);
        if (result != null) {
            return (Object[]) result;
        }
        result = connexion.readObjectByOid(oid, withCache);
        cache.put(oid, result);
        return (Object[]) result;
    }

    public Object[] refreshObjectByOid(Object oid) {
        trace.println(Trace.SESSION, "{0} : refreshObjectByOid({1})", sessionId, oid);
        checkValidity("refreshObjectByOid", true);
        if (deletedObjects.containsKey(oid)) {
            throw new IdentityException("this object has been deleted during the transaction");
        }
        Object[] result = connexion.refreshObjectByOid(oid);
        if (result != null) {
            if (cache.containsKey(oid)) {
                cache.put(oid, result);
                return result;
            }
            if (createdObjects.containsKey(oid)) {
                createdObjects.put(oid, result);
                return result;
            }
            if (updatedObjects.containsKey(oid)) {
                updatedObjects.put(oid, result);
                return result;
            }
        }
        return result;
    }

    public void rollback() {
        trace.println(Trace.SESSION, "{0} : rollback()", sessionId);
        checkIsOpen("rollback");
        identityProvider.rollback();
        createdObjects.clear();
        updatedObjects.clear();
        deletedObjects.clear();
        connexion.rollback();
    }

    public void setOptimistic() {
    }

    public void setPessimistic() {
    }

    public Collection getFloidsFromClid(Object sessionId, Object clid) {
        short c = ((Short)clid).shortValue();
        ArrayList floids = new ArrayList(connexion.getFloidsFromClid(sessionId, clid));
        identityProvider.addFloidForClid(floids, c);
        Iterator createdFloids = createdObjects.keySet().iterator();
        while(createdFloids.hasNext()) {
            LogicalOid floid = (LogicalOid) createdFloids.next();
            if (floid.getClid() == c) {
                floids.add(floid);
            }
        }
        floids.removeAll(deletedObjects.keySet());
        return floids;
    }

    /**
     * **************************************    PRIVATE     ***********************************************
     */


    private Transaction createTransaction() {
        return new TransactionImpl(this);
    }

    private void checkIsOpen(String message) {
        if (!isOpen) {
            throw new TransactionException("Session must be open during " + message);
        }
    }

    protected LogicalOid getFloidFromOid(Object oid) {
        try {
            return (LogicalOid) oid;
        } catch (ClassCastException cce) {
            throw new IdentityException("oid is not a FileLoid but a " + oid.getClass().getName());
        }
    }


    private Transaction transaction;
    private boolean isOpen;

    private JalistoProperties clientProperties;
    private Object sessionId;
    private Map cache;
    private HashMap createdObjects;
    private HashMap updatedObjects;
    private HashMap deletedObjects;
    private InternalMetaRepository repository;
    private RemoteIdentityProvider identityProvider;
    private ClientCommunicationAgent connexion;
    private Trace trace;
}
