/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.client.socket;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.LogicalSystemPageAccess;
import org.objectweb.jalisto.se.api.internal.OidTable;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.internal.multi.LockManager;
import org.objectweb.jalisto.se.api.query.QueryManager;
import org.objectweb.jalisto.se.api.remote.ClientCommunicationAgent;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.exception.remote.RemoteException;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.remote.socket.MethodEncoder;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.*;

public class ClientCommunicationAgentSocketImpl implements ClientCommunicationAgent {

    public ClientCommunicationAgentSocketImpl(JalistoProperties properties) {
        this.trace = JalistoFactory.getInternalFactory().getTracer(properties);
        this.serverHost = properties.getHost();
        this.serverPort = properties.getPort();
        try {
            connect();
            writeObject(toServer, properties.getServerPropertiesPath());
            flush(toServer);
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    private void waitEndMethod() {
        try {
            Object reply = null;
            while (reply == null) {
                reply = fromServer.readObject();
            }
            if (!reply.equals("ok")) {
                if (reply instanceof RemoteException) {
                    throw new RemoteException((RemoteException)reply);
                }
                throw new JalistoException(String.valueOf(reply));
            }
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        } catch (ClassNotFoundException cnfe) {
            throw new JalistoException(cnfe);
        }
    }

    private Object waitResult() {
        try {
            Object reply = null;
            while (reply == null) {
                reply = fromServer.readObject();
            }
            if (reply instanceof RemoteException) {
                throw new RemoteException((RemoteException)reply);
            }
            return reply;
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        } catch (ClassNotFoundException cnfe) {
            throw new JalistoException(cnfe);
        }
    }

    private void writeInt(ObjectOutputStream toServer, int i) throws IOException {
        toServer.writeInt(i);
    }

    private void writeObject(ObjectOutputStream toServer, Object o) throws IOException {
        toServer.writeObject(o);
    }

    private void flush(ObjectOutputStream toServer) throws IOException {
        toServer.flush();
    }

    private void callRemoteMethod(int code) {
        try {
            writeInt(toServer, code);
            flush(toServer);
            waitEndMethod();
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    private Object callRemoteMethod(int code, Object o, boolean withResult) {
        try {
            writeInt(toServer, code);
            writeObject(toServer, o);
            flush(toServer);
            if (withResult) {
                return waitResult();
            } else {
                waitEndMethod();
                return null;
            }
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    private Object callRemoteMethod(int code, Object o1, Object o2, boolean withResult) {
        try {
            writeInt(toServer, code);
            writeObject(toServer, o1);
            writeObject(toServer, o2);
            flush(toServer);
            if (withResult) {
                return waitResult();
            } else {
                waitEndMethod();
                return null;
            }
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    /**
     * *********************************    SESSION    **************************************************
     */

    public void closeSession() {
        callRemoteMethod(MethodEncoder.closeSession);
//        disconnect();
    }

    public boolean contains(Object oid) {
        throw new UnsupportedOperationException();
    }

    public Object createObject(Object[] objectToCreate, Class objectClass) {
        throw new UnsupportedOperationException();
    }

    public Object createObject(Object[] objectToCreate, String objectClassName) {
        throw new UnsupportedOperationException();
    }

    public Object createObject(Object oid, Object[] objectToCreate) {
        return callRemoteMethod(MethodEncoder.createObjectOid, oid, objectToCreate, true);
    }

    public Transaction currentTransaction() {
        throw new UnsupportedOperationException();
    }

    public void defineClass(ClassDescription classDescription) {
        callRemoteMethod(MethodEncoder.defineClass, classDescription, false);
    }

    public void deleteObjectByOid(Object oid) {
        callRemoteMethod(MethodEncoder.deleteObjectByOid, oid, false);
    }

    public void eraseStorage() {
        callRemoteMethod(MethodEncoder.eraseStorage);
    }

    public String getClassNameFor(Object oid) {
        throw new UnsupportedOperationException();
    }

    public Extent getExtent(String fullClassName) {
        return (Extent) callRemoteMethod(MethodEncoder.createObjectOid, fullClassName, true);
    }

    public Extent getExtent(Class theClass) {
        throw new UnsupportedOperationException();
    }

    public SessionInternal getInternalSession() {
        throw new UnsupportedOperationException();
    }

    public MetaRepository getMetaRepository() {
        throw new UnsupportedOperationException();
    }

    public QueryManager getQueryManager() {
        throw new UnsupportedOperationException();
    }

    public boolean isClassDefined(String fullQualifiedClassName) {
        throw new UnsupportedOperationException();
    }

    public boolean isNewBase() {
        throw new UnsupportedOperationException();
    }

    public boolean isOpen() {
        throw new UnsupportedOperationException();
    }

    public Object makeNewFileOid(Class objectClass) {
        throw new UnsupportedOperationException();
    }

    public Object makeNewFileOid(String objectClassName) {
        return callRemoteMethod(MethodEncoder.makeNewFileOid, objectClassName, true);
    }

    public void openSession() {
        callRemoteMethod(MethodEncoder.openSession);
    }

    public Object[] readObjectByOid(Object oid) {
        throw new UnsupportedOperationException();
    }

    public Object[] refreshObjectByOid(Object oid) {
        return (Object[]) callRemoteMethod(MethodEncoder.refreshObjectByOid, oid, true);
    }

    public Collection readObjectsByOids(Collection oids) {
        return (Collection) callRemoteMethod(MethodEncoder.readObjectsByOids, oids, true);
    }

    public void removeClass(String fullClassName) {
        callRemoteMethod(MethodEncoder.removeClass, fullClassName, false);
    }

    public void reorganize() {
        callRemoteMethod(MethodEncoder.reorganize);
    }

    public Object updateObjectByOid(Object oid, Object[] objectToUpdate) {
        return callRemoteMethod(MethodEncoder.updateObjectByOid, oid, objectToUpdate, true);
    }


    /**
     * **********************************    INTERNAL    ********************************************
     */

    public void begin() {
        callRemoteMethod(MethodEncoder.begin);
    }

    public void checkValidity(String message, boolean mustBeActive) {
        throw new UnsupportedOperationException();
    }

    public boolean isRemoteSession() {
        throw new UnsupportedOperationException();
    }

    public void clearObjectCache() {
        throw new UnsupportedOperationException();
    }

    public Object makeNewFileOid(LogicalOid floid) {
        throw new UnsupportedOperationException();
    }

    public void commit() {
        callRemoteMethod(MethodEncoder.commit);
    }

    public Collection getAllClassNames() {
        throw new UnsupportedOperationException();
    }

    public LogicalSystemPageAccess getFileAccess() {
        throw new UnsupportedOperationException();
    }

    public LockManager getLockManager() {
        throw new UnsupportedOperationException();
    }

    public OidTable getOidTable() {
        throw new UnsupportedOperationException();
    }

    public JalistoProperties getProperties() {
        try {
            writeInt(toServer, MethodEncoder.getProperties);
            flush(toServer);
            return (JalistoProperties) waitResult();
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    public Object getSessionId() {
        try {
            writeInt(toServer, MethodEncoder.getSessionId);
            flush(toServer);
            return waitResult();
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    public Object[] readObjectByOid(Object oid, boolean withCache) {
        return (Object[]) callRemoteMethod(MethodEncoder.readObjectByOid, oid,
                                           new Boolean(withCache), true);
    }

    public void rollback() {
        callRemoteMethod(MethodEncoder.rollback);
    }

    public void setOptimistic() {
        throw new UnsupportedOperationException();
    }

    public void setPessimistic() {
        throw new UnsupportedOperationException();
    }

    /**
     * ***********************************      OTHER       ************************************
     */

    public Map getClassMetas() {
        try {
            writeInt(toServer, MethodEncoder.getClassMetas);
            flush(toServer);
            return (Map) waitResult();
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    public Map getClassTable() {
        try {
            writeInt(toServer, MethodEncoder.getClassTable);
            flush(toServer);
            return (Map) waitResult();
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    public Object getClidFromClassName(String className) {
        return callRemoteMethod(MethodEncoder.getClidFromClassName, className, true);
    }

    public void makeOids(Collection oids) {
        callRemoteMethod(MethodEncoder.makeOids, new ArrayList(oids), false);
    }

    public void createObjects(Map mapping) {
        callRemoteMethod(MethodEncoder.createObjects, new HashMap(mapping), false);
    }

    public void updateObjects(Map mapping) {
        callRemoteMethod(MethodEncoder.updateObjects, new HashMap(mapping), false);
    }

    public void deleteObjects(Collection oids) {
        callRemoteMethod(MethodEncoder.deleteObjects, new ArrayList(oids), false);
    }

    public Collection getFloidsFromClid(Object sessionId, Object clid) {
        return (Collection) callRemoteMethod(MethodEncoder.getFloidsFromClid, sessionId, clid, true);
    }

    public Long reserveFloids(short clid, int number) {
        return (Long) callRemoteMethod(MethodEncoder.reserveFloids,
                                       new Short(clid), new Integer(number), true);
    }

    /**
     * **********************************     SOCKET    ******************************************
     */

    public void connect() throws IOException {
        trace.println(Trace.REMOTE, "try connecting on {0} {1}", serverHost, new Integer(serverPort));
        socket = new Socket(serverHost, serverPort);
        toServer = new ObjectOutputStream(socket.getOutputStream());
        try {
            Thread.sleep(100);
        } catch (InterruptedException e) {
        }
        fromServer = new ObjectInputStream(socket.getInputStream());
//        toServer.close();
    }

    public void closeConnection() {
        disconnect();
    }

    public void disconnect() {
        trace.println(Trace.REMOTE, "disconnect fron server");
        try {
            toServer.close();
            fromServer.close();
            socket.close();
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }


    private String serverHost;
    private int serverPort;

    private Socket socket;
    private ObjectInputStream fromServer;
    private ObjectOutputStream toServer;

    private Trace trace;
}
