/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.factory;

import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.internal.multi.LockTable;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.remote.ClientCommunicationAgent;
import org.objectweb.jalisto.se.api.remote.JalistoServer;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.api.cache.JalistoCache;
import org.objectweb.jalisto.se.api.jmx.JalistoMBeanServer;
import org.objectweb.jalisto.se.impl.lock.AccessController;
import org.objectweb.jalisto.se.impl.server.*;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.impl.trace.TraceContext;
import org.objectweb.jalisto.se.impl.meta.InternalMetaRepositoryImpl;
import org.objectweb.jalisto.se.impl.cache.GenericCacheImplEmpty;
import org.objectweb.jalisto.se.impl.JalistoPropertiesImpl;
import org.objectweb.jalisto.se.exception.JalistoException;

import java.util.Hashtable;
import java.util.Map;
import java.util.HashMap;

public class InternalFactoryImpl implements InternalFactory {

    protected Map jalistoProperties;
    protected Map logicals;
    protected Map metaRepositories;
    protected Map oidProviders;
    protected Map oidTables;
    protected Map physicals;
    protected Map sessionIdMap;
    protected Map tracers;

    protected JalistoMBeanServer mbeanServer;

    protected static InternalFactory instance;



    public void setSelfInstance() {
        if (InternalFactoryImpl.instance == null) {
            InternalFactoryImpl.instance = this;
        }
    }

    public void init() {
        jalistoProperties = new Hashtable();
        logicals = new Hashtable();
        metaRepositories = new Hashtable();
        oidProviders = new Hashtable();
        oidTables = new Hashtable();
        physicals = new Hashtable();
        sessionIdMap = new Hashtable();
        tracers = new Hashtable();
    }

    public void cleanFactory() {
        System.out.println("CLEANING FACTORY");
        jalistoProperties.clear();
        logicals.clear();
        metaRepositories.clear();
        oidProviders.clear();
        oidTables.clear();
        physicals.clear();
        sessionIdMap.clear();
        tracers.clear();

        System.gc();
    }

    public synchronized JalistoProperties getProperties(String path) {
        if (!jalistoProperties.containsKey(path)) {
            JalistoProperties p = new JalistoPropertiesImpl(path);
            jalistoProperties.put(path, p);
        }
        return (JalistoProperties) jalistoProperties.get(path);
    }

    public synchronized Object getSessionId(JalistoProperties properties, Session session) {
        String name = properties.getName();
        if (!sessionIdMap.containsKey(name)) {
            Object id = new Integer(0);
            sessionIdMap.put(name, id);
            return name + id;
        }
        Integer oldId = (Integer) sessionIdMap.get(name);
        Integer newId = new Integer(oldId.intValue() + 1);
        sessionIdMap.put(name, newId);
        return name + newId;
    }

    public synchronized InternalMetaRepository getMetaRepository(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!metaRepositories.containsKey(path)) {
            InternalMetaRepositoryImpl metaRepository = InternalMetaRepositoryImpl.getAnMetaRepository(properties);
            metaRepositories.put(path, metaRepository);
        }
        InternalMetaRepositoryImpl result = (InternalMetaRepositoryImpl) metaRepositories.get(path);
        return result;
    }

    public synchronized IdentityProvider getIdentityProvider(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!oidProviders.containsKey(path)) {
            InternalPhysicalFileAccess physicalAccess = getInternalPhysicalAccess(properties);
            IdentityProvider oidProvider = IdentityProvider.getAnIdentityProvider(physicalAccess);
            oidProviders.put(path, oidProvider);
        }
        return (IdentityProvider) oidProviders.get(path);
    }

    public PluggablePhysicalFileAccess getPhysicalAccess(JalistoProperties properties) {
        PluggablePhysicalFileAccess internalAccess;
        String className = properties.getProperty(JalistoProperties.PHYSICAL_ACCESS_CLASS_KEY);
        try {
            internalAccess = (PluggablePhysicalFileAccess) Class.forName(className).newInstance();
        } catch (InstantiationException e) {
            throw new JalistoException("cannot instanciate internal access", e);
        } catch (IllegalAccessException e) {
            throw new JalistoException("cannot initialize internal access", e);
        } catch (ClassNotFoundException e) {
            throw new JalistoException("cannot find internal access class", e);
        }
        internalAccess.init(properties);
        return internalAccess;
    }

    public synchronized Trace getTracer(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!tracers.containsKey(path)) {
            TraceContext traceContext = new TraceContext(properties.isTraceEnable(), properties.getTraceModuleNames());
            Trace trace = traceContext.getTracer();
            tracers.put(path, trace);
        }
        return (Trace) tracers.get(path);
    }

    public synchronized Session getSession(String propertiesFilePath) {
        JalistoProperties properties = getProperties(propertiesFilePath);
        return getSession(properties);
    }

    public synchronized InTransactionBaseImage getAInTransactionBaseImage(
                PluggablePhysicalFileAccess physicalAccess, JalistoProperties properties) {
        return InTransactionBaseImageImpl.getInTransactionBaseImageInstance(physicalAccess, properties);
    }

    public synchronized LogicalSystemPageAccess getLogicalAccess(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!logicals.containsKey(path)) {
            logicals.put(path, new LogicalPageAccessImpl(properties));
        }
        return (LogicalSystemPageAccess) logicals.get(path);
    }

    public DataWrapper getNewDataWrapper(JalistoProperties properties, Object[] arrayOfObjects) {
        return DataWrapperImpl.newInstance(arrayOfObjects);
    }

    public synchronized Map getCache(JalistoProperties properties, int size, String name) {
        if (size > 0) {
            String className = properties.getProperty(JalistoProperties.CACHE_IMPLEMENTATION_KEY);
            try {
                JalistoCache cache = (JalistoCache) Class.forName(className).newInstance();
                cache.init(size, name, properties.getCacheClearPourcent());
                cache.setTrace(getTracer(properties));
                return cache;
            } catch (InstantiationException e) {
                throw new JalistoException("cannot instanciate cache", e);
            } catch (IllegalAccessException e) {
                throw new JalistoException("cannot initialize cache", e);
            } catch (ClassNotFoundException e) {
                throw new JalistoException("cannot find internal cache", e);
            }
        } else if (size == 0) {
            return new GenericCacheImplEmpty();
        } else {
            return new HashMap();
        }
    }

    public void launchMBeanHtmlServer(JalistoProperties props) {
        if ((mbeanServer == null) || (!mbeanServer.isAlive())) {
            try {
                String className = props.getProperty(JalistoProperties.MBEAN_SERVER_CLASS_KEY);
                mbeanServer = (JalistoMBeanServer) Class.forName(className).newInstance();
                mbeanServer.init(props.getPropertiesPath());
                mbeanServer.run();
            } catch (InstantiationException e) {
                throw new JalistoException("cannot instanciate internal access", e);
            } catch (IllegalAccessException e) {
                throw new JalistoException("cannot initialize internal access", e);
            } catch (ClassNotFoundException e) {
                throw new JalistoException("cannot find internal access class", e);
            }
        } else {
            mbeanServer.addSession(props.getPropertiesPath());
        }
    }

    public AccessController getAccessController(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public LockTable getLockTable(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public Object getSessionById(Object sessionId) {
        throw new UnsupportedOperationException();
    }

    public OidTable getOidTable(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public InternalPhysicalFileAccess getInternalPhysicalAccess(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public Session getSession(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public ClientCommunicationAgent getClientCommunicationAgent(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public JalistoServer getJalistoServer(String communicationFactoryClassName) {
        throw new UnsupportedOperationException();
    }

    public InternalMetaRepository getMetaRepository(JalistoProperties properties, ClientCommunicationAgent connexion) {
        throw new UnsupportedOperationException();
    }

}
