/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.factory;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.internal.multi.LockTable;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.server.LogicalPageAccessImpl;
import org.objectweb.jalisto.se.impl.server.InternalPhysicalFileAccessImpl;
import org.objectweb.jalisto.se.impl.server.*;
import org.objectweb.jalisto.se.impl.lock.AccessController;
import org.objectweb.jalisto.se.impl.lock.LockTableImpl;
import org.objectweb.jalisto.se.impl.mono.OidTableMonoImpl;
import org.objectweb.jalisto.se.impl.mono.SessionMonoImpl;
import org.objectweb.jalisto.se.impl.multi.InTransactionBaseImageMultiImpl;
import org.objectweb.jalisto.se.impl.multi.OidTableMultiImpl;
import org.objectweb.jalisto.se.impl.multi.SessionMultiImpl;
import org.objectweb.jalisto.se.impl.readonly.*;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

public class InternalGeneralFactory extends  InternalFactoryImpl {

    protected Map accessController;
    protected Map lockManagers;
    protected Map sessions;

    public static InternalFactory getInstance() {
        if (instance == null) {
            instance = new InternalGeneralFactory();
            instance.init();
        }
        return instance;
    }

    /**
     * ********************************************************************************************
     */

    public void init() {
        super.init();
        accessController = new Hashtable();
        lockManagers = new Hashtable();
        sessions = new Hashtable();
    }

    public synchronized AccessController getAccessController(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!accessController.containsKey(path)) {
            AccessController controller = new AccessController();
            accessController.put(path, controller);
        }
        return (AccessController) accessController.get(path);
    }

    public synchronized Session getSession(JalistoProperties properties) {
        Session session;
        if (properties.isMonoImplementation()) {
            String name = properties.getName();
            if (!sessions.containsKey(name)) {
                session = new SessionMonoImpl(properties);
                sessions.put(name, session);
            } else {
                session = (Session) sessions.get(name);
            }
            if (session.currentTransaction().isActive()) {
                System.out.println("WARNING : get the mono session already active");
            }
        } else if (properties.isReadOnlyImplementation()) {
            session = new SessionReadOnlyImpl(properties);
        } else {
            session = new SessionMultiImpl(properties);
            sessions.put(session.getInternalSession().getSessionId(), session);
        }
        return session;
    }

    public synchronized OidTable getOidTable(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!oidTables.containsKey(path)) {
            OidTable oidTable;
            InternalPhysicalFileAccess physicalAccess = getInternalPhysicalAccess(properties);
            if (properties.isMonoImplementation()) {
                oidTable = OidTableMonoImpl.getAnOidTable(physicalAccess, properties);
            } else if (properties.isReadOnlyImplementation()) {
                oidTable = OidTableReadOnlyImpl.getAnOidTable(physicalAccess, properties);
            } else {
                oidTable = OidTableMultiImpl.getAnOidTable(physicalAccess, properties);
            }
            oidTables.put(path, oidTable);
        }
        return (OidTable) oidTables.get(path);
    }

    public synchronized InTransactionBaseImage getAInTransactionBaseImage(
            PluggablePhysicalFileAccess physicalAccess, JalistoProperties properties) {
        if (properties.isMonoImplementation() || properties.isReadOnlyImplementation()) {
            return InTransactionBaseImageImpl.getInTransactionBaseImageInstance(physicalAccess, properties);
        } else {
            return InTransactionBaseImageMultiImpl.getInTransactionBaseImageInstance(physicalAccess, properties);
        }
    }

    public synchronized LogicalSystemPageAccess getLogicalAccess(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!logicals.containsKey(path)) {
            LogicalSystemPageAccess logical;
            if (properties.isReadOnlyImplementation()) {
                logical = new LogicalPageAccessReadOnlyImpl(properties);
            } else {
                logical = new LogicalPageAccessImpl(properties);
            }
            logicals.put(path, logical);
        }
        return (LogicalSystemPageAccess) logicals.get(path);
    }

    public synchronized InternalPhysicalFileAccess getInternalPhysicalAccess(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!physicals.containsKey(path)) {
            InternalPhysicalFileAccess physical;
            if (properties.isReadOnlyImplementation()) {
                physical = new InternalPhysicalFileAccessReadOnlyImpl(properties);
            } else {
                physical = new InternalPhysicalFileAccessImpl(properties);
            }
            physicals.put(path, physical);
        }
        return (InternalPhysicalFileAccess) physicals.get(path);
    }


    public synchronized LockTable getLockTable(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!lockManagers.containsKey(path)) {
            lockManagers.put(path, new LockTableImpl());
        }
        return (LockTable) lockManagers.get(path);
    }

    public Object getSessionById(Object sessionId) {
        return sessions.get(sessionId);
    }

    public void cleanFactory() {
        Iterator sessionsIt = sessions.values().iterator();
        while (sessionsIt.hasNext()) {
            SessionInternal session = (SessionInternal) sessionsIt.next();
            if (session.currentTransaction().isActive()) {
                throw new JalistoException("cannot clean factory : session " + session.getSessionId() + " is active");
            }
            if (session.isOpen()) {
                throw new JalistoException("cannot clean factory : session " + session.getSessionId() + " is open");
            }
        }

        accessController.clear();
        lockManagers.clear();
        sessions.clear();
        super.cleanFactory();
    }

}
