/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.factory;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.server.InternalPhysicalFileAccessImpl;
import org.objectweb.jalisto.se.impl.mono.OidTableMonoImpl;
import org.objectweb.jalisto.se.impl.mono.SessionMonoImpl;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

public class InternalMonoFactory extends InternalFactoryImpl {

    protected Map sessions;

    public static InternalFactory getInstance() {
        if (instance == null) {
            instance = new InternalMonoFactory();
            instance.init();
        }
        return instance;
    }

    public void init() {
        super.init();
        sessions = new Hashtable();
    }

    public synchronized Session getSession(JalistoProperties properties) {
        Session session;
        if (properties.isMonoImplementation()) {
            String name = properties.getName();
            if (!sessions.containsKey(name)) {
                session = new SessionMonoImpl(properties);
                sessions.put(name, session);
            } else {
                session = (Session) sessions.get(name);
            }
            if (session.currentTransaction().isActive()) {
                System.out.println("WARNING : get the mono session already active");
            }
        } else {
            throw new JalistoException("try to get no mono object implementation with mono factory");
        }
        return session;
    }

    public synchronized OidTable getOidTable(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!oidTables.containsKey(path)) {
            OidTable oidTable;
            InternalPhysicalFileAccess physicalAccess = getInternalPhysicalAccess(properties);
            if (properties.isMonoImplementation()) {
                oidTable = OidTableMonoImpl.getAnOidTable(physicalAccess, properties);
            } else {
                throw new JalistoException("try to get no mono object implementation with mono factory");
            }
            oidTables.put(path, oidTable);
        }
        return (OidTable) oidTables.get(path);
    }

    public synchronized InternalPhysicalFileAccess getInternalPhysicalAccess(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!physicals.containsKey(path)) {
            physicals.put(path, new InternalPhysicalFileAccessImpl(properties));
        }
        return (InternalPhysicalFileAccess) physicals.get(path);
    }

    public Object getSessionById(Object sessionId) {
        return sessions.get(sessionId);
    }

    public void cleanFactory() {
        Iterator sessionsIt = sessions.values().iterator();
        while (sessionsIt.hasNext()) {
            SessionInternal session = (SessionInternal) sessionsIt.next();
            if (session.currentTransaction().isActive()) {
                throw new JalistoException("cannot clean factory : session " + session.getSessionId() + " is active");
            }
            if (session.isOpen()) {
                throw new JalistoException("cannot clean factory : session " + session.getSessionId() + " is open");
            }
        }
        sessions.clear();
        super.cleanFactory();
    }

}
