/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.factory;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.internal.InternalFactory;
import org.objectweb.jalisto.se.api.internal.OidTable;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.readonly.OidTableReadOnlyImpl;
import org.objectweb.jalisto.se.impl.readonly.InternalPhysicalFileAccessReadOnlyImpl;
import org.objectweb.jalisto.se.impl.readonly.SessionReadOnlyImpl;

public class InternalReadOnlyFactory extends InternalFactoryImpl {

    private static InternalFactory instance;

    public static InternalFactory getInstance() {
        if (instance == null) {
            instance = new InternalReadOnlyFactory();
            instance.init();
        }
        return instance;
    }

    public void setSelfInstance() {
        if (InternalReadOnlyFactory.instance == null) {
            InternalReadOnlyFactory.instance = this;
        }
    }

    public synchronized Session getSession(JalistoProperties properties) {
        if (!properties.isReadOnlyImplementation()) {
            throw new JalistoException("try to get a none read only session with Jalisto's read only jar version");
        }
        return new SessionReadOnlyImpl(properties);
    }

    public synchronized OidTable getOidTable(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!oidTables.containsKey(path)) {
            InternalPhysicalFileAccess physicalAccess = getInternalPhysicalAccess(properties);
            oidTables.put(path, OidTableReadOnlyImpl.getAnOidTable(physicalAccess, properties));
        }
        return (OidTable) oidTables.get(path);
    }

    public synchronized InternalPhysicalFileAccess getInternalPhysicalAccess(JalistoProperties properties) {
        String path = properties.getDbFileFullName();
        if (!physicals.containsKey(path)) {
            physicals.put(path, new InternalPhysicalFileAccessReadOnlyImpl(properties));
        }
        return (InternalPhysicalFileAccess) physicals.get(path);
    }

}
