/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.factory;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.cache.JalistoCache;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.api.remote.ClientCommunicationAgent;
import org.objectweb.jalisto.se.api.remote.CommunicationFactory;
import org.objectweb.jalisto.se.api.remote.JalistoServer;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.cache.GenericCacheImplEmpty;
import org.objectweb.jalisto.se.impl.client.InternalMetaRepositoryClientImpl;
import org.objectweb.jalisto.se.impl.client.JalistoPropertiesClientImpl;
import org.objectweb.jalisto.se.impl.client.SessionClientImpl;
import org.objectweb.jalisto.se.impl.server.IdentityProvider;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.impl.trace.TraceContext;

import java.util.*;

public class InternalRemoteFactory extends InternalFactoryImpl {

    protected Map sessions;

    public static InternalFactory getInstance() {
        if (instance == null) {
            instance = new InternalRemoteFactory();
            instance.init();
        }
        return instance;
    }

    public void init() {
        super.init();
        sessions = new Hashtable();
    }

    public synchronized JalistoProperties getProperties(String path) {
        if (!jalistoProperties.containsKey(path)) {
            JalistoPropertiesClientImpl p = new JalistoPropertiesClientImpl(path);
            jalistoProperties.put(path, p);
        }
        return (JalistoProperties) jalistoProperties.get(path);
    }

    public synchronized Trace getTracer(JalistoProperties properties) {
        String path = properties.getServerPropertiesPath();
        if (!tracers.containsKey(path)) {
            TraceContext traceContext = new TraceContext(properties.isTraceEnable(), properties.getTraceModuleNames());
            Trace trace = traceContext.getTracer();
            tracers.put(path, trace);
        }
        return (Trace) tracers.get(path);
    }

    public synchronized Session getSession(JalistoProperties properties) {
        Session session = new SessionClientImpl(properties);
        sessions.put(session.getInternalSession().getSessionId(), session);
        return session;
    }

    public synchronized InternalMetaRepository getMetaRepository(JalistoProperties properties,
                                                                 ClientCommunicationAgent connexion) {
        String path = properties.getServerPropertiesPath();
        if (!metaRepositories.containsKey(path)) {
            metaRepositories.put(path, new InternalMetaRepositoryClientImpl(properties, connexion));
        }
        return (InternalMetaRepositoryClientImpl) metaRepositories.get(path);
    }

    public synchronized Map getCache(JalistoProperties properties, int size, String name) {
        if (size > 0) {
            String className = properties.getProperty(JalistoProperties.CACHE_IMPLEMENTATION_KEY);
            try {
                JalistoCache cache = (JalistoCache) Class.forName(className).newInstance();
                cache.init(size, name, properties.getCacheClearPourcent());
                cache.setTrace(getTracer(properties));
                return cache;
            } catch (InstantiationException e) {
                throw new JalistoException("cannot instanciate cache", e);
            } catch (IllegalAccessException e) {
                throw new JalistoException("cannot initialize cache", e);
            } catch (ClassNotFoundException e) {
                throw new JalistoException("cannot find internal cache", e);
            }
        } else if (size == 0) {
            return new GenericCacheImplEmpty();
        } else {
            return new HashMap();
        }
    }

    public synchronized ClientCommunicationAgent
            getClientCommunicationAgent(JalistoProperties properties) {
        String className = properties.getCommunicationFactoryClassName();
        try {
            CommunicationFactory communicationFactory = (CommunicationFactory) Class.forName(className).newInstance();
            return communicationFactory.getCommunicationAgentClient(properties);
        } catch (InstantiationException e) {
            throw new JalistoException("cannot instanciate CommunicationFactory", e);
        } catch (IllegalAccessException e) {
            throw new JalistoException("cannot initialize CommunicationFactory", e);
        } catch (ClassNotFoundException e) {
            throw new JalistoException("cannot find internal CommunicationFactory", e);
        }
    }

    public synchronized JalistoServer getJalistoServer(String communicationFactoryClassName) {
        try {
            CommunicationFactory communicationFactory =
                    (CommunicationFactory) Class.forName(communicationFactoryClassName).newInstance();
            return communicationFactory.getJalistoServer();
        } catch (InstantiationException e) {
            throw new JalistoException("cannot instanciate CommunicationFactory", e);
        } catch (IllegalAccessException e) {
            throw new JalistoException("cannot initialize CommunicationFactory", e);
        } catch (ClassNotFoundException e) {
            throw new JalistoException("cannot find internal CommunicationFactory", e);
        }
    }

    public void cleanFactory() {
        Iterator sessionsIt = sessions.values().iterator();
        while (sessionsIt.hasNext()) {
            SessionInternal session = (SessionInternal) sessionsIt.next();
            if (session.currentTransaction().isActive()) {
                throw new JalistoException("cannot clean factory : session " + session.getSessionId() + " is active");
            }
            if (session.isOpen()) {
                throw new JalistoException("cannot clean factory : session " + session.getSessionId() + " is open");
            }
        }
        sessions.clear();
        super.cleanFactory();
    }

    public synchronized InTransactionBaseImage getAInTransactionBaseImage(PluggablePhysicalFileAccess physicalAccess,
                                                                          JalistoProperties properties) {
         throw new UnsupportedOperationException();
    }

    public PluggablePhysicalFileAccess getPhysicalAccess(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public synchronized IdentityProvider getIdentityProvider(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public synchronized LogicalSystemPageAccess getLogicalAccess(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public synchronized InternalMetaRepository getMetaRepository(JalistoProperties properties) {
        throw new UnsupportedOperationException();
    }

    public synchronized Object getSessionId(JalistoProperties properties, Session session) {
        throw new UnsupportedOperationException();
    }

    public void launchMBeanHtmlServer(JalistoProperties props) {
        throw new UnsupportedOperationException();
    }

}
