/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.lock;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.internal.DataWrapper;
import org.objectweb.jalisto.se.api.internal.multi.SessionMulti;
import org.objectweb.jalisto.se.api.internal.multi.LockManager;
import org.objectweb.jalisto.se.api.internal.multi.LockTable;
import org.objectweb.jalisto.se.exception.multi.SleepingException;

import java.util.Collection;

public abstract class LockManagerDefault implements LockManager {

    public void setLockTable(LockTable lockTable) {
        this.lockTable = lockTable;
    }

    public DataWrapper getLockedValue(LogicalOid floid, Object sessionId) {
        return lockTable.getLockedValue(floid, sessionId);
    }

    public void setLockedValue(LogicalOid floid, Object sessionId, DataWrapper value, short update) {
        lockTable.getLock(floid).updateTransactionnalValue(sessionId, value, update);
    }

    public short getUpdate(LogicalOid floid, Object sessionId) {
        return lockTable.getLock(floid).getUpdate(sessionId);
    }

    public void setSession(SessionMulti session) {
        this.session = session;
        this.sessionId = session.getSessionId();
        this.accessController = session.getAccessController();
    }

    protected Lock takeLock(Object floid, int action) {
        boolean isLockTaken = false;
        boolean isAsleeping = false;
        Lock lock = null;
        while (!isLockTaken) {
            synchronized (lockTable) {     // avoid someone remove lock after we get it and before we put a lock
                lock = lockTable.getLockOrCreate(floid);
                try {
                    if (action == READ) {
                        lock.takeReadLock(sessionId);
                    } else {
                        lock.takeWriteLock(sessionId);
                    }
                    isLockTaken = true;
                } catch (SleepingException fse) {
                    isAsleeping = lock.addSleepingSession(sessionId);
                }
            }
            if (isAsleeping) {
                session.goToBed();
                session.sleep(floid, "during takeLock with floid = " + String.valueOf(floid) +
                                     " with action code = " + action);
                isAsleeping = false;
            }
        }
        return lock;
    }

    /**
     * ***********************************************************************************************
     */

    public void begin() {
        session.getOidTable().begin(sessionId);
    }

    public void checkLocksForSession(Collection floids, Object sessionId) {
        lockTable.checkLocksForSession(floids, sessionId);
    }

    public boolean containsDeletedValue(LogicalOid floid, Object sessionId) {
        return lockTable.containsDeletedValue(floid, sessionId);
    }

    public boolean containsLockedValue(LogicalOid floid, Object sessionId) {
        return lockTable.containsLockedValue(floid, sessionId);
    }

    public void finishCreateObject(LogicalOid floid, DataWrapper objectToCreate) {
    }

    public void finishDeleteObjectByOid(LogicalOid floid) {
    }

    public void finishMakeNewFileOid(LogicalOid floid) {
    }

    public void finishReadObjectByOid(LogicalOid floid, DataWrapper objectToRead, short update) {
    }

    public void finishUpdateObjectByOid(LogicalOid floid, DataWrapper objectToUpdate, short update) {
    }

    public void prepareCreateObject(LogicalOid floid, DataWrapper objectToCreate) {
    }

    public void prepareDeleteObjectByOid(LogicalOid floid, short update) {
    }

    public void prepareReadObjectByOid(LogicalOid floid) {
    }


    protected SessionMulti session;
    protected LockTable lockTable;
    protected AccessController accessController;
    protected Object sessionId;


    protected static final int READ = 0;
    protected static final int WRITE = 1;
}
