/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.lock;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.internal.DataWrapper;
import org.objectweb.jalisto.se.exception.TransactionException;
import org.objectweb.jalisto.se.exception.multi.LockException;
import org.objectweb.jalisto.se.exception.multi.SleepingException;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class LockManagerOptimistic extends LockManagerDefault {

    public void prepareCreateObject(LogicalOid floid, DataWrapper objectToCreate) {
        Lock lock = takeLock(floid, WRITE);
        lock.addTransactionnalValue(sessionId, objectToCreate, ObjectTransactionnalState.CREATE_ACTION, (short) 0);
    }

    public void prepareReadObjectByOid(LogicalOid floid) {
        takeLock(floid, READ);
    }

    public void prepareDeleteObjectByOid(LogicalOid floid, short update) {
        takeLock(floid, WRITE).addTransactionnalValue(sessionId, null, ObjectTransactionnalState.DELETE_ACTION, update);
    }

    public void finishMakeNewFileOid(LogicalOid floid) {
        lockTable.getLockOrCreate(floid).
                addTransactionnalValue(sessionId, null, ObjectTransactionnalState.NEW_OID_ACTION, (short) 0);
    }

    public void finishCreateObject(LogicalOid floid, DataWrapper objectToCreate) {
        lockTable.getLock(floid).releaseWriteLock(sessionId);
    }

    public void finishReadObjectByOid(LogicalOid floid, DataWrapper objectToRead, short update) {
        Lock lock = lockTable.getLock(floid);
        lock.addTransactionnalValue(sessionId, objectToRead, ObjectTransactionnalState.READ_ACTION, update);
        lock.releaseReadLock(sessionId);
    }

    public void finishUpdateObjectByOid(LogicalOid floid, DataWrapper objectToUpdate, short update) {
        Lock lock = takeLock(floid, WRITE);
        lock.addTransactionnalValue(sessionId, objectToUpdate, ObjectTransactionnalState.UPDATE_ACTION, update);
        lock.releaseWriteLock(sessionId);
    }

    public void finishDeleteObjectByOid(LogicalOid floid) {
        lockTable.getLock(floid).releaseWriteLock(sessionId);
    }

    public void commit() {
        commitRollback(true);
    }

    public void rollback() {
        commitRollback(false);
    }

    public void commitRollback(boolean isCommit) {
        try {
            accessController.getAccessControl(AccessController.COMMIT_ROLLBACK_LOCKMANAGER);
            ArrayList usefullLocks = new ArrayList();
            ArrayList usefullFloids = new ArrayList();
            while (!getUsedLock(usefullFloids, usefullLocks)) {
                session.sleep(usefullFloids.get(0), "during rollback");
            }        // we have all needed lock...
            if (isCommit) {
                    checkUpdates(usefullLocks, usefullFloids);
                    internalCommit(usefullLocks, usefullFloids);
            } else {
                internalRollback(usefullLocks, usefullFloids);
            }
        } finally {
            accessController.releaseAccessControl(AccessController.COMMIT_ROLLBACK_LOCKMANAGER);
        }
    }

    private void internalCommit(List usefullLocks, List usefullFloids) {
        for (int i = 0; i < usefullLocks.size(); i++) {
            Lock lock = (Lock) usefullLocks.get(i);
            LogicalOid floid = (LogicalOid) usefullFloids.get(i);
            if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.UPDATE_ACTION) {
                lock.update(sessionId);
                session.updateObjectCommit(floid);
                session.getOidTable().setUpdate(sessionId, floid, lock.getUpdate(sessionId));
            } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.CREATE_ACTION) {
                lock.update(sessionId);
                session.createObjectCommit(floid);
                session.getOidTable().setUpdate(sessionId, floid, lock.getUpdate(sessionId));
            } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.DELETE_ACTION) {
                lock.update(sessionId);
                session.deleteObjectCommit(floid);
                session.getOidTable().setUpdate(sessionId, floid, lock.getUpdate(sessionId));
            } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.NEW_OID_ACTION) {
                session.makeNewFileOidCommit(floid);
            }
            lock.releaseAllLock(sessionId);
            lockTable.removeLockByFloid(floid);
        }
        session.getOidTable().commit(sessionId);
        session.getFileAccess().commit(sessionId);
    }

    private void internalRollback(List usefullLocks, List usefullFloids) {
        for (int i = 0; i < usefullLocks.size(); i++) {
            Lock lock = (Lock) usefullLocks.get(i);
            synchronized (lock) {
                LogicalOid floid = (LogicalOid) usefullFloids.get(i);
                if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.CREATE_ACTION) {
                    session.createObjectRollback(floid);
                } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.UPDATE_ACTION) {
                    session.updateObjectRollback(floid);
                } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.DELETE_ACTION) {
                    session.deleteObjectRollback(floid);
                } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.NEW_OID_ACTION) {
                    session.makeNewFileOidRollback(floid);
                }
                lock.releaseAllLock(sessionId);
                lockTable.removeLockByFloid(floid);
            }
        }
        session.getOidTable().rollback(sessionId);
        session.getFileAccess().rollback(sessionId);
    }

    // floidList, lockList must be empty
    private boolean getUsedLock(List floidList, List lockList) {
        floidList.clear();
        lockList.clear();
        boolean hasException = false;
        synchronized (lockTable) {
            Iterator lockedFloids = lockTable.getKeys();
            while (lockedFloids.hasNext()) {
                LogicalOid floid = (LogicalOid) lockedFloids.next();
                Lock lock = lockTable.getLock(floid);
                if (lock.hasModifiedValueWith(sessionId)) {
                    try {
                        if (lock.mustTakeWriteLock(sessionId)) {
                            lock.takeWriteLock(sessionId);
                        }
                        lockList.add(lock);
                        floidList.add(floid);
                    } catch (SleepingException fse) {
                        hasException = true;
                        session.goToBed();
                        try {
                            lock.addSleepingSession(sessionId);
                        } catch (LockException dle) {
                        } finally {
                            floidList.clear();
                            lockList.clear();
                            floidList.add(floid);
                        }
                    }
                }
            }
        }
        return (!hasException);
    }

    private void checkUpdates(List locksToCheck, List floidsToCheck) {
        for (int i = 0; i < locksToCheck.size(); i++) {
            Lock lock = (Lock) locksToCheck.get(i);
            int currentAction = lock.getCurrentAction(sessionId);
            if ((currentAction == ObjectTransactionnalState.CREATE_ACTION) ||
                (currentAction == ObjectTransactionnalState.UPDATE_ACTION) ||
                (currentAction == ObjectTransactionnalState.DELETE_ACTION)) {
                checkUpdate((LogicalOid) floidsToCheck.get(i), lock.getUpdate(sessionId));
//                lock.update(sessionId);
            }
        }
    }

    private void checkUpdate(LogicalOid floid, short currentUpdate) {
        short update = session.getOidTable().getUpdate(sessionId, floid, true);
        if ((update == 0) && (currentUpdate == 1)) {
            return;
        }
        if (currentUpdate != update) {
            TransactionException jalistoTransaction = new TransactionException("object have been modified or deleted since you get it");
            jalistoTransaction.setOptimistic();
            throw jalistoTransaction;
        }
    }
}
