/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.lock;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.internal.DataWrapper;

import java.util.Iterator;

public class LockManagerPessimistic extends LockManagerDefault {

    public void prepareCreateObject(LogicalOid floid, DataWrapper objectToCreate) {
        takeLock(floid, WRITE);
    }

    public void prepareDeleteObjectByOid(LogicalOid floid, short update) {
        takeLock(floid, WRITE).addTransactionnalValue(sessionId, null,
                                                      ObjectTransactionnalState.DELETE_ACTION,
                                                      (short) 0);
    }

    public void prepareReadObjectByOid(LogicalOid floid) {
        takeLock(floid, READ);
    }

    public void prepareUpdateObjectByOid(LogicalOid floid, DataWrapper objectToUpdate) {
        takeLock(floid, WRITE);
    }


    public void finishCreateObject(LogicalOid floid, DataWrapper objectToCreate) {
        lockTable.getLock(floid).addTransactionnalValue(sessionId, objectToCreate,
                                                        ObjectTransactionnalState.CREATE_ACTION, (short) 0);
    }

    public void finishReadObjectByOid(LogicalOid floid, DataWrapper objectToRead, short update) {
        lockTable.getLock(floid).addTransactionnalValue(sessionId, objectToRead,
                                                        ObjectTransactionnalState.READ_ACTION, update);
    }

    public void finishUpdateObjectByOid(LogicalOid floid, DataWrapper objectToUpdate, short update) {
        takeLock(floid, WRITE).addTransactionnalValue(sessionId, objectToUpdate,
                                                      ObjectTransactionnalState.UPDATE_ACTION, update);
    }


    public synchronized void commit() {
        try {
            accessController.getAccessControl(AccessController.COMMIT_ROLLBACK_LOCKMANAGER);
            Iterator floids = lockTable.getKeys();
            while (floids.hasNext()) {
                LogicalOid floid = (LogicalOid) floids.next();
                Lock lock = lockTable.getLockOrCreate(floid);
                if (lock.hasModifiedValueWith(sessionId)) {
                    if (lock.hasWriteLockWith(sessionId)) {
                        if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.CREATE_ACTION) {
                            session.createObjectCommit(floid);
                        } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.UPDATE_ACTION) {
                            lock.update(sessionId);
                            session.updateObjectCommit(floid);
                        } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.DELETE_ACTION) {
                            session.deleteObjectCommit(floid);
                        } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.NEW_OID_ACTION) {
                            session.makeNewFileOidCommit(floid);
                        }
                    }
                    lock.releaseAllLock(sessionId);
                    lockTable.removeLockByFloid(floid);
                }
            }
            session.getFileAccess().getPhysicalAccess().commit();
        } finally {
            accessController.releaseAccessControl(AccessController.COMMIT_ROLLBACK_LOCKMANAGER);
        }
    }

    public synchronized void rollback() {
        try {
            accessController.getAccessControl(AccessController.COMMIT_ROLLBACK_LOCKMANAGER);
            Iterator floids = lockTable.getKeys();
            while (floids.hasNext()) {
                LogicalOid floid = (LogicalOid) floids.next();
                Lock lock = lockTable.getLockOrCreate(floid);
                if (lock.hasModifiedValueWith(sessionId)) {
                    if (lock.hasWriteLockWith(sessionId)) {
                        if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.CREATE_ACTION) {
                            session.createObjectRollback(floid);
                        } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.UPDATE_ACTION) {
                            session.updateObjectRollback(floid);
                        } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.DELETE_ACTION) {
                            session.deleteObjectRollback(floid);
                        } else if (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.NEW_OID_ACTION) {
                            session.makeNewFileOidRollback(floid);
                        }
                    }
                    lock.releaseAllLock(sessionId);
                    lockTable.removeLockByFloid(floid);
                }
            }
            session.getFileAccess().rollback(sessionId);
        } finally {
            accessController.releaseAccessControl(AccessController.COMMIT_ROLLBACK_LOCKMANAGER);
        }
    }

}
