/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.lock;

import org.objectweb.jalisto.se.api.internal.DataWrapper;
import org.objectweb.jalisto.se.api.internal.multi.LockTable;
import org.objectweb.jalisto.se.exception.multi.LockException;

import java.util.*;

public class LockTableImpl implements LockTable {

    public LockTableImpl() {
        locks = new Hashtable();
    }

    public synchronized Iterator getKeys() {
        ArrayList keys = new ArrayList();
        keys.addAll(locks.keySet());
        return keys.iterator();
    }

    public synchronized Lock getLockOrCreate(Object floid) {
        Lock lock;
        if (!locks.containsKey(floid)) {
            lock = Lock.newInstance();
            locks.put(floid, lock);
        } else {
            lock = (Lock) locks.get(floid);
        }
        return lock;
    }

    public synchronized Lock getLock(Object floid) {
        if (!locks.containsKey(floid)) {
            throw new LockException();
        }
        return (Lock) locks.get(floid);
    }

    public synchronized DataWrapper getLockedValue(Object floid, Object sessionId) {
        Lock lock = (Lock) locks.get(floid);
        if (lock != null) {
            return lock.getModifiedValue(sessionId);
        }
        throw new LockException("no lock find with this floid " + floid);
    }

    // TODO methode  limminer surement
    public synchronized void checkLocksForSession(Collection classeFloids, Object sessionId) {
        Iterator floids = locks.keySet().iterator();
        while (floids.hasNext()) {
            Object floid = floids.next();
            Lock lock = (Lock) locks.get(floid);
            if (lock.hasToBeRemoveForExtentWith(sessionId)) {
                classeFloids.remove(floid);
            }
        }
    }

    public synchronized boolean containsDeletedValue(Object floid, Object sessionId) {
        Lock lock = (Lock) locks.get(floid);
        if (lock != null) {
            return (lock.getCurrentAction(sessionId) == ObjectTransactionnalState.DELETE_ACTION);
        }
        return false;
    }

    public synchronized boolean containsLockedValue(Object floid, Object sessionId) {
        Lock lock = (Lock) locks.get(floid);
        if (lock != null) {
            return (lock.getModifiedValue(sessionId) != null);
        }
        return false;
    }

    public boolean hasLock(Object floid) {
        return locks.containsKey(floid);
    }

    public void publishLocks() {
        System.out.println(locks.toString());
    }

    // lock is already synchronized by caller
    public synchronized void removeLockByFloid(Object floid) {
        Lock lock = (Lock) locks.get(floid);
        if (lock != null) {
            if (lock.canBeDeleted()) {
                locks.remove(floid);
                lock.release();
            }
        }
    }

    public boolean isEmpty() {
        return locks.isEmpty();
    }


    private Map locks; // key is floid
}
