/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.meta;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.api.query.FielComparator;
import org.objectweb.jalisto.se.exception.SchemaException;
import org.objectweb.jalisto.se.impl.InFileAddress;

import java.util.ArrayList;
import java.util.Iterator;

public class ClassDescriptionImpl implements ClassDescription {

    public ClassDescriptionImpl(String fullClassName) {
        this.className = fullClassName;
        this.fields = new FieldDescriptionImpl[0];
    }

    public String getClassName() {
        return className;
    }

    public int getIndex(String fieldName) {
        for (short i = 0; i < fields.length; i++) {
            if (fields[i].getFieldName().equals(fieldName)) {
                return i;
            }
        }
        throw new SchemaException("no field with the name " + fieldName + " in class " + className);
    }

    public FieldDescription getFieldDescription(int index) {
        return fields[index];
    }

    public FielComparator getComparator(int index) {
        return fields[index].getType().getComparator();
    }

    public boolean isIndexDefined(int index) {
        return fields[index].isIndexDefined();
    }

    public InFileAddress getIndexIfa(int index) {
        return fields[index].getIndexDescription().getValueIndexIfa();
    }

    public void setIndexIfa(int index, InFileAddress ifa) {
        fields[index].getIndexDescription().setValueIndexIfa(ifa);
    }

    public Iterator getIndexedFieldDescription() {
        ArrayList list = new ArrayList();
        for (short i = 0; i < fields.length; i++) {
            if (fields[i].getIndexDescription().hasIndex()) {
                list.add(fields[i].getIndexDescription());
            }
        }
        return list.iterator();
    }

    public short getIndexType(int index) {
        return fields[index].getIndexDescription().getValueIndexType();
    }

    public void setIndexType(int index, short indexType) {
        fields[index].getIndexDescription().setValueIndexType(indexType);
    }

    public short getNodePageSize(int index) {
        return fields[index].getIndexDescription().getNodePageSize();
    }

    public void setNodePageSize(int index, short size) {
        fields[index].getIndexDescription().setNodePageSize(size);
    }

    public short getNodeSize(int index) {
        return fields[index].getIndexDescription().getNodeSize();
    }

    public void setNodeSize(int index, short size) {
        fields[index].getIndexDescription().setNodeSize(size);
    }

    public short getLeafPageSize(int index) {
        return fields[index].getIndexDescription().getLeafPageSize();
    }

    public void setLeafPageSize(int index, short size) {
        fields[index].getIndexDescription().setLeafPageSize(size);
    }

    public Object getValueFor(String fieldName, Object[] values) {
        return values[getIndex(fieldName)];
    }

    /**
     * Return an array of the names of the declared fields of the described class.
     * 
     * @return String[] The names of the described class fields.
     */
    public String[] getFieldNames() {
        String[] result = new String[fields.length];
        for (int i = 0; i < result.length; i++) {
            result[i] = fields[i].getFieldName();
        }
        return result;
    }

    /**
     * Return the number of field already declared for the described class.
     * 
     * @return int The number of declared fields.
     */
    public int getNbrFields() {
        return fields.length;
    }

    /**
     * Insert a new field in the description of the class.
     * 
     * @param fieldDescription The name of the field to insert.
     */
    public void addField(FieldDescription fieldDescription) {
        fieldDescription.setClassMeta(this);
        FieldDescriptionImpl[] nfd = new FieldDescriptionImpl[fields.length + 1];
        short index = fieldDescription.getIndex();
        if (index < 0) {
            index = (short) fields.length;
            fieldDescription.setIndex(index);
        }
        for (short i = 0; i < nfd.length; i++) {
            if (i < index) {
                nfd[i] = fields[i];
            } else if (i == index) {
                nfd[i] = (FieldDescriptionImpl) fieldDescription;
            } else {
                fields[i - 1].setIndex(i);
                nfd[i] = fields[i - 1];
            }
        }
        fields = nfd;
    }

    /**
     * Remove a field from the description of the class.
     * 
     * @param index The index of the place where to remove the field.
     */
    public void removeField(int index) {
        FieldDescriptionImpl[] nfd = new FieldDescriptionImpl[fields.length - 1];
        for (short i = 0; i < nfd.length; i++) {
            if (i < index) {
                nfd[i] = fields[i];
            } else {
                fields[i + 1].setIndex(i);
                nfd[i] = fields[i + 1];
            }
        }
        fields = nfd;
    }

    public void updateInBase() {
        repository.writePersistentClassMetaDescription(this);
    }

    public void setRepository(InternalMetaRepositoryImpl repository) {
        this.repository = repository;
    }

    public String toString() {
        return className + " with " + fields.length + " fields";
    }

    public String toStringFull(int nbrTab) {
        String tab = "";
        for (int i = 0; i < nbrTab; i++) {
            tab = tab + "\t";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(tab).append(className).append("\n");
        for (int i = 0; i < fields.length; i++) {
            sb.append(tab).append("\t").append(fields[i]).append("\n");
        }
        return sb.toString();
    }

    public boolean equals(Object o) {
        try {
            ClassDescriptionImpl candidate = (ClassDescriptionImpl) o;
            boolean classTest = candidate.className.equals(this.className);
            boolean sizeTest = (candidate.fields.length == this.fields.length);
            if (classTest && sizeTest) {
                for (int i = 0; i < fields.length; i++) {
                    if (!candidate.fields[i].equals(this.fields[i])) {
                        return false;
                    }
                }
                return true;
            }
            return false;
        } catch (Exception e) {
        }
        return false;
    }

    private String className;
    private FieldDescriptionImpl[] fields;
    private transient InternalMetaRepositoryImpl repository;


    static final long serialVersionUID = -7589377097934761459L;
}
