/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.meta;

import org.objectweb.jalisto.se.api.query.Index;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.server.page.LeafPage;
import org.objectweb.jalisto.se.impl.server.page.NodePage;

import java.io.Serializable;

public class IndexDescription implements Serializable {

    public IndexDescription(FieldDescriptionImpl fieldDescription) {
        this.fieldDescription = fieldDescription;
        this.nodePageSize = NodePage.DEFAULT_SIZE;
        this.leafPageSize = LeafPage.DEFAULT_SIZE;
        this.nodeSize = Index.NODE_DEFAULT_SIZE;
    }

    public boolean hasIndex() {
        return (valueIndexIfa != null);
    }

    public FieldDescriptionImpl getFieldDescription() {
        return fieldDescription;
    }

    public short getLeafPageSize() {
        return leafPageSize;
    }

    public void setLeafPageSize(short leafPageSize) {
        this.leafPageSize = leafPageSize;
    }

    public short getNodePageSize() {
        return nodePageSize;
    }

    public void setNodePageSize(short nodePageSize) {
        this.nodePageSize = nodePageSize;
    }

    public short getNodeSize() {
        return nodeSize;
    }

    public void setNodeSize(short nodeSize) {
        this.nodeSize = nodeSize;
    }

    public boolean isIndexDefined() {
        return (valueIndexIfa != null);
    }

    public InFileAddress getValueIndexIfa() {
        if (!fieldDescription.getType().isBasicType()) {
            throw new JalistoException("cannot use or define index on none basic type field : " + fieldDescription);
        }
        return valueIndexIfa;
    }

    public void setValueIndexIfa(InFileAddress valueIndexIfa) {
        if (!fieldDescription.getType().isBasicType()) {
            throw new JalistoException("cannot use or define index on none basic type field : " + fieldDescription);
        }
        this.valueIndexIfa = valueIndexIfa;
    }

    public short getValueIndexType() {
        return valueIndexType;
    }

    public void setValueIndexType(short valueIndexType) {
        this.valueIndexType = valueIndexType;
    }

    public boolean equals(Object o) {
        try {
            IndexDescription candidate = (IndexDescription) o;
            return (candidate.valueIndexIfa.equals(this.valueIndexIfa) &&
                    (candidate.valueIndexType == this.valueIndexType) &&
                    (candidate.nodeSize == this.nodeSize) &&
                    (candidate.nodePageSize == this.nodePageSize) &&
                    (candidate.leafPageSize == this.leafPageSize));
        } catch (Exception e) {
        }
        return false;
    }

    private FieldDescriptionImpl fieldDescription;
    private InFileAddress valueIndexIfa;
    private short valueIndexType;
    private short nodeSize;
    private short nodePageSize;
    private short leafPageSize;


    static final long serialVersionUID = -6989377087964761459L;
}
