/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.mono;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.DataWrapper;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.server.PhysicalOid;
import org.objectweb.jalisto.se.impl.server.SessionImpl;
import org.objectweb.jalisto.se.impl.trace.Trace;

public class SessionMonoImpl extends SessionImpl {
    public SessionMonoImpl(JalistoProperties properties) {
        super(properties);
        trace.println(Trace.DEBUG, "CREATE A MONO SESSION, id = {0}", sessionId);
    }

    // return oid
    public Object createObject(Object oid, Object[] objectToCreate) {
        trace.println(Trace.SESSION, "{0} : createObject({1}, {2})", sessionId, oid, objectToCreate);
        checkValidity("createObject", true);

        DataWrapper datas = JalistoFactory.getInternalFactory().getNewDataWrapper(props, objectToCreate);
        LogicalOid floid = getFloidFromOid(oid);
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        cache.put(floid, datas);
        if (queryManager != null) {
            ClassDescription meta = repository.getPersistentClassMetaDescription(fpoid.getClid());
            queryManager.getIndexManager().createObject(meta, floid, objectToCreate);
        }
        fileAccess.insertDatas(sessionId, fpoid, datas);
        oidTable.markAsCreated(sessionId, floid);
        return floid;
    }

    public void deleteObjectByOid(Object oid) {
        trace.println(Trace.SESSION, "{0} : deleteObjectByOid({1})", sessionId, oid);
        checkValidity("deleteObjectByOid", true);

        LogicalOid floid = getFloidFromOid(oid);
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        Object[] objectToRemove = null;
        if (cache.containsKey(floid)) {
            objectToRemove = ((DataWrapper) cache.remove(floid)).getDatas();
        }
        if (fpoid != null) {
            fileAccess.deleteDatas(sessionId, fpoid);
            if (queryManager != null) {
                ClassDescription meta = repository.getPersistentClassMetaDescription(fpoid.getClid());
                if (objectToRemove == null) {
                    objectToRemove = fileAccess.readDatas(sessionId, fpoid).getDatas();
                }
                queryManager.getIndexManager().removeObject(meta, floid, objectToRemove);
            }
            oidTable.removeFloid(sessionId, floid);
        }
    }

    // make new oid and put empty datas in base
    public Object makeNewFileOid(Class objectClass) {
        return makeNewFileOid(objectClass.getName());
    }

    public Object makeNewFileOid(String objectClassName) {
        trace.println(Trace.SESSION, "{0} : makeNewFileOid({1})", sessionId, objectClassName);
        checkValidity("makeNewFileOid", true);
        Object theClid = repository.getClidFromClassName(objectClassName);
        LogicalOid floid = oidTable.allocateNewFloid(((Short) theClid).shortValue());
        PhysicalOid fpoid = fileAccess.allocateNewFpoid(sessionId, theClid);
        oidTable.insertFpoid(sessionId, floid, fpoid);
        return floid.getClone();
    }

    public Object makeNewFileOid(LogicalOid floid) {
        trace.println(Trace.SESSION, "{0} : makeNewFileOid({1})", sessionId, floid);
        checkValidity("makeNewFileOid", true);
        LogicalOid clone = floid.getClone();
        Object theClid = new Short(clone.getClid());
        PhysicalOid fpoid = fileAccess.allocateNewFpoid(sessionId, theClid);
        oidTable.insertFpoid(sessionId, clone, fpoid);
        return floid;
    }

    public Object[] readObjectByOid(Object oid, boolean withCache) {
        trace.println(Trace.SESSION, "{0} : readObjectByOid({1})", sessionId, oid);
        checkValidity("readObjectByOid", true);

        LogicalOid floid = getFloidFromOid(oid);
        if (withCache && cache.containsKey(floid)) {
            return ((DataWrapper) cache.get(floid)).getDatas();
        } else {
            PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
            if (fpoid != null) {
                DataWrapper datas = fileAccess.readDatas(sessionId, fpoid);
                if (datas != null) {
                    if (withCache) {
                        cache.put(floid, datas);
                    }
                    return datas.getDatas();
                }
            }
            throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore");
        }
    }

    public Object updateObjectByOid(Object oid, Object[] objectToUpdate) {
        trace.println(Trace.SESSION, "{0} : readObjectByOid({1}, {2})", sessionId, oid, objectToUpdate);
        checkValidity("updateObjectByOid", true);
        checkSerialisable(objectToUpdate);

        LogicalOid floid = getFloidFromOid(oid);
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        DataWrapper datas = JalistoFactory.getInternalFactory().getNewDataWrapper(props, objectToUpdate);
        if (fpoid != null) {
            PhysicalOid updatedFpoid = fileAccess.updateDatas(sessionId, fpoid, datas);
            cache.put(floid, datas);
            if (!updatedFpoid.equals(fpoid)) {
                oidTable.updatePoid(sessionId, floid, updatedFpoid);
            }
            if (queryManager != null) {
                ClassDescription meta = repository.getPersistentClassMetaDescription(fpoid.getClid());
                Object[] oldValue = fileAccess.readDatas(sessionId, fpoid).getDatas();
                queryManager.getIndexManager().updateObject(meta, floid, oldValue, objectToUpdate);
            }
            return floid.getClone();
        }
        throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore");
    }
}
