/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.multi;

import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.multi.LockManager;
import org.objectweb.jalisto.se.api.internal.multi.LockTable;
import org.objectweb.jalisto.se.api.internal.multi.SessionMulti;
import org.objectweb.jalisto.se.api.internal.DataWrapper;
import org.objectweb.jalisto.se.api.internal.Constants;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.exception.TransactionException;
import org.objectweb.jalisto.se.impl.*;
import org.objectweb.jalisto.se.impl.server.PhysicalOid;
import org.objectweb.jalisto.se.impl.server.SessionImpl;
import org.objectweb.jalisto.se.impl.lock.*;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.JalistoFactory;

public class SessionMultiImpl extends SessionImpl implements SessionMulti {

    public SessionMultiImpl(JalistoProperties properties) {
        super(properties);
        trace.println(Trace.DEBUG, "CREATE A MULTI SESSION, id = {0}", sessionId);
        lockTable = JalistoFactory.getInternalFactory().getLockTable(properties);
        accessController = JalistoFactory.getInternalFactory().getAccessController(properties);
        String txMode = properties.getProperty(JalistoProperties.CONCURRENCY_MODE_KEY);
        if (txMode.equalsIgnoreCase(Constants.OPTIMISTIC_MODE)) {
            setOptimistic();
        } else if (txMode.equalsIgnoreCase(Constants.PESSIMISTIC_MODE)) {
            setPessimistic();
        } else {
            throw new JalistoException(
                    JalistoProperties.CONCURRENCY_MODE_KEY + " not valide in database properties");
        }
        try {
            timeOut = (new Integer(properties.getProperty(JalistoProperties.TIME_OUT_KEY))).intValue();
        } catch (Exception e) {
            throw new JalistoException(JalistoProperties.TIME_OUT_KEY + " not valide in database properties");
        }
        new DeadLockToken(sessionId);
    }


    public Object makeNewFileOid(Class objectClass) {
        return makeNewFileOid(objectClass.getName());
    }

    public Object makeNewFileOid(String objectClassName) {
        trace.println(Trace.SESSION, "{0} makeNewFileOid({1})", sessionId, objectClassName);
        checkValidity("makeNewFileOid", true);
        try {
            accessController.getAccessControl(AccessController.NEW_OID);
            Object theClid = repository.getClidFromClassName(objectClassName);
            LogicalOid floid = oidTable.allocateNewFloid(((Short) theClid).shortValue());
            PhysicalOid fpoid = fileAccess.allocateNewFpoid(sessionId, theClid);
            oidTable.insertFpoid(sessionId, floid, fpoid);
            lockManager.finishMakeNewFileOid(floid);
            return floid.getClone();
        } finally {
            accessController.releaseAccessControl(AccessController.NEW_OID);
        }
    }

    public Object makeNewFileOid(LogicalOid floid) {
        trace.println(Trace.SESSION, "{0} makeNewFileOid({1})", sessionId, floid);
        checkValidity("makeNewFileOid", true);
        try {
            LogicalOid clone = floid.getClone();
            accessController.getAccessControl(AccessController.NEW_OID);
            Object theClid = new Short(clone.getClid());
            PhysicalOid fpoid = fileAccess.allocateNewFpoid(sessionId, theClid);
            oidTable.insertFpoid(sessionId, clone, fpoid);
            lockManager.finishMakeNewFileOid(clone);
            return floid;
        } finally {
            accessController.releaseAccessControl(AccessController.NEW_OID);
        }
    }

    public void makeNewFileOidCommit(LogicalOid floid) {
        // nothing to do
    }

    public void makeNewFileOidRollback(LogicalOid floid) {
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        oidTable.removeFloid(sessionId, floid);
        if (fpoid != null) {
            fileAccess.desallocateFpoid(sessionId, fpoid);
        } else {
            throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
        }
    }

    public Object createObject(Object oid, Object[] objectToCreate) {
        checkValidity("createObject", true);
        DataWrapper datas = JalistoFactory.getInternalFactory().getNewDataWrapper(props, objectToCreate);
        LogicalOid floid = getFloidFromOid(oid);
        oidTable.markAsCreated(sessionId, floid);
        lockManager.prepareCreateObject(floid, datas);
        cache.put(floid, datas);
        lockManager.finishCreateObject(floid, datas);
        return floid;
    }

    public void createObjectCommit(LogicalOid floid) {
        trace.println(Trace.SESSION, "{0} createObjectCommit({1})", sessionId, floid);
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        DataWrapper datas = (DataWrapper) cache.get(floid);
        if (datas == null) {
            datas = lockManager.getLockedValue(floid, sessionId);
        }
        fileAccess.insertDatas(sessionId, fpoid, datas);
    }

    public void createObjectRollback(LogicalOid floid) {
        trace.println(Trace.SESSION, "{0} createObjectRollback({1})", sessionId, floid);
        if (cache.containsKey(floid)) {
            cache.remove(floid);
        }
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        oidTable.removeFloid(sessionId, floid);
        if (fpoid != null) {
            fileAccess.desallocateFpoid(sessionId, fpoid);
        } else {
            throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
        }
    }

    public Object[] readObjectByOid(Object oid, boolean withCache) {
        return readObjectByOidInDataObject(oid, withCache).getDatas();
    }

    public DataWrapper readObjectByOidInDataObject(Object oid, boolean withCache) {
        trace.println(Trace.SESSION, "{0} readObjectByOidInDataObject({1})", sessionId, oid);
        checkValidity("readObjectByOidInDataObject", true);
        LogicalOid floid = getFloidFromOid(oid);

        // delete not commited for this session
        if (lockManager.containsDeletedValue(floid, sessionId)) {
            lockManager.finishReadObjectByOid(floid, null, (short) -1);
            throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore");
        }

        lockManager.prepareReadObjectByOid(floid);

        DataWrapper result = null;
        short update;
        boolean readedFromCache = false;

        if (withCache && cache.containsKey(floid)) {
            result = (DataWrapper) cache.get(floid);
            if (result != null) {
                readedFromCache = true;
            }
        }
        // create or update not commited for this session
        if (lockManager.containsLockedValue(floid, sessionId)) {
            if (!readedFromCache) {
                result = lockManager.getLockedValue(floid, sessionId);
            }
            update = lockManager.getUpdate(floid, sessionId);
        // object not modified by current session, read it in base
        } else {
            try {
                if (!readedFromCache) {
                    PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
                    if (fpoid != null) {
                        result = fileAccess.readDatas(sessionId, fpoid);
                    }
                }
                update = oidTable.getUpdate(sessionId, floid, true);
            } catch (IdentityException jalistoIe) {
                lockManager.finishReadObjectByOid(floid, null, (short) -1);
                throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore", jalistoIe);
            }
        }

        if (result == null) {
            lockManager.finishReadObjectByOid(floid, null, (short) -1);
            throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore");
        }

        if (withCache && readedFromCache) {
            cache.put(floid, result);
        }

        lockManager.finishReadObjectByOid(floid, result, update);
        return result;
    }

    public Object[] refreshObjectByOid(Object oid) {
        trace.println(Trace.SESSION, "{0} refreshObjectByOid({1})", sessionId, oid);
        checkValidity("refreshObjectByOid", true);
        LogicalOid floid = getFloidFromOid(oid);

        if (!lockManager.containsLockedValue(floid, sessionId)) {
            return readObjectByOidInDataObject(oid, true).getDatas();
        }

        // delete not commited for this session
        if (lockManager.containsDeletedValue(floid, sessionId)) {
            throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore");
        }

        DataWrapper result = null;
        short update = lockManager.getUpdate(floid, sessionId);
        try {
            PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
            if (fpoid != null) {
                result = fileAccess.readDatas(sessionId, fpoid);
                update = oidTable.getUpdate(sessionId, floid, false);
                oidTable.setUpdate(sessionId, floid, update);
            }
        } catch (IdentityException jalistoIe) {
            throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore", jalistoIe);
        }
        if (result == null) {
            throw new IdentityException("the given oid " + oid + " doesn't exist in this Jalisto datastore");
        }

        lockManager.setLockedValue(floid, sessionId, result, update);
        cache.put(floid, result);

        return result.getDatas();
    }

    public Object updateObjectByOid(Object oid, Object[] objectToUpdate) {
        trace.println(Trace.SESSION, "{0} updateObjectByOid({1})", sessionId, oid);
        checkValidity("updateObjectByOid", true);
        checkSerialisable(objectToUpdate);
        LogicalOid floid = getFloidFromOid(oid);
        short update;
        DataWrapper datas;
        if (lockManager.containsLockedValue(floid, sessionId)) {
            datas = lockManager.getLockedValue(floid, sessionId);
            update = lockManager.getUpdate(floid, sessionId);
        } else {
            datas = JalistoFactory.getInternalFactory().getNewDataWrapper(props, objectToUpdate);
            update = oidTable.getUpdate(sessionId, floid, true);
        }
        datas.setDatas(objectToUpdate);
        cache.put(floid, datas);
        lockManager.finishUpdateObjectByOid(floid, datas, update);
        return floid.getClone();
    }

    public void updateObjectCommit(LogicalOid floid) {
        trace.println(Trace.SESSION, "{0} updateObjectCommit({1})", sessionId, floid);
        boolean findInCache = true;
        DataWrapper datas = (DataWrapper) cache.get(floid);
        if (datas == null) {
            findInCache = false;
            datas = lockManager.getLockedValue(floid, sessionId);
        }
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        if (fpoid != null) {
            PhysicalOid updatedFpoid = fileAccess.updateDatas(sessionId, fpoid, datas);
            if (!updatedFpoid.equals(fpoid)) {
                oidTable.updatePoid(sessionId, floid, updatedFpoid);
            }
        } else {
            throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
        }
        if (!findInCache) {
            cache.put(floid, datas);
        }
    }

    public void updateObjectRollback(LogicalOid floid) {
        trace.println(Trace.SESSION, "{0} updateObjectRollback({1})", sessionId, floid);
        if (cache.containsKey(floid)) {
            cache.remove(floid);
        }
    }

    public void deleteObjectByOid(Object oid) {
        trace.println(Trace.SESSION, "{0} deleteObjectByOid({1})", sessionId, oid);
        checkValidity("deleteObjectByOid", true);
        LogicalOid floid = getFloidFromOid(oid);

        short update;
        if (lockManager.containsLockedValue(floid, sessionId)) {
            update = lockManager.getUpdate(floid, sessionId);
        } else {
            update = oidTable.getUpdate(sessionId, floid, true);
        }
        if (update == -1) {
            throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
        }

        lockManager.prepareDeleteObjectByOid(floid, update);

        if (cache.containsKey(floid)) {
            cache.remove(floid);
        }
        oidTable.removeFloid(sessionId, floid);
        lockManager.finishDeleteObjectByOid(floid);
    }

    public void deleteObjectCommit(LogicalOid floid) {
        trace.println(Trace.SESSION, "{0} deleteObjectCommit({1})", sessionId, floid);
        PhysicalOid fpoid = oidTable.getDeletedFpoid(sessionId, floid);
        if (fpoid != null) {
            fileAccess.deleteDatas(sessionId, fpoid);
        }
    }

    public void deleteObjectRollback(LogicalOid floid) {
        // nothing to do here
    }


    public void begin() {
        trace.println(Trace.SESSION, "{0} begin()", sessionId);
        checkIsOpen("begin");
        cache.clear();
        lockManager.begin();
        fileAccess.begin(sessionId);
    }

    public void commit() {
        trace.println(Trace.SESSION, "{0} commit()", sessionId);
        checkIsOpen("commit");
        lockManager.commit();
    }

    public void rollback() {
        trace.println(Trace.SESSION, "{0} rollback()", sessionId);
        checkIsOpen("rollback");
        lockManager.rollback();
    }

    public LockManager getLockManager() {
        return lockManager;
    }

    public AccessController getAccessController() {
        return accessController;
    }

    public void setOptimistic() {
        trace.println(Trace.SESSION, "{0} setOptimistic()", sessionId);
        if (isOpen) {
            throw new TransactionException("Session must not be open during setOptimistic");
        }
        if (transaction.isActive()) {
            throw new TransactionException("Transaction must not be active during setOptimistic");
        }
        lockManager = new LockManagerOptimistic();
        lockManager.setLockTable(lockTable);
        lockManager.setSession(this);
    }

    public void setPessimistic() {
        trace.println(Trace.SESSION, "{0} setPessimistic()", sessionId);
        if (isOpen) {
            throw new TransactionException("Session must not be open during setPessimistic");
        }
        if (transaction.isActive()) {
            throw new TransactionException("Transaction must not be active during setPessimistic");
        }
        lockManager = new LockManagerPessimistic();
        lockManager.setLockTable(lockTable);
        lockManager.setSession(this);
    }


    public void goToBed() {
        trace.println(Trace.SESSION, "{0} goToBed()", sessionId);
        sessionIsSleeping = true;
    }

    public void inspectLocks() {
        lockTable.publishLocks();
    }


    public void sleep(LogicalOid floid) {
        sleep(floid, "");
    }

    public void sleep(Object floid, String message) {
        trace.println(Trace.SESSION, "{0} sleeps()", sessionId);
        trace.println(Trace.DEBUG, "{0} sleeps()", sessionId);
        if (sessionIsSleeping) {
            int i = 0;
            while (sessionIsSleeping && (i < timeOut)) {
                Thread.yield();
                i++;
            }
            if (i == timeOut) {
                sessionIsSleeping = false;
                lockTable.getLock(floid).resolveTimeOut(sessionId);
            }
        }
        trace.println(Trace.DEBUG, "{0} awakes", sessionId);
    }

    public void wakeUp() {
        trace.println(Trace.SESSION, "{0} wakeUp()", sessionId);
        sessionIsSleeping = false;
    }


    private LockManager lockManager;
    private LockTable lockTable;
    private AccessController accessController;
    private volatile boolean sessionIsSleeping = false;
    private int timeOut;
}
