/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.readonly;

import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.JalistoFactory;

import java.util.Collection;
import java.util.Map;

public class InTransactionBaseImageReadOnly implements InTransactionBaseImage {

    public InTransactionBaseImageReadOnly(PluggablePhysicalFileAccess physicalAccess, JalistoProperties properties) {
        this.readObjects = JalistoFactory.getInternalFactory().getCache(properties,
                                                          properties.getPageCacheSize(),
                                                          "inTransactionBaseImageCache");
        this.physicalAccess = physicalAccess;
        this.keepInMemory = properties.isKeepingInMemory();
    }

    public boolean isWorking() {
        return false;
    }

    public boolean startWorking() {
        return true;
    }

    public void stopWorking() {
    }

    public void begin() {
    }

    public synchronized void commit() {
        if (!keepInMemory) {
            readObjects.clear();
        }
    }

    public synchronized void rollback() {
        if (!keepInMemory) {
            readObjects.clear();
        }
    }

    public synchronized JalistoObject readFileObjectAt(InFileAddress ifa) {
        String address = ifa.getAddress();
        JalistoObject result = (JalistoObject) readObjects.get(address);
        if (result != null)
            return result;
        result = physicalAccess.readFileObjectAt(ifa);
        readObjects.put(address, result);
        return result;
    }

    public Map getReadObjects() {
        return readObjects;
    }

    public boolean isKeepInMemory() {
        return keepInMemory;
    }

    public void setKeepInMemory(boolean keepInMemory) {
        this.keepInMemory = keepInMemory;
    }


    public void deleteFileObject(InFileAddress ifa) {
        throw new UnsupportedOperationException();
    }

    public int getDeletedObjectsSize() {
        throw new UnsupportedOperationException();
    }

    public int getModifiedObjectsSize() {
        throw new UnsupportedOperationException();
    }

    public int getNewObjectsSize() {
        throw new UnsupportedOperationException();
    }

    public void insertFileObject(InFileAddress ifa, JalistoObject fo) {
        throw new UnsupportedOperationException();
    }

    public void updateFileObject(InFileAddress ifa, JalistoObject fo) {
        throw new UnsupportedOperationException();
    }

    public Collection getKeysStartingWith(String filter, boolean withOrder) {
        return physicalAccess.getKeysStartingWith(filter);
    }

    private Map readObjects;
    private PluggablePhysicalFileAccess physicalAccess;
    private boolean keepInMemory;
}
