/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.readonly;

import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.util.Collection;

public class InternalPhysicalFileAccessReadOnlyImpl implements InternalPhysicalFileAccess {

    public InternalPhysicalFileAccessReadOnlyImpl(JalistoProperties properties) {
        InternalFactory internalFactory = JalistoFactory.getInternalFactory();
        this.tracer = internalFactory.getTracer(properties);
        this.internalAccess = internalFactory.getPhysicalAccess(properties);
        this.baseImage = new InTransactionBaseImageReadOnly(internalAccess, properties);
    }


    public boolean isNewBase() {
        return internalAccess.isNewBase();
    }


    public void insertFileObject(JalistoObject fo) {
        throw new UnsupportedOperationException();
    }

    public JalistoObject readFileObjectAt(InFileAddress ifa) {
        tracer.println(Trace.PHYSICAL, "readFileObjectAt({0})", ifa);
        try {
            return baseImage.readFileObjectAt(ifa);
        } catch (Exception e) {
            throw new JalistoException("could not read object at " + ifa, e);
        }
    }

    public void updateFileObject(JalistoObject fo) {
        throw new UnsupportedOperationException();
    }

    public void deleteFileObject(InFileAddress ifa) {
        throw new UnsupportedOperationException();
    }

    public void writeObjectInBase(InFileAddress ifa, Object o, boolean isUpdate) {
        throw new UnsupportedOperationException();
    }


    public PluggablePhysicalFileAccess getInternalAccess() {
        return internalAccess;
    }


    public void begin() {
        tracer.println(Trace.PHYSICAL, "begin()");
    }

    public void commit() {
        tracer.println(Trace.PHYSICAL, "commit()");
        baseImage.commit();
    }

    public void rollback() {
        tracer.println(Trace.PHYSICAL, "rollback()");
        baseImage.rollback();
    }

    public void close() {
    }

    public void open() {
    }

    public Collection getKeysStartingWith(String filter, boolean withOrder) {
        return baseImage.getKeysStartingWith(filter, withOrder);
    }


    public InTransactionBaseImage getBaseImage() {
        return baseImage;
    }

    public String toString() {
        return internalAccess.toString();
    }


    private InTransactionBaseImage baseImage;
    private PluggablePhysicalFileAccess internalAccess;
    private Trace tracer;


}
