/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.readonly;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.impl.server.IfaIndex;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.server.PhysicalOid;
import org.objectweb.jalisto.se.impl.server.page.*;
import org.objectweb.jalisto.se.impl.server.page.info.PageInfo;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.JalistoFactory;

public class LogicalPageAccessReadOnlyImpl implements LogicalSystemPageAccess {

    public LogicalPageAccessReadOnlyImpl(JalistoProperties properties) {
        InternalFactory internalFactory = JalistoFactory.getInternalFactory();
        tracer = internalFactory.getTracer(properties);
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : create new instance");
        physicalAccess = internalFactory.getInternalPhysicalAccess(properties);
        ifaIndex = IfaIndex.getAnIfaIndex(physicalAccess, properties);
    }

    public String toString() {
        return physicalAccess.toString();
    }

    public InternalPhysicalFileAccess getPhysicalAccess() {
        return physicalAccess;
    }

    public void begin(Object sessionId) {
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : begin() : nothing to do");
    }

    public void commit(Object sessionId) {
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : commit()");
        physicalAccess.commit();
    }

    public void rollback(Object sessionId) {
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : rollback()");
        physicalAccess.rollback();
    }

    public void cleanAllNullValue() {
        throw new UnsupportedOperationException();
    }

    /**
     * ********************************  INSTANCE  *********************************************
     */

    public PhysicalOid allocateNewFpoid(Object sessionId, Object clid) {
        throw new UnsupportedOperationException();
    }

    public void desallocateFpoid(Object sessionId, PhysicalOid fpoid) {
        throw new UnsupportedOperationException();
    }

    public void insertDatas(Object sessionId, PhysicalOid fpoid, DataWrapper datas) {
        throw new UnsupportedOperationException();
    }

    public DataWrapper readDatas(Object sessionId, PhysicalOid fpoid) {
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : readDatas({0})", fpoid);
        InFileAddress ifa = getInstanceIfa(fpoid);
        InstancePage dataPage = (InstancePage) physicalAccess.readFileObjectAt(ifa);
        return (DataWrapper) dataPage.getDataAt(ifa);
    }

    public PhysicalOid updateDatas(Object sessionId, PhysicalOid fpoid, DataWrapper datas) {
        throw new UnsupportedOperationException();
    }

    public void deleteDatas(Object sessionId, PhysicalOid fpoid) {
        throw new UnsupportedOperationException();
    }


    /**
     * ************************************  INDEX  ************************************************
     */

    public InFileAddress allocateNodeAddressAndInsert(Object sessionId, FieldDescription meta, DataWrapper node) {
        throw new UnsupportedOperationException();
    }

    public Object readNode(Object sessionId, InFileAddress ifa) {
        NodePage page = (NodePage) physicalAccess.readFileObjectAt(ifa);
        return page.getDataAt(ifa);
    }

    public void updateNode(Object sessionId, InFileAddress ifa, DataWrapper node) {
        throw new UnsupportedOperationException();
    }

    public void removeNode(Object sessionId, InFileAddress ifa) {
        throw new UnsupportedOperationException();
    }

    public InFileAddress allocateLeafAddressAndInsert(Object sessionId, FieldDescription meta, DataWrapper oids) {
        throw new UnsupportedOperationException();
    }

    public Object readLeaf(Object sessionId, InFileAddress ifa) {
        LeafPage page = (LeafPage) physicalAccess.readFileObjectAt(ifa);
        return page.getDataAt(ifa);
    }

    public void updateLeaf(Object sessionId, InFileAddress ifa, DataWrapper oids) {
        throw new UnsupportedOperationException();
    }

    public void removeLeaf(Object sessionId, InFileAddress ifa) {
        throw new UnsupportedOperationException();
    }

    /**
     * ********************************     PRIVATE     ********************************************
     */

    private InFileAddress getInstanceIfa(PhysicalOid fpoid) {
        InFileAddress systemIfa = ifaIndex.getPidIfa(fpoid.getPid());
        SystemPage systemPage = (SystemPage) physicalAccess.readFileObjectAt(systemIfa);
        PageInfo infos = (PageInfo) systemPage.getDataAt(systemIfa);
        InFileAddress instanceIfa = infos.getPageIfa();
        instanceIfa.setIndex(fpoid.getIidAsInteger().intValue());
        return instanceIfa;
    }

    public void updateIndexObject(InFileAddress ifa, DataWrapper indexObject) {
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : updateIndexObject({0}, {1})", ifa, indexObject);
        UserPage page = (UserPage) physicalAccess.readFileObjectAt(ifa);
        page.setDataAt(ifa, indexObject);
        physicalAccess.updateFileObject(page);
    }

    private IfaIndex ifaIndex;
    private InternalPhysicalFileAccess physicalAccess;


    private Trace tracer;
}
