/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.readonly;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.OidTable;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.*;
import org.objectweb.jalisto.se.impl.server.IdentityProvider;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.server.PhysicalOid;
import org.objectweb.jalisto.se.impl.server.oid.OidInfo;
import org.objectweb.jalisto.se.impl.server.page.OidPage;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.JalistoFactory;

import java.util.*;

public class OidTableReadOnlyImpl implements OidTable {

    private OidTableReadOnlyImpl(InternalPhysicalFileAccess physicalAccess, JalistoProperties properties) {
        tracer = JalistoFactory.getInternalFactory().getTracer(properties);
        atSize = properties.getOidTableSize();
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : create new instance with size = {0}",
                       new Integer(atSize));

        floidTable = JalistoFactory.getInternalFactory().getCache(properties, atSize, "atCache");
        this.physicalAccess = physicalAccess;
    }

    public LogicalOid allocateNewFloid(short clid) {
        throw new UnsupportedOperationException();
    }

    public void insertFpoid(Object sessionId, LogicalOid floid, PhysicalOid fpoid) {
        throw new UnsupportedOperationException();
    }

    public void updatePoid(Object sessionId, LogicalOid floid, PhysicalOid newFpoid) {
        throw new UnsupportedOperationException();
    }

    public void removeFloid(Object sessionId, LogicalOid floid) {
        throw new UnsupportedOperationException();
    }

    public void markAsCreated(Object sessionId, LogicalOid floid) {
        throw new UnsupportedOperationException();
    }


    public Collection getFloidsFromClid(Object sessionId, Object clid, boolean fullyTransactionnal) {
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : getFloidsFromClid({0})", clid);
        Collection workingExtent = new ArrayList();
        Iterator oidIfaList = physicalAccess.getKeysStartingWith(IFA_PREFIXE + clid, true).iterator();
        while (oidIfaList.hasNext()) {
            InFileAddress ifa = new InFileAddress((String) oidIfaList.next());
            ifa.setFileIndex(JalistoProperties.OID_INDEX);
            OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(ifa);
            Object[] infos = oidPage.getAllDatas();
            for (int j = 0; j < infos.length; j++) {
                OidInfo info = (OidInfo) infos[j];
                if (info != null) {
                    if (info.getFpoid().hasClidEqualsWith(clid)) {
                        workingExtent.add(info.getFloid());
                    }
                }
            }
        }
        return workingExtent;
    }

    public Collection getFloidsFromClid(Object sessionId, Object clid, ExtentImpl extent, int number) {
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : getFloidsFromClid({0})", clid);
        Collection workingExtent = new ArrayList();
        boolean continueSearching = true;
        String startingAddress = extent.getStartingAddress();
        LogicalOid lastFloid = extent.getLastFloid();
        String oidPageAddress = null;
        Iterator oidIfaList = physicalAccess.getKeysStartingWith(IFA_PREFIXE + clid, true).iterator();
        while (oidIfaList.hasNext() && continueSearching) {
            oidPageAddress = (String) oidIfaList.next();
            if (oidPageAddress.compareTo(startingAddress) >= 0) {
                InFileAddress ifa = new InFileAddress(oidPageAddress);
                ifa.setFileIndex(JalistoProperties.OID_INDEX);
                OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(ifa);
                Object[] infos = oidPage.getAllDatas();
                for (int j = 0; (j < infos.length) && continueSearching; j++) {
                    OidInfo info = (OidInfo) infos[j];
                    if (info != null) {
                        LogicalOid floid = info.getFloid();
                        if (info.getFpoid().hasClidEqualsWith(clid)) {
                            if ((lastFloid == null) || (floid.compareTo(lastFloid) > 0)) {
                                workingExtent.add(floid);
                            }
                        }
                    }
                    if (workingExtent.size() >= number) {
                        continueSearching = false;
                    }
                }
            }
        }
        extent.setStartingAddress(oidPageAddress);
        return workingExtent;
    }

    public synchronized PhysicalOid getFpoid(Object sessionId, LogicalOid floid) {
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : getFpoid({0})", floid);
        if (!floidTable.containsKey(floid)) {
            if (!loadOidPage(floid)) {
                return null;
            }
        }
        OidInfo oidInfo = (OidInfo) floidTable.get(floid);
        if (oidInfo != null) {
            return oidInfo.getFpoid();
        } else {
            return null;
        }
    }

    public boolean containsFloid(Object sessionId, LogicalOid floid) {
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : containsFloid({0})", floid);
        if (!floidTable.containsKey(floid)) {
            return (!loadOidPage(floid));
        }
        OidInfo oidInfo = (OidInfo) floidTable.get(floid);
        if (oidInfo != null) {
            return (!oidInfo.getActionCaller().isDelete());
        } else {
            return false;
        }
    }

    public PhysicalOid getDeletedFpoid(Object sessionId, LogicalOid floid) {
        throw new UnsupportedOperationException();
    }

    public void setUpdate(Object sessionId, LogicalOid floid, short value) {
        throw new UnsupportedOperationException("should not be used in this implementation");
    }

    public short getUpdate(Object sessionId, LogicalOid floid, boolean transactionnal) {
        throw new UnsupportedOperationException("should not be used in this implementation");
    }

    public synchronized void checkStateFromBase(Object sessionId, JalistoProperties properties) {
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : getStateFromBase()");
        floidTable = JalistoFactory.getInternalFactory().getCache(properties, atSize, "atCache");
        readAtTable();
    }

    public Map getOidTable() {
        return floidTable;
    }


    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("OidTableReadOnlyImpl : \n");
        sb.append("\tfloid list : ").append(floidTable).append("\n");
        return sb.toString();
    }


    /**
     * *********************************    protected    **********************************************
     */

    private synchronized boolean loadOidPage(LogicalOid floid) {
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : loadOidPage({0})", floid);
//        int index = floid.getFloidAsInt() % oidPageSize;
//        int pageNumber = floid.getFloidAsInt() / oidPageSize;
//        InFileAddress atIfa = new InFileAddress(IFA_PREFIXE + String.valueOf(pageNumber));
//        atIfa.setIndex(index);
//        atIfa.setFileIndex(JalistoProperties.OID_INDEX);
//        try {
//            // could raise exception if oidPage don't exist
//            OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(atIfa);
//            Object[] infos = oidPage.getAllDatas();
//            for (int j = 0; j < infos.length; j++) {
//                OidInfo info = (OidInfo) infos[j];
//                if (info != null) {
//                    floidTable.put(info.getFloid(), info);
//                }
//            }
//            return true;
//        } catch (Exception e) {
//            trace.println(Trace.OIDTABLE, "OidTableImplPage : loadOidPage : oidPage doesn't exist");
//        }
        return false;
    }

    private synchronized void readAtTable() {
        tracer.println(Trace.OIDTABLE, "OidTableReadOnlyImpl : readAtTable()");
        int dataRead = 0;
        Iterator oidIfaList = physicalAccess.getKeysStartingWith(IFA_PREFIXE, true).iterator();
        while (oidIfaList.hasNext()) {
            InFileAddress ifa = new InFileAddress((String) oidIfaList.next());
            ifa.setFileIndex(JalistoProperties.OID_INDEX);
            OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(ifa);
            Object[] infos = oidPage.getAllDatas();
            for (int i = 0; i < infos.length; i++) {
                OidInfo info = (OidInfo) infos[i];
                if (info != null) {
                    floidTable.put(info.getFloid(), info);
                    dataRead++;
                    if (dataRead == atSize) {
                        return;
                    }
                }
            }
        }
    }

    public void begin(Object sessionId) {
    }

    public void commit(Object sessionId) {
    }

    public void rollback(Object sessionId) {
    }

    public void close(Object sessionId) {
    }

    public void open(Object sessionId) {
    }

    public void deleteOidInfoInBase(LogicalOid floid) {
    }

    public void insertOidInfoInBase(OidInfo info) {
    }

    public void updateOidInfoInBase(OidInfo info) {
    }

    public IdentityProvider getIdentityProvider() {
        return null;
    }


    private InternalPhysicalFileAccess physicalAccess;

    private Map floidTable;

    private int atSize;

    private Trace tracer;


    /**
     * ************************************    STATIC    *********************************************
     */

    public static OidTable getAnOidTable(InternalPhysicalFileAccess physicalAccess, JalistoProperties properties) {
        OidTableReadOnlyImpl oidTable;
        try {
            oidTable = new OidTableReadOnlyImpl(physicalAccess, properties);
            oidTable.readAtTable();
        } catch (Exception e) {
            throw new JalistoException("cannot create a base in read only mode", e);
        }
        return oidTable;
    }

    private static final String IFA_PREFIXE = "id";
}
