/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.server;

import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.InFileAddress;

public class IdentityProvider {

    private IdentityProvider(InternalPhysicalFileAccess physicalAccess) {
        floids = new long[0];
        clid = -1;
        pid = 0;
        this.physicalAccess = physicalAccess;
    }

    private IdentityProvider(Object[] table) {
        floids = (long[]) table[0];
        clid = ((Short) table[1]).shortValue();
        pid = ((Integer) table[2]).intValue();
    }

    public Object getNewPid() {
        pid++;
        Integer result = new Integer(pid);
        return result;
    }

    public Object getNewClid() {
        clid++;
        if (floids.length < clid + 1) {
            long[] newFloids = new long[clid + 1];
            System.arraycopy(floids, 0, newFloids, 0, floids.length);
            floids = newFloids;
        }

        floids[clid] = 0;
        Short result = new Short(clid);
        return result;
    }

    public LogicalOid allocateNewFloid(short clid) {
        LogicalOid result = new LogicalOid(floids[clid]++ + LogicalOid.classMulti * clid);
        return result;
    }

    public synchronized long reserveFloids(short clid, int number) {
        long result = floids[clid];
        floids[clid] = floids[clid] + number;
        return result;
    }

    public void commit() {
        updateStateInBase(true);
    }

    private synchronized void updateStateInBase(boolean isUpdate) {
        Object[] datasToSave = new Object[3];
        datasToSave[0] = floids;
        datasToSave[1] = new Short(clid);
        datasToSave[2] = new Integer(pid);
        physicalAccess.writeObjectInBase(OID_PROVIDER_IFA, datasToSave, isUpdate);
    }

    public long[] getFloidCounters() {
        return floids;
    }

    public short getClidCounter() {
        return clid;
    }

    public void resetFloidCounters() {
        for (int i = 0; i < floids.length; i++) {
            floids[i] = 0;
        }
    }

    public void resetClidCounter() {
        clid = -1;
    }

    public void resetAll() {
        resetClidCounter();
        resetFloidCounters();
        updateStateInBase(true);
    }

    public long getCurrentFloidCounter(short clid) {
        return floids[clid];
    }


    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("FileIdentityProviderImplPage : \n");
        sb.append("\tfloids : ").append(floids).append("\n");
        sb.append("\tclid : ").append(clid).append("\n");
        sb.append("\tpid : ").append(pid).append("\n");
        return sb.toString();
    }

    /**
     * *****************************    STATIC    **************************************
     */

    public static IdentityProvider getAnIdentityProvider(InternalPhysicalFileAccess physicalAccess) {
        IdentityProvider oidProvider;
        try {
            Object[] datas = (Object[]) physicalAccess.readFileObjectAt(OID_PROVIDER_IFA).getData();
            oidProvider = new IdentityProvider(datas);
            oidProvider.physicalAccess = physicalAccess;
        } catch (Exception e) {
            oidProvider = new IdentityProvider(physicalAccess);
            oidProvider.updateStateInBase(false);
        }
        return oidProvider;
    }


    private transient InternalPhysicalFileAccess physicalAccess;
    private long[] floids;
    private short clid;
    private int pid;

    private static final InFileAddress OID_PROVIDER_IFA = new InFileAddress("ipi");
}
