/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.server;

import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.impl.server.page.SystemPage;
import org.objectweb.jalisto.se.impl.InFileAddress;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Map;

public class IfaIndex {

    private IfaIndex(InternalPhysicalFileAccess physicalAccess) {
        this.physicalAccess = physicalAccess;
        pidIndex = new Hashtable();
        systemPageIndex = 0;
        instancePageIndex = 0;
        nodePageIndex = 0;
        leafPageIndex = 0;
    }

    public Collection getAllPid() {
        return pidIndex.keySet();
    }

    public Collection getAllPidIfa() {
        return pidIndex.values();
    }

    public void setCurrentAvalaibleSystemPage(InFileAddress currentAvalaibleSystemPage) {
        this.currentAvalaibleSystemPage = currentAvalaibleSystemPage.getClone();
        updateStateInBase(true);
    }

    public InFileAddress getCurrentAvalaibleSystemPage() {
        return currentAvalaibleSystemPage;
    }

    public String getNewInstancePageAddress() {
        return "ip" + (instancePageIndex++);
    }

    public String getNewSystemPageAddress() {
        return "sp" + (systemPageIndex++);
    }

    public String getNewNodePageAddress() {
        return "no" + (nodePageIndex++);
    }

    public String getNewLeafPageAddress() {
        return "le" + (leafPageIndex++);
    }

    public InFileAddress getPidIfa(Object pid) {
        return (InFileAddress) pidIndex.get(pid);
    }

    public void putPidIfa(Object pid, InFileAddress ifa) {
        pidIndex.put(pid, ifa);
        physicalAccess.writeObjectInBase(OID_IFA_PID_INDEX_IFA, new Hashtable(pidIndex), true);
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("FileIndexImplBasic2 : \n");
        sb.append("\tpid index : ").append(pidIndex).append("\n");
        sb.append("\tsystemPageIndex : ").append(systemPageIndex).append("\n");
        sb.append("\tinstancePageIndex : ").append(instancePageIndex).append("\n");
        sb.append("\tcurrentAvalaibleSystemPage : ").append(currentAvalaibleSystemPage).append("\n");
        return sb.toString();
    }

    private void init(short systemPageSize) {
        currentAvalaibleSystemPage = new InFileAddress(getNewSystemPageAddress());
        SystemPage firstSystemPage = new SystemPage(systemPageSize);
        firstSystemPage.setIfa(currentAvalaibleSystemPage);
        physicalAccess.insertFileObject(firstSystemPage);
        updateStateInBase(false);
        physicalAccess.writeObjectInBase(OID_IFA_PID_INDEX_IFA, new Hashtable(pidIndex), false);
    }

    public void getStateFromBase() {
        pidIndex = (Map) physicalAccess.readFileObjectAt(OID_IFA_PID_INDEX_IFA).getData();
        Object[] datas = (Object[]) physicalAccess.readFileObjectAt(OID_IFAINDEX_IFA).getData();
        currentAvalaibleSystemPage = (InFileAddress) datas[0];
        systemPageIndex = ((Integer) datas[1]).intValue();
        instancePageIndex = ((Integer) datas[2]).intValue();
        nodePageIndex = ((Integer) datas[3]).intValue();
        leafPageIndex = ((Integer) datas[4]).intValue();
    }

    public void updateStateInBase(boolean isUpdate) {
        Object[] datas = new Object[5];
        datas[0] = currentAvalaibleSystemPage;
        datas[1] = new Integer(systemPageIndex);
        datas[2] = new Integer(instancePageIndex);
        datas[3] = new Integer(nodePageIndex);
        datas[4] = new Integer(leafPageIndex);
        physicalAccess.writeObjectInBase(OID_IFAINDEX_IFA, datas, isUpdate);
    }

    /**
     * *******************************     STATIC     ***************************************
     */

    public static IfaIndex getAnIfaIndex(InternalPhysicalFileAccess physicalAccess, JalistoProperties properties) {
        IfaIndex ifaIndex = new IfaIndex(physicalAccess);
        try {
            ifaIndex.getStateFromBase();
        } catch (Exception e) {
            ifaIndex.init(properties.getSystemPageSize());
        }
        return ifaIndex;
    }


    private InternalPhysicalFileAccess physicalAccess;
    private InFileAddress currentAvalaibleSystemPage;

    private Map pidIndex;      //   pid <-> ifa de la pageInfo corresponadnte

    private int systemPageIndex;
    private int instancePageIndex;
    private int nodePageIndex;
    private int leafPageIndex;


    private static final InFileAddress OID_IFAINDEX_IFA = new InFileAddress("IFAIst");
    private static final InFileAddress OID_IFA_PID_INDEX_IFA = new InFileAddress("pids");
}
