/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.server;

import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.JalistoFactory;

import java.util.Collection;

public class InternalPhysicalFileAccessImpl implements InternalPhysicalFileAccess {

    public InternalPhysicalFileAccessImpl(JalistoProperties properties) {
        InternalFactory internalFactory = JalistoFactory.getInternalFactory();
        this.tracer = internalFactory.getTracer(properties);
        this.internalAccess = internalFactory.getPhysicalAccess(properties);
        this.baseImage = internalFactory.getAInTransactionBaseImage(internalAccess, properties);
        this.sessionCounter = 0;
    }


    public boolean isNewBase() {
        return internalAccess.isNewBase();
    }


    public void insertFileObject(JalistoObject fo) {
        tracer.println(Trace.PHYSICAL, "insertFileObject({0})", fo);
        try {
            baseImage.insertFileObject(fo.getIfa(), fo);
        } catch (Exception e) {
            throw new JalistoException("could not insert object " + fo, e);
        }
    }

    public JalistoObject readFileObjectAt(InFileAddress ifa) {
        tracer.println(Trace.PHYSICAL, "readFileObjectAt({0})", ifa);
        try {
            return baseImage.readFileObjectAt(ifa);
        } catch (Exception e) {
            throw new JalistoException("could not read object at " + ifa, e);
        }
    }

    public void updateFileObject(JalistoObject fo) {
        tracer.println(Trace.PHYSICAL, "updateFileObject({0})", fo);
        try {
            baseImage.updateFileObject(fo.getIfa(), fo);
        } catch (Exception e) {
            throw new JalistoException("could not update object " + fo, e);
        }
    }

    public void deleteFileObject(InFileAddress ifa) {
        tracer.println(Trace.PHYSICAL, "deleteFileObject({0})", ifa);
        try {
            baseImage.deleteFileObject(ifa);
        } catch (Exception e) {
            throw new JalistoException("could not delete object at " + ifa, e);
        }
    }

    public void writeObjectInBase(InFileAddress ifa, Object o, boolean isUpdate) {
        JalistoObject objectInBase = new JalistoObjectImpl();
        objectInBase.setIfa(ifa);
        objectInBase.setData(o);
        if (isUpdate) {
            updateFileObject(objectInBase);
        } else {
            insertFileObject(objectInBase);
        }
    }


    public PluggablePhysicalFileAccess getInternalAccess() {
        return internalAccess;
    }


    public void begin() {
        tracer.println(Trace.PHYSICAL, "begin()");
    }

    public void commit() {
        tracer.println(Trace.PHYSICAL, "commit()");
        baseImage.commit();
    }

    public void rollback() {
        tracer.println(Trace.PHYSICAL, "rollback()");
        baseImage.rollback();
    }

    public void open() {
        if (sessionCounter == 0) {
            internalAccess.open();
        }
        sessionCounter++;
    }

    public void close() {
        if (sessionCounter > 0) {
            sessionCounter--;
        } else {
            throw new JalistoException("invalid close call : sessionCounter = " + sessionCounter);
        }
        if (sessionCounter == 0) {
            internalAccess.close();
        }
    }

    public Collection getKeysStartingWith(String filter, boolean withOrder) {
        return baseImage.getKeysStartingWith(filter, withOrder);
    }


    public InTransactionBaseImage getBaseImage() {
        return baseImage;
    }

    public String toString() {
        return internalAccess.toString();
    }

    private InTransactionBaseImage baseImage;
    private PluggablePhysicalFileAccess internalAccess;
    private int sessionCounter;
    private Trace tracer;


}
