/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.server;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.server.page.*;
import org.objectweb.jalisto.se.impl.server.page.info.InMemoryPageInfo;
import org.objectweb.jalisto.se.impl.server.page.info.IndexPageInfo;
import org.objectweb.jalisto.se.impl.server.page.info.PageInfo;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.util.*;

public class LogicalPageAccessImpl implements LogicalSystemPageAccess {

    public LogicalPageAccessImpl(JalistoProperties properties) {
        InternalFactory internalFactory = JalistoFactory.getInternalFactory();
        tracer = internalFactory.getTracer(properties);
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : create new instance");
        physicalAccess = internalFactory.getInternalPhysicalAccess(properties);
        ifaIndex = IfaIndex.getAnIfaIndex(physicalAccess, properties);
        oidProvider = internalFactory.getIdentityProvider(properties);
        workingSessions = new ArrayList();
        instancePageSize = properties.getInstancePageSize();
        systemPageSize = properties.getSystemPageSize();
        instancePageNumber = properties.getInstanceDbFileNumber();
        isMonoImplementation = properties.isMonoImplementation();
        getInMemoryInfos();
    }

    public String toString() {
        return physicalAccess.toString();
    }

    public InternalPhysicalFileAccess getPhysicalAccess() {
        return physicalAccess;
    }

    public synchronized void begin(Object sessionId) {
        tracer.println(Trace.LOGICAL, "{0} : LogicalPageAccessImpl : begin() : nothing to do", sessionId);
        workingSessions.add(sessionId);
        physicalAccess.begin();
        beginInfos(instancePageInfos, sessionId);
        beginIndexInfos(leafPageInfos, sessionId);
        beginIndexInfos(nodePageInfos, sessionId);
    }

    private void beginInfos(Map infos, Object sessionId) {
        synchronized (infos) {
            Iterator values = infos.values().iterator();
            while (values.hasNext()) {
                InMemoryPageInfo memoryInfo = (InMemoryPageInfo) values.next();
                memoryInfo.begin(sessionId);
            }
        }
    }

    private void beginIndexInfos(Map infos, Object sessionId) {
        synchronized (infos) {
            Iterator mapByClasse = infos.values().iterator();
            while (mapByClasse.hasNext()) {
                Iterator values = ((Map) mapByClasse.next()).values().iterator();
                while (values.hasNext()) {
                    InMemoryPageInfo memoryInfo = (InMemoryPageInfo) values.next();
                    memoryInfo.begin(sessionId);
                }
            }
        }
    }

    public synchronized void commit(Object sessionId) {
        tracer.println(Trace.LOGICAL, "{0} : LogicalPageAccessImpl : commit()", sessionId);
        workingSessions.remove(sessionId);
        ifaIndex.updateStateInBase(true);
        physicalAccess.commit();
    }

    public synchronized void rollback(Object sessionId) {
        tracer.println(Trace.LOGICAL, "{0} : LogicalPageAccessImpl : rollback()", sessionId);
        workingSessions.remove(sessionId);
        if (isMonoImplementation) {
            physicalAccess.rollback();
            ifaIndex.getStateFromBase();
        } else {
            // we have to commit changes done by unitary rollbacking operations from lockmanager and session
            physicalAccess.commit();
        }
//        ifaIndex.getStateFromBase();
        rollbackInfos(sessionId);
        rollbackIndexInfos(leafPageInfos, sessionId);
        rollbackIndexInfos(nodePageInfos, sessionId);
    }

    private void rollbackInfos(Object sessionId) {
        synchronized (instancePageInfos) {
            Iterator values = instancePageInfos.values().iterator();
            ArrayList toDelete = new ArrayList();
            while (values.hasNext()) {
                InMemoryPageInfo memoryInfo = (InMemoryPageInfo) values.next();
                if (!memoryInfo.rollback(sessionId)) {
                    toDelete.add(memoryInfo);
                }
            }
            for (short i = 0; i < toDelete.size(); i++) {
                Object pid = ((InMemoryPageInfo) toDelete.get(i)).getPid();
                instancePageInfos.remove(pid);
                instanceFreePagePidList.remove(pid);
            }
        }
    }

    private void rollbackIndexInfos(Map infos, Object sessionId) {
        synchronized (infos) {
            Iterator valuesByClasses = infos.values().iterator();
            ArrayList toDelete = new ArrayList();
            while (valuesByClasses.hasNext()) {
                Iterator values = ((Map) valuesByClasses.next()).values().iterator();
                while (values.hasNext()) {
                    InMemoryPageInfo memoryInfo = (InMemoryPageInfo) values.next();
                    if (!memoryInfo.rollback(sessionId)) {
                        toDelete.add(memoryInfo);
                    }
                }
            }
            for (short i = 0; i < toDelete.size(); i++) {
                infos.remove(((InMemoryPageInfo) toDelete.get(i)).getPid());
            }
        }
    }

    public void cleanAllNullValue() {
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : cleanAllNullValue()");
        Iterator pidIfas = ifaIndex.getAllPidIfa().iterator();
        while (pidIfas.hasNext()) {
            InFileAddress ifa = (InFileAddress) pidIfas.next();
            SystemPage systemPage = (SystemPage) physicalAccess.readFileObjectAt(ifa);
            PageInfo pageInfo = (PageInfo) systemPage.getDataAt(ifa);
            InFileAddress instanceIfa = pageInfo.getPageIfa();
            UserPage dataPage = (UserPage) physicalAccess.readFileObjectAt(instanceIfa);
            Iterator freePlaces = pageInfo.getAllFreePlace();
            while (freePlaces.hasNext()) {
                instanceIfa.setIndex(((Integer) freePlaces.next()).intValue());
                dataPage.setDataAt(instanceIfa, null);
            }
            physicalAccess.updateFileObject(dataPage);
        }
    }

    /**
     * ********************************  INSTANCE  *********************************************
     */

    public PhysicalOid allocateNewFpoid(Object sessionId, Object clid) {
        tracer.println(Trace.LOGICAL, "{1} : LogicalPageAccessImpl : allocateNewFpoid({0})", clid, sessionId);
        synchronized (instancePageInfos) {
            // find the next instance page to use
            InFileAddress availableSystemIfa = getFreeSystemIfaIn(instanceFreePagePidList, instancePageInfos);
            PageInfo infos;
            SystemPage systemPage;
            if (availableSystemIfa != null) {
                systemPage = (SystemPage) physicalAccess.readFileObjectAt(availableSystemIfa);
                infos = (PageInfo) systemPage.getDataAt(availableSystemIfa);
            } else {
                systemPage = getOrCreateAvailableSystemPage();
                infos = buildNewInstancePage(sessionId, oidProvider.getNewPid(), systemPage.getIfa());
                systemPage.setDataAt(systemPage.getIfa(), infos);
            }
            Object iid = infos.getAFreePlace();
            InMemoryPageInfo memInfos = (InMemoryPageInfo) instancePageInfos.get(infos.getPid());
            memInfos.getAFreeSpace(sessionId);
            if (memInfos.getFreeSpaceSize() == 0) {
                instanceFreePagePidList.remove(infos.getPid());
            }
            physicalAccess.updateFileObject(systemPage);
            PhysicalOid fpoid = new PhysicalOid(clid, infos.getPid(), iid);

            InFileAddress instanceIfa = infos.getPageIfa();
            instanceIfa.setIndex(fpoid.getIidAsInteger().intValue());
            InstancePage dataPage = (InstancePage) physicalAccess.readFileObjectAt(instanceIfa);
            if (dataPage.getDataAt(instanceIfa) != null) {
                dataPage.setDataAt(instanceIfa, null);
                physicalAccess.updateFileObject(dataPage);
            }

            return fpoid;
        }
    }

    public void desallocateFpoid(Object sessionId, PhysicalOid fpoid) {
        tracer.println(Trace.LOGICAL, "{1} : LogicalPageAccessImpl : desallocateFpoid({0})", fpoid, sessionId);
        deleteDatas(sessionId, fpoid);
    }

    // allocation is done
    public void insertDatas(Object sessionId, PhysicalOid fpoid, DataWrapper datas) {
        tracer.println(Trace.LOGICAL, "{2} : LogicalPageAccessImpl : insertDatas({0}, {1})", fpoid, datas, sessionId);
        InFileAddress instanceIfa = getInstanceIfa(fpoid);
        InstancePage dataPage = (InstancePage) physicalAccess.readFileObjectAt(instanceIfa);
        dataPage.setDataAt(instanceIfa, datas);
        physicalAccess.updateFileObject(dataPage);
    }

    public DataWrapper readDatas(Object sessionId, PhysicalOid fpoid) {
        tracer.println(Trace.LOGICAL, "{2} : LogicalPageAccessImpl : readDatas({0})", fpoid);
        InFileAddress ifa = getInstanceIfa(fpoid);
        InstancePage dataPage = (InstancePage) physicalAccess.readFileObjectAt(ifa);
        return (DataWrapper) dataPage.getDataAt(ifa);
    }

    public PhysicalOid updateDatas(Object sessionId, PhysicalOid fpoid, DataWrapper datas) {
        tracer.println(Trace.LOGICAL, "{2} : LogicalPageAccessImpl : updateDatas({0}, {1})", fpoid, datas, sessionId);
        InFileAddress ifa = getInstanceIfa(fpoid);
        InstancePage dataPage = (InstancePage) physicalAccess.readFileObjectAt(ifa);
        dataPage.setDataAt(ifa, datas);
        physicalAccess.updateFileObject(dataPage);
        return fpoid;
    }

    public void deleteDatas(Object sessionId, PhysicalOid fpoid) {
        tracer.println(Trace.LOGICAL, "{1} : LogicalPageAccessImpl : deleteDatas({0})", fpoid, sessionId);
        // get ifa of the system page
        InFileAddress systemIfa = ifaIndex.getPidIfa(fpoid.getPid());
        // get the page
        SystemPage systemPage = (SystemPage) physicalAccess.readFileObjectAt(systemIfa);
        synchronized (instancePageInfos) {
            // get the page info
            PageInfo infos = (PageInfo) systemPage.getDataAt(systemIfa);
            infos.allocateFreeSpaceAt(fpoid.getIidAsInteger());
            InMemoryPageInfo memInfos = (InMemoryPageInfo) instancePageInfos.get(infos.getPid());
            memInfos.freeASpace(sessionId);
            if (memInfos.getFreeSpaceSize() == 1) {
                instanceFreePagePidList.addFirst(memInfos.getPid());
            }
            // update page where modifications has been done
            physicalAccess.updateFileObject(systemPage);
        }
    }


    /**
     * ************************************  INDEX  ************************************************
     */

    public InFileAddress allocateNodeAddressAndInsert(Object sessionId, FieldDescription meta, DataWrapper node) {
        tracer.println(Trace.LOGICAL, "{0} : LogicalPageAccessImpl : allocateNodeAddressAndInsert()", sessionId);
        synchronized (nodePageInfos) {
            String fieldKey = meta.getClassMeta().getClassName() + "-" + meta.getFieldName();
            Map m = (Map) nodePageInfos.get(fieldKey);
            if (m == null) {
                m = new HashMap();
                nodePageInfos.put(fieldKey, m);
            }
            InFileAddress availableSystemIfa = getFreeSystemIfaIn(null, m);
            PageInfo infos;
            SystemPage systemPage;
            if (availableSystemIfa != null) {
                availableSystemIfa = availableSystemIfa.getClone();
                systemPage = (SystemPage) physicalAccess.readFileObjectAt(availableSystemIfa);
                infos = (PageInfo) systemPage.getDataAt(availableSystemIfa);
            } else {
                systemPage = getOrCreateAvailableSystemPage();
                infos = buildNewNodePage(sessionId, meta, oidProvider.getNewPid(), systemPage.getIfa());
                systemPage.setDataAt(systemPage.getIfa(), infos);
            }
            Integer iid = infos.getAFreePlace();
            ((InMemoryPageInfo) m.get(infos.getPid())).getAFreeSpace(sessionId);
            InFileAddress pageIfa = infos.getPageIfa();
            pageIfa.setIndex(iid.intValue());
            node.setIfa(pageIfa);
            NodePage nodePage = (NodePage) physicalAccess.readFileObjectAt(pageIfa);
            nodePage.setDataAt(pageIfa, node);
            physicalAccess.updateFileObject(systemPage);
            physicalAccess.updateFileObject(nodePage);
            return pageIfa.getClone();
        }
    }

    public Object readNode(Object sessionId, InFileAddress ifa) {
        NodePage page = (NodePage) physicalAccess.readFileObjectAt(ifa);
        return page.getDataAt(ifa);
    }

    public void updateNode(Object sessionId, InFileAddress ifa, DataWrapper node) {
        updateIndexObject(ifa, node);
    }

    public void removeNode(Object sessionId, InFileAddress ifa) {
        tracer.println(Trace.LOGICAL, "{1} : LogicalPageAccessImpl : removeNode({0})", ifa, sessionId);
        NodePage nodePage = (NodePage) physicalAccess.readFileObjectAt(ifa);
        InFileAddress systemIfa = ifaIndex.getPidIfa(nodePage.getPid());
        SystemPage systemPage = (SystemPage) physicalAccess.readFileObjectAt(systemIfa);
        synchronized (nodePageInfos) {
            IndexPageInfo indexInfos = (IndexPageInfo) systemPage.getDataAt(systemIfa);
            indexInfos.allocateFreeSpaceAt(new Integer(ifa.getIndex()));
            Map m = (Map) nodePageInfos.get(indexInfos.getFieldKey());
            ((InMemoryPageInfo) m.get(nodePage.getPid())).getAFreeSpace(sessionId);
            physicalAccess.updateFileObject(systemPage);
        }
    }

    public InFileAddress allocateLeafAddressAndInsert(Object sessionId, FieldDescription meta, DataWrapper oids) {
        tracer.println(Trace.LOGICAL, "{0} : LogicalPageAccessImpl : allocateLeafAddressAndInsert()");
        synchronized (leafPageInfos) {
            String fieldKey = meta.getClassMeta().getClassName() + "-" + meta.getFieldName();
            Map m = (Map) leafPageInfos.get(fieldKey);
            if (m == null) {
                m = new HashMap();
                leafPageInfos.put(fieldKey, m);
            }
            InFileAddress availableSystemIfa = getFreeSystemIfaIn(null, m);
            PageInfo infos;
            SystemPage systemPage;
            if (availableSystemIfa != null) {
                availableSystemIfa = availableSystemIfa.getClone();
                systemPage = (SystemPage) physicalAccess.readFileObjectAt(availableSystemIfa);
                infos = (PageInfo) systemPage.getDataAt(availableSystemIfa);
            } else {
                systemPage = getOrCreateAvailableSystemPage();
                infos = buildNewLeafPage(sessionId, meta, oidProvider.getNewPid(), systemPage.getIfa());
                systemPage.setDataAt(systemPage.getIfa(), infos);
            }
            Integer iid = infos.getAFreePlace();
            ((InMemoryPageInfo) m.get(infos.getPid())).getAFreeSpace(sessionId);
            InFileAddress pageIfa = infos.getPageIfa();
            pageIfa.setIndex(iid.intValue());
            oids.setIfa(pageIfa);
            LeafPage leafPage = (LeafPage) physicalAccess.readFileObjectAt(pageIfa);
            leafPage.setDataAt(pageIfa, oids);
            physicalAccess.updateFileObject(systemPage);
            physicalAccess.updateFileObject(leafPage);
            return pageIfa.getClone();
        }
    }

    public Object readLeaf(Object sessionId, InFileAddress ifa) {
        LeafPage page = (LeafPage) physicalAccess.readFileObjectAt(ifa);
        return page.getDataAt(ifa);
    }

    public void updateLeaf(Object sessionId, InFileAddress ifa, DataWrapper oids) {
        updateIndexObject(ifa, oids);
    }

    public void removeLeaf(Object sessionId, InFileAddress ifa) {
        tracer.println(Trace.LOGICAL, "{1} : LogicalPageAccessImpl : removeNode({0})", ifa, sessionId);
        LeafPage leafPage = (LeafPage) physicalAccess.readFileObjectAt(ifa);
        InFileAddress systemIfa = ifaIndex.getPidIfa(leafPage.getPid());
        SystemPage systemPage = (SystemPage) physicalAccess.readFileObjectAt(systemIfa);
        synchronized (leafPageInfos) {
            IndexPageInfo indexInfos = (IndexPageInfo) systemPage.getDataAt(systemIfa);
            indexInfos.allocateFreeSpaceAt(new Integer(ifa.getIndex()));
            Map m = (Map) leafPageInfos.get(indexInfos.getFieldKey());
            ((InMemoryPageInfo) m.get(leafPage.getPid())).getAFreeSpace(sessionId);
            physicalAccess.updateFileObject(systemPage);
        }
    }

    /**
     * ********************************     PRIVATE     ********************************************
     */

    private InFileAddress getInstanceIfa(PhysicalOid fpoid) {
        InFileAddress systemIfa = ifaIndex.getPidIfa(fpoid.getPid());
        SystemPage systemPage = null;
        systemPage = (SystemPage) physicalAccess.readFileObjectAt(systemIfa);
        PageInfo infos = (PageInfo) systemPage.getDataAt(systemIfa);
        InFileAddress instanceIfa = infos.getPageIfa();
        instanceIfa.setIndex(fpoid.getIidAsInteger().intValue());
        return instanceIfa;
    }

    private void getInMemoryInfos() {
        instancePageInfos = Collections.synchronizedMap(new HashMap());
        instanceFreePagePidList = new LinkedList();
        leafPageInfos = Collections.synchronizedMap(new HashMap());
        nodePageInfos = Collections.synchronizedMap(new HashMap());

        Iterator pids = ifaIndex.getAllPid().iterator();
        while (pids.hasNext()) {
            Object pid = pids.next();
            InFileAddress ifa = ifaIndex.getPidIfa(pid);
            SystemPage systemPage = (SystemPage) physicalAccess.readFileObjectAt(ifa);
            PageInfo pageInfo = (PageInfo) systemPage.getDataAt(ifa);
            InMemoryPageInfo memoryPageInfo = InMemoryPageInfo.createNewInstance(
                    null, isMonoImplementation, pageInfo);
            if (pageInfo.getPageType() == Page.INSTANCEPAGE_TYPE) {
                instancePageInfos.put(pid, memoryPageInfo);
                if (memoryPageInfo.getFreeSpaceSize() > 0) {
                    instanceFreePagePidList.add(pid);
                }
            } else {
                String fieldKey = ((IndexPageInfo) pageInfo).getFieldKey();
                if (pageInfo.getPageType() == Page.LEAFPAGE_TYPE) {
                    Map m = (Map) leafPageInfos.get(fieldKey);
                    if (m == null) {
                        m = new HashMap();
                        leafPageInfos.put(fieldKey, m);
                    }
                    m.put(pid, memoryPageInfo);
                } else {
                    Map m = (Map) nodePageInfos.get(fieldKey);
                    if (m == null) {
                        m = new HashMap();
                        nodePageInfos.put(fieldKey, m);
                    }
                    m.put(pid, memoryPageInfo);
                }
            }
        }
    }

    private int getInstanceFileIndex() {
        fileIndex = (fileIndex + 1) % instancePageNumber;
        return (fileIndex + 2);
    }

    private int getLeafFileIndex() {
        fileIndex = (fileIndex + 1) % instancePageNumber;
        return (fileIndex + 2);
    }

    private int getNodeFileIndex() {
        fileIndex = (fileIndex + 1) % instancePageNumber;
        return (fileIndex + 2);
    }

    private InFileAddress getFreeSystemIfaIn(LinkedList pids, Map infos) {
        synchronized (infos) {
            if (pids != null) {
                while (!pids.isEmpty()) {
                    Object pid = pids.getFirst();
                    InMemoryPageInfo memoryInfo = (InMemoryPageInfo) infos.get(pid);
                    if (memoryInfo.getFreeSpaceSize() > 0) {
                        return ifaIndex.getPidIfa(pid);
                    }
                    pids.removeFirst();
                }
            } else {
                Iterator pidsIterator = infos.keySet().iterator();
                while (pidsIterator.hasNext()) {
                    Object pid = pidsIterator.next();
                    InMemoryPageInfo memoryInfo = (InMemoryPageInfo) infos.get(pid);
                    if (memoryInfo.getFreeSpaceSize() > 0) {
                        return ifaIndex.getPidIfa(pid);
                    }
                }
            }
        }
        return null;
    }

    private synchronized PageInfo buildNewInstancePage(Object sessionId, Object pid, InFileAddress systemIfa) {
        InstancePage page = new InstancePage(instancePageSize, pid);
        InFileAddress pageIfa = new InFileAddress(ifaIndex.getNewInstancePageAddress());
        pageIfa.setFileIndex(getInstanceFileIndex());
        page.setIfa(pageIfa);
        physicalAccess.insertFileObject(page);

        PageInfo pageInfo = new PageInfo(instancePageSize, Page.INSTANCEPAGE_TYPE);
        pageInfo.setPageIfa(pageIfa);
        pageInfo.setPid(pid);

        InMemoryPageInfo inMemoryPageInfo =
                InMemoryPageInfo.createNewInstance(workingSessions, isMonoImplementation, pageInfo);
        instancePageInfos.put(pid, inMemoryPageInfo);
        instanceFreePagePidList.addFirst(pid);
        ifaIndex.putPidIfa(pid, systemIfa.getClone());

        return pageInfo;
    }

    private synchronized PageInfo buildNewLeafPage(
            Object sessionId, FieldDescription meta, Object pid, InFileAddress systemIfa) {
        String fieldKey = meta.getClassMeta().getClassName() + "-" + meta.getFieldName();
        short size = meta.getIndexDescription().getLeafPageSize();
        LeafPage page = new LeafPage(size, pid);
        InFileAddress pageIfa = new InFileAddress(ifaIndex.getNewLeafPageAddress());
        pageIfa.setFileIndex(getLeafFileIndex());
        page.setIfa(pageIfa);
        physicalAccess.insertFileObject(page);

        IndexPageInfo pageInfo = new IndexPageInfo(size, Page.LEAFPAGE_TYPE, fieldKey);
        pageInfo.setPageIfa(pageIfa);
        pageInfo.setPid(pid);

        Map m = (Map) leafPageInfos.get(fieldKey);
        m.put(pid, InMemoryPageInfo.createNewInstance(workingSessions, isMonoImplementation, pageInfo));
        ifaIndex.putPidIfa(pid, systemIfa.getClone());

        return pageInfo;
    }

    private synchronized PageInfo buildNewNodePage(
            Object sessionId, FieldDescription meta, Object pid, InFileAddress systemIfa) {
        String fieldKey = meta.getClassMeta().getClassName() + "-" + meta.getFieldName();
        short size = meta.getIndexDescription().getNodePageSize();
        NodePage page = new NodePage(size, pid);
        InFileAddress pageIfa = new InFileAddress(ifaIndex.getNewNodePageAddress());
        pageIfa.setFileIndex(getNodeFileIndex());
        page.setIfa(pageIfa);
        physicalAccess.insertFileObject(page);

        IndexPageInfo pageInfo = new IndexPageInfo(size, Page.NODEPAGE_TYPE, fieldKey);
        pageInfo.setPageIfa(pageIfa);
        pageInfo.setPid(pid);

        Map m = (Map) nodePageInfos.get(fieldKey);
        m.put(pid, InMemoryPageInfo.createNewInstance(workingSessions, isMonoImplementation, pageInfo));
        ifaIndex.putPidIfa(pid, systemIfa.getClone());

        return pageInfo;
    }

    public void updateIndexObject(InFileAddress ifa, DataWrapper indexObject) {
        tracer.println(Trace.LOGICAL, "LogicalPageAccessImpl : updateIndexObject({0}, {1})", ifa, indexObject);
        UserPage page = (UserPage) physicalAccess.readFileObjectAt(ifa);
        page.setDataAt(ifa, indexObject);
        physicalAccess.updateFileObject(page);
    }

    private synchronized SystemPage getOrCreateAvailableSystemPage() {
        InFileAddress availableSystemPageIfa = ifaIndex.getCurrentAvalaibleSystemPage().getClone();
        SystemPage systemPage;
        if (availableSystemPageIfa.getIndex() == (systemPageSize - 1)) {
            availableSystemPageIfa = new InFileAddress(ifaIndex.getNewSystemPageAddress());
            systemPage = new SystemPage(systemPageSize);
            systemPage.setIfa(availableSystemPageIfa);
            physicalAccess.insertFileObject(systemPage);
        } else {
            availableSystemPageIfa.setIndex(availableSystemPageIfa.getIndex() + 1);
            systemPage = (SystemPage) physicalAccess.readFileObjectAt(availableSystemPageIfa);
            systemPage.setIfa(availableSystemPageIfa);
        }
        ifaIndex.setCurrentAvalaibleSystemPage(availableSystemPageIfa);
        return systemPage;
    }


    private ArrayList workingSessions;
    private IfaIndex ifaIndex;
    private IdentityProvider oidProvider;
    private InternalPhysicalFileAccess physicalAccess;

    private Map instancePageInfos;
    private LinkedList instanceFreePagePidList;
    private Map leafPageInfos;
    private Map nodePageInfos;

    private int fileIndex = 0;
    private int instancePageNumber;
    private short instancePageSize;
    private short systemPageSize;

    private boolean isMonoImplementation;

    private Trace tracer;
}
