/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.server.page;

import org.objectweb.jalisto.se.api.internal.JalistoObject;
import org.objectweb.jalisto.se.impl.InFileAddress;

public abstract class Page implements JalistoObject {

    public Page(int size) {
        storedDatas = new Object[size];
    }

    public boolean isEmpty() {
        for (short i = 0; i < storedDatas.length; i++) {
            if (storedDatas[i] != null) {
                return false;
            }
        }
        return true;
    }

    public Object[] getAllDatas() {
        return storedDatas;
    }

    public void setData(Object data) {
    }

    public Object getData() {
        return storedDatas;
    }

    public void setDataAt(InFileAddress dataIfa, Object dataToInsert) {
        storedDatas[dataIfa.getIndex()] = dataToInsert;
    }

    public void setDataAt(int index, Object dataToInsert) {
        storedDatas[index] = dataToInsert;
    }

    public Object getDataAt(InFileAddress dataIfa) {
        return storedDatas[dataIfa.getIndex()];
    }

    public Object getDataAt(int index) {
        return storedDatas[index];
    }

    public void setIfa(InFileAddress ifa) {
        this.ifa = ifa.getClone();
    }

    public InFileAddress getIfa() {
        return ifa;
    }

    public int getSize() {
        return storedDatas.length;
    }

    public JalistoObject internalClone(Page clone) {
        clone.setIfa(ifa.getClone());
        System.arraycopy(storedDatas, 0, clone.storedDatas, 0, storedDatas.length);
        return clone;
    }

    public String toString() {
        if (ifa != null) {
            return pageName() + "@" + String.valueOf(ifa) + ":" + dataToString();
        } else {
            return pageName() + ":" + dataToString();
        }
    }

    private String dataToString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for (int i = 0; i < storedDatas.length; i++) {
            if (storedDatas[i] == null) {
                sb.append("null");
            } else {
                sb.append(storedDatas[i]);
            }
            if (i != (storedDatas.length - 1)) {
                sb.append(";");
            }
        }
        sb.append("]");
        return sb.toString();
    }

    protected String pageName() {
        return "P";
    }


    protected Object[] storedDatas;
    protected transient InFileAddress ifa;


    static final long serialVersionUID = -7589377099514761459L;


    public static final short CLASSPAGE_TYPE = 0;
    public static final short INSTANCEPAGE_TYPE = 1;
    public static final short OIDPAGE_TYPE = 2;
    public static final short SYSTEMPAGE_TYPE = 3;
    public static final short NODEPAGE_TYPE = 4;
    public static final short LEAFPAGE_TYPE = 5;
}
