/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.trace;

import java.text.MessageFormat;
import java.util.*;

public abstract class TraceImpl extends Trace {
    public TraceImpl(TraceContext context) {
        this.context = context;
        this.enabledModules = Collections.EMPTY_SET;
        this.isShowThread = false;
        this.maxModuleSize = 3;
    }

    public boolean isEnabled(Trace.Module component) {
        return context.isTracing(component);
    }

    public boolean isShowThread() {
        return isShowThread;
    }

    public void enableShowThread() {
        isShowThread = true;
    }

    public void enableTrace(boolean enable) {
        this.enabled = enable;
    }

    public Set enabledModules() {
        return enabledModules;
    }

    protected void setEnabledModules(Set set) {
        enabledModules = set;
    }


    public void init(Iterator traceModuleNames) {
        this.mask = new BitSet();
        TRACE_ON.putInMask(mask);

        Set resolvedModules = new HashSet();
        Set unresolvedModuleNames = new HashSet();
        getModules(traceModuleNames, resolvedModules, unresolvedModuleNames);
        this.enabledModules = resolvedModules;

        if (!resolvedModules.isEmpty()) {
            for (Iterator iterator = resolvedModules.iterator(); iterator.hasNext();) {
                Module module = (Module) iterator.next();
                module.putInMask(mask);
                maxModuleSize = Math.max(maxModuleSize, module.getName().length() + 2);
            }
            println(TRACE_ON, "TRACE ENABLED");
            println(TRACE_ON, "the module(s) {0} has/have been enabled", resolvedModules);
        }

        if (!unresolvedModuleNames.isEmpty()) {
            println(TRACE_ON, "the specified module(s) {0} is/are not valid", unresolvedModuleNames);
        }
    }

    public synchronized void println(Module component, String message) {
        if ((!enabled) || (!component.isInMask(mask))) {
            return;
        }
        print(component, message);
    }

    public synchronized void println(Module component, String message, Object parameter1) {
        if ((!enabled) || (!component.isInMask(mask))) {
            return;
        }
        args1[0] = parameter1;
        printFormatted(component, message, args1);
    }

    public synchronized void println(Module component, String message, Object parameter1, Object parameter2) {
        if ((!enabled) || (!component.isInMask(mask))) {
            return;
        }
        args2[0] = parameter1;
        args2[1] = parameter2;
        printFormatted(component, message, args2);
    }

    public synchronized void println(Module component, String message,
                                     Object parameter1, Object parameter2,
                                     Object parameter3) {
        if ((!enabled) || (!component.isInMask(mask))) {
            return;
        }
        args3[0] = parameter1;
        args3[1] = parameter2;
        args3[2] = parameter3;
        printFormatted(component, message, args3);
    }

    public synchronized void println(Module component, String message, Object[] parameters) {
        if ((!enabled) || (!component.isInMask(mask))) {
            return;
        }
        printFormatted(component, message, parameters);
    }

    protected String pad(Trace.Module module) {
        return rightPad(module.toString(), maxModuleSize, " ");
    }

    protected void printFormatted(Trace.Module module, String message, Object[] args) {
        String formatedMessage = MessageFormat.format(message, args);
        print(module, formatedMessage);
    }

    public String toString() {
        return getTraceType() + " " + enabledModules();
    }

    private static void getModules(Iterator moduleNames, Set resolvedModules, Set unresolvedModuleNames) {
        while (moduleNames.hasNext()) {
            String moduleName = ((String) moduleNames.next()).toUpperCase();
            Module module = (Module) MODULE_REPOSITORY.get(moduleName);
            if (module == null) {
                unresolvedModuleNames.add(moduleName);
            } else {
                resolvedModules.add(module);
            }
        }
    }

    private static String rightPad(String str, int size, String delim) {
        size = (size - str.length()) / delim.length();
        if (size > 0) {
            str += repeat(delim, size);
        }
        return str;
    }

    private static String repeat(String str, int repeat) {
        StringBuffer buffer = new StringBuffer(repeat * str.length());
        for (int i = 0; i < repeat; i++) {
            buffer.append(str);
        }
        return buffer.toString();
    }

    protected abstract String getTraceType();

    protected abstract void print(Module module, String message);

    private BitSet mask;
    private TraceContext context;
    private Set enabledModules;
    private Object[] args1 = new Object[1];
    private Object[] args2 = new Object[2];
    private Object[] args3 = new Object[3];
    private boolean isShowThread;
    private int maxModuleSize;


    static final long serialVersionUID = -7589377097964441459L;
}
