/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.util;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.Session;

import java.util.Properties;
import java.util.Iterator;
import java.io.*;
import java.net.URL;

public class JalistoUtils {

    public static File loadPropertiesFileIn(String propertiesFilePath, Properties prop) throws IOException {
        boolean isLoaded = false;
        File propfile = null;
        if (!propertiesFilePath.equalsIgnoreCase("")) {
            propfile = new File(propertiesFilePath);
            if (propfile.exists() && (!propfile.isDirectory())) {
                FileInputStream fis = new FileInputStream(propfile);
                prop.load(fis);
                isLoaded = true;
            }
            if (!isLoaded) {
                ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                InputStream testIS = classLoader.getResourceAsStream(propertiesFilePath);
                URL url = classLoader.getResource(propertiesFilePath);
                propfile = new File(url.getFile());
                prop.load(testIS);
            }
        }
        return propfile;
    }

    public static void truncateProperties(Properties prop) {
        Iterator keys = prop.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            String value = prop.getProperty(key);
            while (value.endsWith(" ") || value.endsWith("\t")) {
                value = value.substring(0, value.length() - 1);
            }
            prop.setProperty(key, value);
        }
    }

    /**
     * Gets a substring of the specified string avoiding exceptions.
     * A negative start position can be used to start n characters from
     * the end of the string.
     * 
     * @param str   the string to get the substring from
     * @param start the position to start from,  negative means
     *              count back from the end of the string by this many characters
     * @return substring from start position
     */
    public static String substring(String str, int start) {
        if (str == null) {
            return null;
        }

        // handle negatives, which means last n characters
        if (start < 0) {
            start = str.length() + start; // remember start is negative
        }

        if (start < 0) {
            start = 0;
        }

        if (start > str.length()) {
            return "";
        }

        return str.substring(start);
    }

    public static String arrayToString(Object[] array) {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for (short i = 0; i < array.length; i++) {
            sb.append(array[i]);
            if (i < array.length - 1) {
                sb.append(",");
            }
        }
        sb.append("]");
        return sb.toString();
    }

    public static String arrayToString(Object[] array, Session session) {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for (short i = 0; i < array.length; i++) {
            Object o = array[i];
            if (o instanceof LogicalOid) {
                sb.append(arrayToString(session.readObjectByOid(o), session));
            } else {
                sb.append(o);
            }
            if (i < array.length - 1) {
                sb.append(",");
            }
        }
        sb.append("]");
        return sb.toString();
    }

    public static boolean areEquals(byte[] b1, byte[] b2) {
        if (b1.length != b2.length) {
            return false;
        }
        for (int i = 0; i < b1.length; i++) {
            if (b1[i] != b2[i]) {
                return false;
            }
        }
        return true;
    }

    public static boolean areEquals(Object[] b1, Object[] b2) {
        if (b1.length != b2.length) {
            return false;
        }
        for (int i = 0; i < b1.length; i++) {
            if (!b1[i].equals(b2[i])) {
                return false;
            }
        }
        return true;
    }
}
