/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

public class RecordHeader {
    /** Number of bytes of data that this record can hold (4 bytes). */
    private int dataCapacity;

    /** Actual number of bytes of data held in this record (4 bytes). */
    private int dataCount;

    /** Indicates this header's position in the file index. */
    private int indexPosition;

    /** File pointer to the first byte of record data (8 bytes). */
    private long dataPointer;

    public RecordHeader() {
    }

    public RecordHeader(long dataPointer, int dataCapacity) {
        if (dataCapacity < 1) {
            throw new IllegalArgumentException("Bad record size: " + dataCapacity);
        }
        this.dataPointer = dataPointer;
        this.dataCapacity = dataCapacity;
        this.dataCount = 0;
    }

    public void setDataCapacity(int dataCapacity) {
        this.dataCapacity = dataCapacity;
    }

    public int getDataCapacity() {
        return dataCapacity;
    }

    public void setDataCount(int dataCount) {
        this.dataCount = dataCount;
    }

    public int getDataCount() {
        return dataCount;
    }

    public void setDataPointer(long dataPointer) {
        this.dataPointer = dataPointer;
    }

    public long getDataPointer() {
        return dataPointer;
    }

    public int getFreeSpace() {
        return dataCapacity - dataCount;
    }

    public void setIndexPosition(int indexPosition) {
        this.indexPosition = indexPosition;
    }

    public int getIndexPosition() {
        return indexPosition;
    }

    public static RecordHeader readHeader(DataInput in) throws IOException {
        RecordHeader r = new RecordHeader();
        r.read(in);
        return r;
    }

    /**
     * Returns a new record header which occupies the free space of this record.
     * Shrinks this record size by the size of its free space.
     */
    public RecordHeader split() {
        long newFp = dataPointer + (long) dataCount;
        RecordHeader newRecord = new RecordHeader(newFp, getFreeSpace());
        dataCapacity = dataCount;
        return newRecord;
    }

    public void write(DataOutput out) throws IOException {
        out.writeLong(dataPointer);
        out.writeInt(dataCapacity);
        out.writeInt(dataCount);
    }

    public void read(DataInput in) throws IOException {
        dataPointer = in.readLong();
        dataCapacity = in.readInt();
        dataCount = in.readInt();
    }

    public boolean equals(Object object) {
        try {
            RecordHeader candidate = (RecordHeader) object;
//            return ((dataCapacity == candidate.dataCapacity) &&
//                    (dataCount == candidate.dataCount) &&
//                    (dataPointer == candidate.dataPointer) &&
//                    (indexPosition == candidate.indexPosition));
            return (dataCount == candidate.dataCount);
        } catch (Exception e) {
        }
        return false;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("RecordHeader(");
        sb.append("dCap").append(dataCapacity);
        sb.append(",dCnt").append(dataCount);
        sb.append("dPnt").append(dataPointer);
        sb.append("iPos").append(indexPosition);
        sb.append(")");
        return sb.toString();
    }
}
