/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.asynraf;

import org.objectweb.jalisto.se.api.internal.JalistoObject;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.storage.raf.DbByteArrayOutputStream;
import org.objectweb.jalisto.se.storage.raf.RecordReader;
import org.objectweb.jalisto.se.storage.raf.RecordWriter;

import java.io.EOFException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.*;

public class AsynrafLogger {

    public AsynrafLogger(JalistoProperties properties, PhysicalFileAccessLogAsynrafImpl physical)
            throws IOException {
        this.physical = physical;
        this.trace = physical.getTrace();
        infos = new HashMap();
        demon = new LogDemon(this, physical);
        file = new RandomAccessFile(properties.getLogFileName(), "rw");
        file.seek(0);
        currentWriteAddress = 0;
        maxKeyLength = properties.getKeyLength();
    }

    public Object[] getAddressesForExtent(String filter) {
        Object[] result = new Object[2];
        ArrayList toRemove = new ArrayList();
        ArrayList toAdd = new ArrayList();
        result[0] = toAdd;
        result[1] = toRemove;
        synchronized (infos) {
            Iterator addresses = infos.keySet().iterator();
            while (addresses.hasNext()) {
                String address = (String) addresses.next();
                if (address.startsWith(filter)) {
                    LogInfo logInfo = (LogInfo) infos.get(address);
                    if (logInfo.getActionType() == LogInfo.INSERT_TYPE) {
                        toAdd.add(address);
                    } else if (logInfo.getActionType() == LogInfo.DELETE_TYPE) {
                        toRemove.add(address);
                    }
                }
            }
        }
        return result;
    }

    public void insertFileObject(InFileAddress ifa, JalistoObject fo) {
        if (!isCommitting) {
            throw new JalistoException("access minilog outside commit process");
        }
        try {
            synchronized (file) {
                RecordWriter rw = new RecordWriter(ifa.getAddress());
                rw.writeObject(fo);
                indexCounter++;
                LogInfo info = new LogInfo(LogInfo.INSERT_TYPE, ifa.getClone(), indexCounter);
                infos.put(ifa.getAddress(), info);
                writeType(LogInfo.INSERT_TYPE);
                writeIfa(ifa);
                writeLong(rw.getDataLength());
                info.setValueAddress(currentWriteAddress);
                info.setValueLength(rw.getDataLength());
                file.seek(currentWriteAddress);
                rw.writeTo(file);
                currentWriteAddress = file.getFilePointer();
                rw.reset();
            }
        } catch (Exception e) {
            throw new JalistoException("could not insert object", e);
        }
    }

    public JalistoObject readFileObjectAt(InFileAddress ifa) {
        try {
            RecordReader rr;
            try {
                LogInfo info = (LogInfo) infos.get(ifa.getAddress());
                if (info != null) {
                    if (info.getActionType() == LogInfo.DELETE_TYPE) {
                        throw new JalistoException("this object has been deleted : " + ifa);
                    }
                    byte[] datas = new byte[(int) info.getValueLength()];
                    synchronized (file) {
                        file.seek(info.getValueAddress());
                        file.readFully(datas);
                    }
                    rr = new RecordReader(ifa.getAddress(), datas);
                } else {
                    rr = physical.internalRead(ifa);
                }
            } catch (EOFException eofe) {
                rr = physical.internalRead(ifa);
            }
            JalistoObject result = (JalistoObject) rr.readObject();
            result.setIfa(ifa);
            rr.reset();
            return result;
        } catch (Exception e) {
            throw new JalistoException(e);
        }
    }

    public void updateFileObject(InFileAddress ifa, JalistoObject fo) {
        if (!isCommitting) {
            throw new JalistoException("access minilog outside commit process");
        }
        try {
            synchronized (file) {
                RecordWriter rw = new RecordWriter(ifa.getAddress());
                rw.writeObject(fo);
                indexCounter++;
                LogInfo info = new LogInfo(LogInfo.UPDATE_TYPE, ifa.getClone(), indexCounter);
                infos.put(ifa.getAddress(), info);
                writeType(LogInfo.UPDATE_TYPE);
                writeIfa(ifa);
                writeLong(rw.getDataLength());
                info.setValueAddress(currentWriteAddress);
                info.setValueLength(rw.getDataLength());
                file.seek(currentWriteAddress);
                rw.writeTo(file);
                rw.reset();
                currentWriteAddress = file.getFilePointer();
                RecordReader rr = physical.internalRead(ifa);
                writeLong(rr.getData().length);
                info.setOldValueAddress(currentWriteAddress);
                info.setOldValueLength(rr.getData().length);
                file.seek(currentWriteAddress);
                file.write(rr.getData());
                rr.reset();
                currentWriteAddress = file.getFilePointer();
            }
        } catch (Exception e) {
            throw new JalistoException("could not update object", e);
        }
    }

    public void deleteFileObject(InFileAddress ifa) {
        if (!isCommitting) {
            throw new JalistoException("access minilog outside commit process");
        }
        try {
            synchronized (file) {
                indexCounter++;
                LogInfo info = new LogInfo(LogInfo.DELETE_TYPE, ifa.getClone(), indexCounter);
                infos.put(ifa.getAddress(), info);
                writeType(LogInfo.DELETE_TYPE);
                writeIfa(ifa);
                RecordReader rr = physical.internalRead(ifa);
                writeLong(rr.getData().length);
                info.setOldValueAddress(currentWriteAddress);
                info.setOldValueLength(rr.getData().length);
                file.seek(currentWriteAddress);
                file.write(rr.getData());
                rr.reset();
                currentWriteAddress = file.getFilePointer();
            }
        } catch (Exception e) {
            throw new JalistoException("could not delete object", e);
        }
    }

    public void open() {
        demon.open();
    }

    public void close() {
        while (!infos.isEmpty()) {
            Thread.yield();
        }
        demon.close();
    }

    public void startCommitting() throws IOException {
        while (demon.isWorking()) {
            Thread.yield();
        }
        isCommitting = true;
        indexCounter = -1;
        writeStatus(LOGGER_WORKING);
    }

    public void finishCommitting() throws IOException {
        writeStatus(DEMON_WORKING);
        isCommitting = false;
        demon.setWorking();
    }

    public boolean isCommitting() {
        return isCommitting;
    }

    public RandomAccessFile getFile() {
        return file;
    }

    public List getSortedInfos() {
        if (isCommitting) {
            throw new JalistoException("AsynrafLogger is committing");
        }
        List sortedInfos = new ArrayList();
//        synchronized(infos) {
        sortedInfos.addAll(infos.values());
//        }
        Collections.sort(sortedInfos);
        return sortedInfos;
    }

    public void removeInfo(LogInfo info) {
        if (isCommitting) {
            throw new JalistoException("AsynrafLogger is committing");
        }
        infos.remove(info.getIfa().getAddress());
    }

    /**
     * *******************************    I/O METHODS    ****************************************
     */

    private void writeType(short type) throws IOException {
        file.seek(currentWriteAddress);
        file.writeShort(type);
        currentWriteAddress = file.getFilePointer();
    }

    public void writeStatus(short status) throws IOException {
        synchronized (file) {
            file.seek(0);
            file.writeShort(status);
            currentWriteAddress = file.getFilePointer();
        }
    }

    private void writeLong(long value) throws IOException {
        file.seek(currentWriteAddress);
        file.writeLong(value);
        currentWriteAddress = file.getFilePointer();
    }

    private void writeIfa(InFileAddress ifa) throws IOException {
        String key = ifa.getAddress();
        DbByteArrayOutputStream temp = DbByteArrayOutputStream.getInstance();
        temp.getOutput().writeUTF(key);

        if (temp.size() > maxKeyLength) {
            throw new JalistoException("Key " + key + " of size " + temp.size() +
                                           " is larger than permitted size of " +
                                           maxKeyLength + " bytes");
        }
        file.seek(currentWriteAddress);
        temp.writeTo(file);
        temp.reset();
        file.writeInt(ifa.getFileIndex());
        file.writeInt(ifa.getIndex());
        currentWriteAddress = file.getFilePointer();
    }

    public byte[] readBytes(long address, int size) throws IOException {
        synchronized (file) {
            file.seek(address);
            byte[] result = new byte[size];
            file.readFully(result);
            return result;
        }
    }

    public void setLength(long length) throws IOException {
        synchronized (file) {
            file.setLength(length);
        }
    }


    private PhysicalFileAccessLogAsynrafImpl physical;
    private RandomAccessFile file;
    private HashMap infos;
    private LogDemon demon;
    private long currentWriteAddress;
    private int maxKeyLength;
    private Trace trace;
    private boolean isCommitting = false;

    private int indexCounter;

    public final static short LOGGER_WORKING = 32;
    public final static short DEMON_WORKING = 478;
    public final static short NOBODY_WORKING = 174;
}
