/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.asynraf;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.JalistoObject;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.storage.raf.*;
import org.objectweb.jalisto.se.storage.raf.log.RecordsFileLog;

import java.io.*;
import java.util.*;

public class PhysicalFileAccessLogAsynrafImpl implements PluggablePhysicalFileAccess {

    private void init(int initialSize, JalistoProperties properties)
            throws IOException, RecordsFileException {
        int nbrDbFiles = properties.getInstanceDbFileNumber();
        if (nbrDbFiles > 1) {
            files = new RecordsFileLog[nbrDbFiles + 2];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(), properties.isCleanLog());
            files[0].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());
            files[1] = new RecordsFileLog(properties.getOidDbFilePath(), properties.isCleanLog());
            files[1].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());

            for (int i = 0; i < nbrDbFiles; i++) {
                files[i + 2] = new RecordsFileLog(properties.getInstanceDbFilePathAt(i), properties.isCleanLog());
                files[i + 2].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());
            }
        } else {
            files = new RecordsFileLog[1];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(), properties.isCleanLog());
            files[0].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());
        }
    }

    private void init(String accessFlags, JalistoProperties properties)
            throws IOException, RecordsFileException {
        int nbrDbFiles = properties.getInstanceDbFileNumber();
        if (nbrDbFiles > 1) {
            files = new RecordsFileLog[nbrDbFiles + 2];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(),
                                          accessFlags,
                                          properties.isCleanLog());
            files[0].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());
            files[1] = new RecordsFileLog(properties.getOidDbFilePath(),
                                          accessFlags,
                                          properties.isCleanLog());
            files[1].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());

            for (int i = 0; i < nbrDbFiles; i++) {
                files[i + 2] = new RecordsFileLog(properties.getInstanceDbFilePathAt(i),
                                                  accessFlags,
                                                  properties.isCleanLog());
                files[i + 2].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());
            }
        } else {
            files = new RecordsFileLog[1];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(),
                                          accessFlags,
                                          properties.isCleanLog());
            files[0].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());
        }
    }

    public void init(JalistoProperties properties) {
        try {
            String dbPath = properties.getDbFileFullName();
            File f = new File(dbPath);
            this.isNewBase = (!f.exists());
            trace = JalistoFactory.getInternalFactory().getTracer(properties);
            if (isNewBase && properties.isReadOnlyImplementation()) {
                throw new JalistoException("cannot create a database instance in read only mode");
            }
            if (isNewBase) {
                init(properties.getInitialSize(), properties);
            } else {
                init("rw", properties);
            }
            for (int i = 0; i < files.length; i++) {
                files[i].setTrace(trace);
            }
            minilog = new AsynrafLogger(properties, this);
            extentMap = new HashMap();
            trace.println(Trace.DEBUG, "Physical implementation : {0}",
                          PhysicalFileAccessLogAsynrafImpl.class.getName());
            trace.println(Trace.PHYSICAL, "new instance with {0} underlying files", new Integer(files.length));
        } catch (Exception e) {
            throw new JalistoException(e);
        }
    }

    public boolean isNewBase() {
        return isNewBase;
    }

    public void reorganize() {
        trace.println(Trace.PHYSICAL, "reorganize");
        for (int i = 0; i < files.length; i++) {
            try {
                files[i].reorganize();
            } catch (Exception e) {
                throw new JalistoException(
                        "error during reorganization of a physical store file : " + files[i].toString(), e);
            }
        }
    }

    public Collection getKeysStartingWith(String filter) {
        trace.println(Trace.PHYSICAL, "asynraf getKeysStartingWith : filter = {0}", filter);
        Object[] minilogAddresses = minilog.getAddressesForExtent(filter);
        Iterator keys = ((Collection) minilogAddresses[0]).iterator();
        while (keys.hasNext()) {
            extentMap.put(keys.next(), null);
        }
        for (short i = 0; i < files.length; i++) {
            keys = files[i].getKeysStartingWith(filter).iterator();
            while (keys.hasNext()) {
                extentMap.put(keys.next(), null);
            }
        }
        keys = ((Collection) minilogAddresses[1]).iterator();
        while (keys.hasNext()) {
            extentMap.remove(keys.next());
        }
        Collection result = new ArrayList(extentMap.keySet());
        extentMap.clear();
        return result;
    }

    public void insertFileObject(InFileAddress ifa, JalistoObject fo) {
        trace.println(Trace.PHYSICAL, "asynraf insertFileObject({0}, {1})", ifa, fo);
        minilog.insertFileObject(ifa, fo);
    }

    public JalistoObject readFileObjectAt(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "asynraf readFileObjectAt({0})", ifa);
        return minilog.readFileObjectAt(ifa);
    }

    public void updateFileObject(InFileAddress ifa, JalistoObject fo) {
        trace.println(Trace.PHYSICAL, "asynraf updateFileObject({0}, {1})", ifa, fo);
        minilog.updateFileObject(ifa, fo);
    }

    public void deleteFileObject(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "asynraf deleteFileObject({0})", ifa);
        minilog.deleteFileObject(ifa);
    }

    public void startCommit() {
        trace.println(Trace.PHYSICAL, "asynraf startCommit");
        try {
            minilog.startCommitting();
        } catch (IOException e) {
            throw new JalistoException(e);
        }
    }

    public void commit() {
        trace.println(Trace.PHYSICAL, "asynraf commit");
        try {
            minilog.finishCommitting();
        } catch (IOException e) {
            throw new JalistoException(e);
        }
    }

    public void rollback() {
    }

    public void open() {
        minilog.open();
    }

    public void close() {
        minilog.close();
    }

    public AsynrafLogger getMinilog() {
        return minilog;
    }

    public void internalInsert(InFileAddress ifa, byte[] datas) {
        trace.println(Trace.PHYSICAL, "asynraf internalInsert : ifa = {0}", ifa);
        try {
            RecordWriter rw = new RecordWriter(ifa.getAddress());
            rw.writeDatas(datas);
            int i = 0;
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }
            files[i].startBackUp();
            files[i].insertRecord(rw);
            files[i].finishBackUp();
            rw.reset();
        } catch (Exception e) {
            throw new JalistoException("could not insert datas at " + ifa, e);
        }
    }

    public RecordReader internalRead(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "asynraf internalRead : ifa = {0}", ifa);
        RecordReader result;
        try {
            int i = 0;
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }
            result = files[i].readRecord(ifa.getAddress());
        } catch (Exception e) {
            throw new JalistoException("could not read object at " + ifa, e);
        }
        return result;
    }

    public void internalUpdateFileObject(InFileAddress ifa, byte[] datas) {
        trace.println(Trace.PHYSICAL, "asynraf internalUpdateFileObject : ifa = {0}", ifa);
        try {
            RecordWriter rw = new RecordWriter(ifa.getAddress());
            rw.writeDatas(datas);
            int i = 0;
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }
            files[i].startBackUp();
            files[i].updateRecord(rw);
            files[i].finishBackUp();
            rw.reset();
        } catch (Exception e) {
            throw new JalistoException("could not update datas ", e);
        }
    }

    public void internalDeleteFileObject(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "asynraf internalDeleteFileObject : ifa = {0}", ifa);
        try {
            int i = 0;
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }
            files[i].startBackUp();
            files[i].deleteRecord(ifa.getAddress());
            files[i].finishBackUp();
        } catch (Exception e) {
            throw new JalistoException("could not delete object at " + ifa, e);
        }
    }

    public Trace getTrace() {
        return trace;
    }


    public String toString() {
        RecordsFileInspector rfi = new RecordsFileInspector();
        StringWriter sw = new StringWriter();
        PrintWriter out = new PrintWriter(sw);
        try {
            rfi.inspectRecordsFile(out, files[0]);
        } catch (Exception e) {
            out.println("error during inspection of the database : " + e.getMessage());
        }
        out.flush();
        return sw.toString();
    }


    private RecordsFileLog[] files;
    private AsynrafLogger minilog;
    private boolean isNewBase;
    private HashMap extentMap;
    private Trace trace;
}

