/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.asynraf.recover;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.Constants;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.util.JalistoUtils;
import org.objectweb.jalisto.se.storage.raf.RecordsFile;
import org.objectweb.jalisto.se.storage.raf.log.asynraf.AsynrafLogger;
import org.objectweb.jalisto.se.storage.raf.log.asynraf.LogInfo;
import org.objectweb.jalisto.se.storage.raf.log.asynraf.PhysicalFileAccessLogAsynrafImpl;
import org.objectweb.jalisto.se.storage.raf.log.backup.LogBoundary;
import org.objectweb.jalisto.se.storage.raf.log.backup.LogObject;

import java.io.EOFException;
import java.io.File;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;

public class RecoverBase {

    public void recoverBase(JalistoProperties properties) throws Exception {

        String[] pathes = properties.getPathes();

        String datastoreName = properties.getName();
        String logActionFileName = properties.getLogFileName();
        RandomAccessFile logFile = new RandomAccessFile(logActionFileName, "rw");

        short status = logFile.readShort();

        if (status == AsynrafLogger.NOBODY_WORKING) {
            System.out.println("no need to repair datastore " + datastoreName + " -> exit");
            return;
        } else if (status == AsynrafLogger.LOGGER_WORKING) {
            System.out.println("crash during log period -> nothing to undo");
            return;
        } else if (status != AsynrafLogger.DEMON_WORKING) {
            throw new JalistoException("wrong log status : " + status);
        }

        System.out.println("need to repair " + datastoreName + " datastore");

        List logInfos = new ArrayList();

        int indexInfo = 0;
        while (logFile.getFilePointer() < logFile.length()) {
            short actionType = logFile.readShort();
            InFileAddress ifa = new InFileAddress(logFile.readUTF());
            ifa.setFileIndex(logFile.readInt());
            ifa.setIndex(logFile.readInt());
            LogInfo logInfo = new LogInfo(actionType, ifa, indexInfo++);
            logInfos.add(logInfo);
            long size = logFile.readLong();
            byte[] datas = new byte[(int) size];
            logFile.readFully(datas);
            logInfo.setValue(datas);
            if (actionType == LogInfo.UPDATE_TYPE) {
                long oldSize = logFile.readLong();
                byte[] oldDatas = new byte[(int) oldSize];
                logFile.readFully(oldDatas);
                logInfo.setOldValue(oldDatas);
            }
        }

        for (short i = 0; i < pathes.length; i++) {
            String baseFileName = pathes[i];
            String bufferLogFileName = baseFileName + Constants.LOG_POSTFIX;
            RandomAccessFile base = new RandomAccessFile(baseFileName, "rw");
            RandomAccessFile bufferLog = new RandomAccessFile(bufferLogFileName, "r");
            bufferLog.seek(0);

            ArrayList logObjects = new ArrayList();
            boolean readError = false;
            boolean structureError = false;
            boolean hasError;
            while ((!readError) && (bufferLog.getFilePointer() < bufferLog.length())) {
                try {
                    Object lo = LogObject.readFrom(bufferLog);
                    logObjects.add(lo);
                } catch (EOFException e) {
                    readError = true;
                }
            }

            if (!logObjects.isEmpty()) {
                try {
                    LogBoundary boundary = (LogBoundary) logObjects.get(logObjects.size() - 1);
                    structureError = boundary.isBegin();   // must not be begin
                } catch (ClassCastException cce) {      // last logElement is not a boundary
                    structureError = true;
                }
            }

            hasError = readError || structureError;

            if (hasError) {
                System.out.println("recoverying " + baseFileName + " from log file " + bufferLogFileName);
                for (int j = (logObjects.size() - 1); j > -1; j--) {
                    LogObject logObject = (LogObject) logObjects.get(j);
                    logObject.undoOn(base);
                }
            } else {
                System.out.println("no need to recover physical file " + pathes[i]);
            }

            base.close();
            bufferLog.close();
        }

        System.out.println("undo actions from the last transaction :");
        PhysicalFileAccessLogAsynrafImpl physicalAccessInternal = (PhysicalFileAccessLogAsynrafImpl)
                JalistoFactory.getInternalFactory().getPhysicalAccess(properties);

        while (!logInfos.isEmpty()) {
            LogInfo logInfo = (LogInfo) logInfos.remove(logInfos.size() - 1);
            if (logInfo.getActionType() == LogInfo.INSERT_TYPE) {
                try {
                    physicalAccessInternal.internalDeleteFileObject(logInfo.getIfa());
                    System.out.println("undo " + logInfo + " data with size = " +
                                       logInfo.getValueLength());
                } catch (JalistoException e) {
                    if (e.getMessage().startsWith("could not delete object")) {
                        System.out.println("skip undo of " + logInfo);
                    } else {
                        throw e;
                    }
                }
            } else if (logInfo.getActionType() == LogInfo.UPDATE_TYPE) {
                byte[] dataInBase = physicalAccessInternal.internalRead(logInfo.getIfa()).getData();
                if (JalistoUtils.areEquals(dataInBase, logInfo.getValue())) {
                    System.out.println("skip undo of " + logInfo);
                } else {
                    physicalAccessInternal.internalUpdateFileObject(logInfo.getIfa(), logInfo.getOldValue());
                }
            } else if (logInfo.getActionType() == LogInfo.DELETE_TYPE) {
                try {
                    physicalAccessInternal.internalInsert(logInfo.getIfa(), logInfo.getValue());
                    System.out.println("undo " + logInfo + " data with size = " +
                                       logInfo.getValueLength());
                } catch (Exception e) {
                    if (e.getMessage().startsWith("could not insert datas at")) {
                        System.out.println("skip undo of " + logInfo);
                    } else {
                        throw e;
                    }
                }
            } else {
                throw new JalistoException("wrong action type");
            }
        }

        logFile.setLength(0);
        logFile.writeShort(AsynrafLogger.NOBODY_WORKING);

        System.out.println("repair of " + datastoreName + " is finished");


        String recordsFilePath = properties.getSystemDbFilePath();
        File testFile = new File(recordsFilePath + "s");
        if (testFile.exists()) {
            RecordsFile repairedFile = new RecordsFile(recordsFilePath, "rw");
            repairedFile.initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());

            RecordsFile oldFile = new RecordsFile(recordsFilePath + "s", "rw");
            oldFile.initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());

            if (oldFile.equals(repairedFile)) {
                System.out.println("repaired storage is equals to saved storage");
            }
        }
    }

    public static void main(String[] args) {
        try {
            JalistoProperties prop;
            if (args.length < 1) {
                prop = JalistoFactory.getInternalFactory().getProperties("jalisto.properties");
            } else {
                prop = JalistoFactory.getInternalFactory().getProperties(args[0]);
            }
            prop.setProperty(JalistoProperties.PHYSICAL_ACCESS_CLASS_KEY, PhysicalFileAccessLogAsynrafImpl.class.getName());
            (new RecoverBase()).recoverBase(prop);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
