/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.backup;

import java.io.*;

public class LogFileLengthModification extends LogObject {

    public LogFileLengthModification() {
    }

    public LogFileLengthModification(byte[] deletedBytes) {
        this.deletedBytes = deletedBytes;
        this.isShrink = true;
        this.sizeModification = deletedBytes.length;
    }

    public LogFileLengthModification(long sizeModification) {
        this.sizeModification = sizeModification;
        this.isShrink = false;
    }

    public Object internalReadFrom(DataInput in) throws IOException {
        LogFileLengthModification result;
        boolean isShrink = in.readBoolean();
        long sizeModification = in.readLong();
        if (isShrink) {
            byte[] datas = new byte[(new Long(sizeModification)).intValue()];
            in.readFully(datas);
            result = new LogFileLengthModification(datas);
        } else {
            result = new LogFileLengthModification(sizeModification);
        }
        return result;
    }

    public void writeTo(DataOutput out) throws IOException {
        if (isShrink) {
            LogObject.writeLogFileLengthModification(out, deletedBytes);
        } else {
            LogObject.writeLogFileLengthModification(out, sizeModification);
        }
    }

    public void undoOn(RandomAccessFile raf) throws IOException {
        System.out.println("undo " + this.toString());
        if (isShrink) {
            raf.seek(raf.length());
            raf.write(deletedBytes);
        } else {
            raf.setLength(raf.length() - sizeModification);
        }
    }

    public void corrupt(RandomAccessFile raf) throws IOException {
        if (isShrink) {
            raf.seek(raf.length());
            for (int i = 0; i < (deletedBytes.length / 2); i++) {
                raf.writeByte(deletedBytes[i]);
            }
        } else {
            raf.setLength(raf.length() - (sizeModification / 2));
        }
    }


    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("LoggFileLengthModification(");
        sb.append("isShrink=").append(isShrink).append(",");
        sb.append(sizeModification);
        if (isShrink) {
            sb.append(",b[").append(deletedBytes.length).append("]");
        }
        sb.append(")");
        return sb.toString();
    }


    private boolean isShrink;
    private long sizeModification;
    private byte[] deletedBytes;

    static final long serialVersionUID = -7589636669041261459L;
}
