/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.synraf;

import org.objectweb.jalisto.se.api.internal.JalistoObject;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.storage.raf.*;
import org.objectweb.jalisto.se.storage.raf.log.RecordsFileLog;
import org.objectweb.jalisto.se.storage.raf.log.synraf.action.LogAction;

import java.io.*;
import java.util.Collection;

public class PhysicalFileAccessLogSynrafImpl implements PluggablePhysicalFileAccess {

    private void init(int initialSize, JalistoProperties properties)
            throws IOException, RecordsFileException {
        int nbrDbFiles = properties.getInstanceDbFileNumber();
        if (nbrDbFiles > 1) {
            files = new RecordsFileLog[nbrDbFiles + 2];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(), properties.isCleanLog());
            files[0].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());
            files[1] = new RecordsFileLog(properties.getOidDbFilePath(), properties.isCleanLog());
            files[1].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());

            for (int i = 0; i < nbrDbFiles; i++) {
                files[i + 2] = new RecordsFileLog(properties.getInstanceDbFilePathAt(i), properties.isCleanLog());
                files[i + 2].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());
            }
        } else {
            files = new RecordsFileLog[1];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(), properties.isCleanLog());
            files[0].initNewStorage(initialSize, properties.getKeyLength(), properties.getAdditionalSpace());
        }
    }

    private void init(String accessFlags, JalistoProperties properties)
            throws IOException, RecordsFileException {
        int nbrDbFiles = properties.getInstanceDbFileNumber();
        if (nbrDbFiles > 1) {
            files = new RecordsFileLog[nbrDbFiles + 2];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(),
                                          accessFlags,
                                          properties.isCleanLog());
            files[0].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());
            files[1] = new RecordsFileLog(properties.getOidDbFilePath(),
                                          accessFlags,
                                          properties.isCleanLog());
            files[1].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());

            for (int i = 0; i < nbrDbFiles; i++) {
                files[i + 2] = new RecordsFileLog(properties.getInstanceDbFilePathAt(i),
                                                  accessFlags,
                                                  properties.isCleanLog());
                files[i + 2].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());
            }
        } else {
            files = new RecordsFileLog[1];
            files[0] = new RecordsFileLog(properties.getSystemDbFilePath(),
                                          accessFlags,
                                          properties.isCleanLog());
            files[0].initExistingStorage(properties.getKeyLength(), properties.getAdditionalSpace());
        }
    }

    public void init(JalistoProperties properties) {
        try {
            String dbPath = properties.getDbFileFullName();
            File f = new File(dbPath);
            this.isNewBase = (!f.exists());
            trace = JalistoFactory.getInternalFactory().getTracer(properties);
            if (isNewBase && properties.isReadOnlyImplementation()) {
                throw new JalistoException("cannot create a database instance in read only mode");
            }
            if (isNewBase) {
                init(properties.getInitialSize(), properties);
            } else {
                init("rw", properties);
            }
            for (int i = 0; i < files.length; i++) {
                files[i].setTrace(trace);
            }
            logFile = new RandomAccessFile(properties.getLogFileName(), "rw");
            trace.println(Trace.PHYSICAL, "new instance with {0} underlying files", new Integer(files.length));
        } catch (Exception e) {
            throw new JalistoException(e);
        }
    }

    public boolean isNewBase() {
        return isNewBase;
    }

    public void reorganize() {
        trace.println(Trace.PHYSICAL, "synraf reorganize : ");
        for (int i = 0; i < files.length; i++) {
            try {
                files[i].reorganize();
            } catch (Exception e) {
                throw new JalistoException(
                        "error during reorganization of a physical store file : " + files[i].toString(), e);
            }
        }
    }

    public Collection getKeysStartingWith(String filter) {
        trace.println(Trace.PHYSICAL, "synraf getKeysStartingWith : filter = {0}", filter);
        if (files.length > 1) {
            Collection result = files[0].getKeysStartingWith(filter);
            for (short i = 1; i < files.length; i++) {
                result.addAll(files[i].getKeysStartingWith(filter));
            }
            return result;
        } else {
            return files[0].getKeysStartingWith(filter);
        }
    }

    public void insertFileObject(InFileAddress ifa, JalistoObject fo) {
        trace.println(Trace.PHYSICAL, "synraf insertFileObject({0}, {1})", ifa, fo);
        int i = 0;
        try {
            RecordWriter rw = new RecordWriter(ifa.getAddress());
            rw.writeObject(fo);
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }

            LogAction logAction = new LogAction(LogAction.INSERT_ACTION, ifa, fo);

            files[i].startBackUp();
            logAction.writeTo(logFile);
            files[i].insertRecord(rw);
            files[i].finishBackUp();
            rw.reset();
        } catch (Exception e) {
            Object v;
            try {
                RecordReader rr = files[i].readRecord(ifa.getAddress());
                v = rr.readObject();
                rr.reset();
            } catch (Exception e2) {
                throw new JalistoException("could not insert object " + fo, e);
            }
            throw new JalistoException("could not insert object " + fo +
                                           ";\n\t\tvalue already in base : " + String.valueOf(v), e);
        }
    }

    public JalistoObject readFileObjectAt(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "synraf readFileObjectAt : ifa = {0}", ifa);
        JalistoObject result;
        int i = 0;
        try {
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }
            RecordReader rr = files[i].readRecord(ifa.getAddress());
            result = (JalistoObject) rr.readObject();
            result.setIfa(ifa);
            rr.reset();
        } catch (Exception e) {
            throw new JalistoException("could not read object at " + ifa, e);
        }
        return result;
    }

    public void updateFileObject(InFileAddress ifa, JalistoObject fo) {
        trace.println(Trace.PHYSICAL, "synraf updateFileObject({0}, {1})", ifa, fo);
        try {
            RecordWriter rw = new RecordWriter(ifa.getAddress());
            rw.writeObject(fo);
            int i = 0;
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }

            RecordReader rr = files[i].readRecord(ifa.getAddress());
            JalistoObject oldValue = (JalistoObject) rr.readObject();
            LogAction logAction = new LogAction(LogAction.UPDATE_ACTION, ifa, oldValue);

            files[i].startBackUp();
            logAction.writeTo(logFile);
            files[i].updateRecord(rw);
            files[i].finishBackUp();
            rw.reset();
            rr.reset();
        } catch (Exception e) {
            throw new JalistoException("could not update object " + fo, e);
        }
    }

    public void deleteFileObject(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "synraf deleteFileObject : ifa = {0}", ifa);
        try {
            int i = 0;
            if (files.length > 1) {
                i = ifa.getFileIndex();
            }

            RecordReader rr = files[i].readRecord(ifa.getAddress());
            JalistoObject fo = (JalistoObject) rr.readObject();
            rr.reset();
            LogAction logAction = new LogAction(LogAction.DELETE_ACTION, ifa, fo);

            files[i].startBackUp();
            logAction.writeTo(logFile);
            files[i].deleteRecord(ifa.getAddress());
            files[i].finishBackUp();
        } catch (Exception e) {
            throw new JalistoException("could not delete object at " + ifa, e);
        }
    }

    public void startCommit() {
        trace.println(Trace.PHYSICAL, "synraf startCommit : ");
        try {
            logFile.seek(0);
            logFile.setLength(0);
            LogAction logAction = new LogAction(LogAction.BEGIN_ACTION);
            logAction.writeTo(logFile);
        } catch (IOException e) {
            throw new JalistoException(e);
        }
    }

    public void commit() {
        trace.println(Trace.PHYSICAL, "synraf commit : ");
        try {
            LogAction logAction = new LogAction(LogAction.COMMIT_ACTION);
            logAction.writeTo(logFile);
            logFile.setLength(logFile.getFilePointer());
        } catch (IOException e) {
            throw new JalistoException(e);
        }
    }

    public void rollback() {
        trace.println(Trace.PHYSICAL, "synraf rollback : ");
        try {
            LogAction logAction = new LogAction(LogAction.COMMIT_ACTION);
            logAction.writeTo(logFile);
            logFile.setLength(logFile.getFilePointer());
        } catch (IOException e) {
            throw new JalistoException(e);
        }
    }

    public void close() {
    }

    public void open() {
    }


    public String toString() {
        RecordsFileInspector rfi = new RecordsFileInspector();
        StringWriter sw = new StringWriter();
        PrintWriter out = new PrintWriter(sw);
        try {
            rfi.inspectRecordsFile(out, files[0]);
        } catch (Exception e) {
            out.println("error during inspection of the database : " + e.getMessage());
        }
        out.flush();
        return sw.toString();
    }


    private RecordsFileLog[] files;
    private RandomAccessFile logFile;
    private boolean isNewBase;
    private Trace trace;
}

