/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.synraf.action;

import org.objectweb.jalisto.se.api.internal.JalistoObject;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.storage.raf.DbByteArrayInputStream;
import org.objectweb.jalisto.se.storage.raf.DbByteArrayOutputStream;

import java.io.*;

public class LogAction {

    public LogAction() {
    }

    public LogAction(short actionType, InFileAddress ifa, JalistoObject object) {
        this.actionType = actionType;
        this.ifa = ifa;
        this.object = object;
    }

    public LogAction(short actionType) {
        if ((actionType == BEGIN_ACTION) || (actionType == COMMIT_ACTION)) {
            this.actionType = actionType;
        } else {
            throw new JalistoException("not valid action type : " + actionType);
        }
    }

    public short getActionType() {
        return actionType;
    }

    public void setActionType(short actionType) {
        this.actionType = actionType;
    }

    public InFileAddress getIfa() {
        return ifa;
    }

    public void setIfa(InFileAddress ifa) {
        this.ifa = ifa;
    }

    public JalistoObject getObject() {
        return object;
    }

    public void setObject(JalistoObject object) {
        this.object = object;
    }

    public void writeTo(DataOutput dest) throws IOException {
        DbByteArrayOutputStream out = DbByteArrayOutputStream.getInstance();
        ObjectOutputStream objOut = new ObjectOutputStream(out);
        objOut.writeObject(new Short(actionType));
        if ((actionType != BEGIN_ACTION) && (actionType != COMMIT_ACTION)) {
            objOut.writeObject(ifa);
            if (actionType != INSERT_ACTION) {
                objOut.writeObject(object);
            }
        }
        int size = out.size();
        dest.writeInt(size);
        out.writeTo(dest);
        out.reset();
    }

    public void readFrom(DataInput source) throws IOException {
        int size = source.readInt();
        byte[] data = new byte[size];
        source.readFully(data);
        ByteArrayInputStream in = DbByteArrayInputStream.getInstance(data);
        ObjectInputStream objIn = new ObjectInputStream(in);
        try {
            actionType = ((Short) objIn.readObject()).shortValue();
            if ((actionType != BEGIN_ACTION) && (actionType != COMMIT_ACTION)) {
                ifa = (InFileAddress) objIn.readObject();
                if (actionType != INSERT_ACTION) {
                    object = (JalistoObject) objIn.readObject();
                }
            }
        } catch (Exception e) {
            throw new JalistoException(e);
        } finally {
            in.reset();
        }
    }

    public void undoOn(PluggablePhysicalFileAccess physicalAccesInternal) {
        if ((actionType == BEGIN_ACTION) || (actionType == COMMIT_ACTION)) {
            return;
        }
        if (actionType == INSERT_ACTION) {
            physicalAccesInternal.deleteFileObject(ifa);
        } else if (actionType == DELETE_ACTION) {
            physicalAccesInternal.insertFileObject(ifa, object);
        } else if (actionType == UPDATE_ACTION) {
            physicalAccesInternal.updateFileObject(ifa, object);
        } else {
            throw new JalistoException();
        }
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("LogAction(");
        if (actionType == INSERT_ACTION) {
            sb.append("insert");
        } else if (actionType == DELETE_ACTION) {
            sb.append("delete");
        } else if (actionType == UPDATE_ACTION) {
            sb.append("update");
        } else if (actionType == BEGIN_ACTION) {
            sb.append("begin)");
            return sb.toString();
        } else if (actionType == COMMIT_ACTION) {
            sb.append("commit)");
            return sb.toString();
        }
        sb.append(",").append(ifa);
        if (actionType != INSERT_ACTION) {
            sb.append(",").append(object);
        }
        sb.append(")");
        return sb.toString();
    }

    private short actionType;
    private InFileAddress ifa;
    private JalistoObject object;

    public static final short INSERT_ACTION = 0;
    public static final short DELETE_ACTION = 1;
    public static final short UPDATE_ACTION = 2;
    public static final short BEGIN_ACTION = 3;
    public static final short COMMIT_ACTION = 4;


    static final long serialVersionUID = -7699636669241162559L;
}
