/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.synraf.action;

import org.objectweb.jalisto.se.storage.raf.log.backup.LogObject;

import java.io.*;

public class LogRecord extends LogObject {

    public LogRecord() {
    }

    public LogRecord(long address, byte[] before, int newDatasLength) {
        this.address = address;
        this.before = before;
        this.newDatasLength = newDatasLength;
    }

    public long getAddress() {
        return address;
    }

    public void setAddress(long address) {
        this.address = address;
    }

    public byte[] getBefore() {
        return before;
    }

    public void setBefore(byte[] before) {
        this.before = before;
    }

    public int getNewDatasLength() {
        return newDatasLength;
    }

    public void setNewDatasLength(int newDatasLength) {
        this.newDatasLength = newDatasLength;
    }

    public Object internalReadFrom(DataInput in) throws IOException {
        long address = in.readLong();
        int newDatasLength = in.readInt();
        byte[] before = new byte[in.readInt()];
        in.readFully(before);
        return new LogRecord(address, before, newDatasLength);
    }

    public void writeTo(DataOutput out) throws IOException {
        LogObject.writeLogRecord(out, address, before, newDatasLength);
    }

    public void undoOn(RandomAccessFile raf) throws IOException {
        System.out.println("undo " + this.toString());
        if (raf.length() == (address + newDatasLength)) {
            raf.setLength(address);
        }
        raf.seek(address);
        raf.write(before);
    }

    public void corrupt(RandomAccessFile raf) throws IOException {
        byte[] newDatas = new byte[newDatasLength / 2];
        raf.seek(address);
        raf.readFully(newDatas);

        if (raf.length() == (address + newDatasLength)) {
            raf.setLength(address);
        }
        raf.seek(address);
        raf.write(before);

        raf.seek(address);
        raf.write(newDatas);
    }


    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("LogRecord(");
        sb.append(address).append(",");
        sb.append(newDatasLength).append(",");
        sb.append("b[").append(before.length).append("])");
        return sb.toString();
    }


    private long address;
    private int newDatasLength;
    private byte[] before;


    static final long serialVersionUID = -7589636669041161459L;
}
