/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.test.data;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.meta.type.MetaTypeFactory;

import java.io.Serializable;

public class Book implements Serializable {
    private Book() {
    }

    private Book(String title, int pages, int price, String author) {
        this.pages = pages;
        this.title = title;
        this.price = price;
        this.author = author;
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public int getPrice() {
        return price;
    }

    public void setPrice(int price) {
        this.price = price;
    }

    public int getPages() {
        return pages;
    }

    public void setPages(int pages) {
        this.pages = pages;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public static Book newBook() {
        counter++;
        return new Book(getNewTitle(counter),
                        getNewPages(counter),
                        getNewPrice(counter),
                        getNewAuthor(counter));
    }

    public Object[] toArray() {
        Object[] result = new Object[4];
        result[0] = title;
        result[1] = new Integer(pages);
        result[2] = new Integer(price);
        result[3] = author;
        return result;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("'").append(title);
        sb.append("from ").append(author).append(", ");
        sb.append(pages).append(" pages, ");
        sb.append(price).append(" E");
        sb.append("'");
        return sb.toString();
    }

    public static Book toBook(Object[] array) {
        Book book = new Book();
        book.setTitle((String) array[0]);
        book.setPages(((Integer) array[1]).intValue());
        book.setPrice(((Integer) array[2]).intValue());
        book.setAuthor((String) array[3]);
        return book;
    }

    public static ClassDescription getMetaDescription() {
        ClassDescription meta = JalistoFactory.createClassDescription(Book.class.getName());
        meta.addField(JalistoFactory.createFieldDescription("title", MetaTypeFactory.StringType));
        meta.addField(JalistoFactory.createFieldDescription("pages", MetaTypeFactory.IntegerType));
        meta.addField(JalistoFactory.createFieldDescription("price", MetaTypeFactory.IntegerType));
        meta.addField(JalistoFactory.createFieldDescription("author", MetaTypeFactory.StringType));
        return meta;
    }

    private static String getNewTitle(int c) {
        return titles[c % titles.length];
    }

    private static String getNewAuthor(int c) {
        return authors[c % authors.length];
    }

    private static int getNewPrice(int c) {
        return prices[c % prices.length];
    }

    public static int getNewPages(int c) {
        return (c % maxPage);
    }

    private String title;
    private int pages;
    private int price;
    private String author;

    public static int counter = -1;
    public static final String[] titles = {"Death Off Stage",
                                           "At the Edge",
                                           "Fingersmith",
                                           "Metes and Bounds",
                                           "A Fountain Filled with Blood",
                                           "A Cold Case of Murder",
                                           "Innocent Hearts",
                                           "Tipping the Velvet",
                                           "Bittersweet",
                                           "The Jester"};
    public static final int[] prices = {5,
                                        10,
                                        15,
                                        20,
                                        25,
                                        30};
    public static final String[] authors = {"James McManus",
                                            "John Sandford",
                                            "Robert Dallek",
                                            "Andy Andrews"};
    public static final int maxPage = 1000;


    static final long serialVersionUID = -7589376666964761459L;
}
