/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.test.data;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.meta.type.MetaTypeFactory;

import java.io.Serializable;

public class BookWithFormat implements Serializable {
    private BookWithFormat() {
    }

    private BookWithFormat(String title, int pages, int price, Format format) {
        this.pages = pages;
        this.title = title;
        this.price = price;
        this.format = format;
    }

    public Author getAuthor() {
        return author;
    }

    public void setAuthor(Author author) {
        this.author = author;
    }

    public int getPrice() {
        return price;
    }

    public void setPrice(int price) {
        this.price = price;
    }

    public int getPages() {
        return pages;
    }

    public void setPages(int pages) {
        this.pages = pages;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public Format getFormat() {
        return format;
    }

    public void setFormat(Format format) {
        this.format = format;
    }

    public boolean equals(Object o) {
        try {
            BookWithFormat candidate = (BookWithFormat) o;
            boolean titleTest = ((candidate.title != null) && (candidate.title.equals(title))) ||
                                ((candidate.title == null) && (title == null));
            boolean intTest = (candidate.pages == pages) && (candidate.price == price);
            boolean authorTest = ((candidate.author != null) && (candidate.author.equals(author))) ||
                                 ((candidate.author == null) && (author == null));
            boolean formatTest = ((candidate.format != null) && (candidate.format.equals(format))) ||
                                 ((candidate.format == null) && (format == null));

            return (titleTest && intTest && authorTest && formatTest);
        } catch (Exception e) {
        }
        return false;
    }

    public Object[] toArray() {
        Object[] result = new Object[5];
        result[0] = title;
        result[1] = new Integer(pages);
        result[2] = new Integer(price);
        result[3] = author;
        result[4] = format;
        return result;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("'").append(title);
        sb.append("from ").append(author).append(", ");
        sb.append(pages).append(" pages, ");
        sb.append(price).append(" E");
        sb.append("'");
        return sb.toString();
    }

    public static BookWithFormat toBook(Object[] array) {
        BookWithFormat book = new BookWithFormat();
        book.setTitle((String) array[0]);
        book.setPages(((Integer) array[1]).intValue());
        book.setPrice(((Integer) array[2]).intValue());
        book.setAuthor((Author) array[3]);
        book.setFormat((Format) array[4]);
        return book;
    }

    public static ClassDescription getMetaDescription() {
        ClassDescription meta = JalistoFactory.createClassDescription(BookWithFormat.class.getName());
        meta.addField(JalistoFactory.createFieldDescription("title", MetaTypeFactory.StringType));
        meta.addField(JalistoFactory.createFieldDescription("pages", MetaTypeFactory.IntegerType));
        meta.addField(JalistoFactory.createFieldDescription("price", MetaTypeFactory.IntegerType));
        meta.addField(JalistoFactory.createFieldDescription("author", MetaTypeFactory.LinkType));
        meta.addField(JalistoFactory.createFieldDescription("format", MetaTypeFactory.SerializedType));
        return meta;
    }

    public static BookWithFormat newBook() {
        counter++;
        return new BookWithFormat(getNewTitle(counter),
                                  getNewPages(counter),
                                  getNewPrice(counter),
                                  Format.getFormat());
    }

    private static String getNewTitle(int c) {
        return titles[c % titles.length];
    }

    private static int getNewPrice(int c) {
        return prices[c % prices.length];
    }

    public static int getNewPages(int c) {
        return (c % maxPage);
    }

    private String title;
    private int pages;
    private int price;
    private Author author;
    private Format format;

    public static int counter = -1;
    public static final String[] titles = {"Death Off Stage",
                                           "At the Edge",
                                           "Fingersmith",
                                           "Metes and Bounds",
                                           "A Fountain Filled with Blood",
                                           "A Cold Case of Murder",
                                           "Innocent Hearts",
                                           "Tipping the Velvet",
                                           "Bittersweet",
                                           "The Jester"};
    public static final int[] prices = {5,
                                        10,
                                        15,
                                        20,
                                        25,
                                        30};

    public static final int maxPage = 1000;


    static final long serialVersionUID = -7589461099514761459L;
}
