/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.test.threads;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.test.data.Book;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public abstract class BookClientThread extends Thread {

    public BookClientThread(String jalistoPropertiesFilename, int numClient,
                            int nbrBook, int nbrAction, int nbrIteration) {
        this.jalistoPropertiesFilename = jalistoPropertiesFilename;
        this.nbrBook = nbrBook;
        this.numClient = numClient;
        this.nbrCreate = nbrAction;
        this.nbrRead = nbrAction;
        this.nbrUpdate = nbrAction;
        this.nbrDelete = nbrAction;
        this.nbrIteration = nbrIteration;
        this.continueWorking = true;
        this.withExtent = false;
        oids = new ArrayList();
    }

    public void define(boolean withDefineClass) {
        System.out.println("JalistoClientThread " + numClient + " : define");
        session = JalistoFactory.getSession(jalistoPropertiesFilename);
        session.openSession();
        tx = session.currentTransaction();
        if (withDefineClass) {
            session.defineClass(Book.getMetaDescription());
        }
    }

    public void cleanUp() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).readFully().iterator();
        while (extent.hasNext()) {
            session.deleteObjectByOid(extent.next());
        }
        tx.commit();
    }

    public void populate() {
        System.out.println("JalistoClientThread " + numClient + " : populateJalisto");
        tx.begin();
        oids.clear();
        if (nbrBook > 0) {
            for (int i = 0; i < nbrBook; i++) {
                oids.add(session.createObject(Book.newBook().toArray(), Book.class));
            }
        } else {
            oids.addAll(session.getExtent(Book.class).readFully().collection());
        }
        tx.commit();
    }

    public void run() {
        try {
            stabilityTest();
        } catch (Exception e) {
            e.printStackTrace();
        }
        if ((tx != null) && tx.isActive()) {
            tx.rollback();
        }
        if (session != null) {
            session.closeSession();
        }
    }

    public void stopWorking() {
        continueWorking = false;
    }

    public Session getSession() {
        return session;
    }

    public abstract void stabilityTest();

    /**
     * ************************************************************************************
     */

    public List getOids() {
        return oids;
    }

    public void setOids(List oids) {
        this.oids = oids;
    }

    public void setWithTimer(boolean withTimer) {
        this.withTimer = withTimer;
    }

    public void setWithRead(boolean withRead) {
        this.withRead = withRead;
    }

    public void setNbrCreate(int nbrCreate) {
        this.nbrCreate = nbrCreate;
    }

    public void setNbrDelete(int nbrDelete) {
        this.nbrDelete = nbrDelete;
    }

    public void setNbrRead(int nbrRead) {
        this.nbrRead = nbrRead;
    }

    public void setNbrUpdate(int nbrUpdate) {
        this.nbrUpdate = nbrUpdate;
    }

    public boolean isWithExtent() {
        return withExtent;
    }

    public void setWithExtent(boolean withExtent) {
        this.withExtent = withExtent;
    }


    protected String jalistoPropertiesFilename;
    protected int numClient;

    protected int nbrBook;
    protected int nbrIteration;
    protected int nbrCreate;
    protected int nbrRead;
    protected int nbrUpdate;
    protected int nbrDelete;
    protected boolean withExtent;

    protected boolean withTimer = true;
    protected boolean withRead = false;

    protected boolean continueWorking;

    protected Session session;
    protected Transaction tx;

    protected List oids;
}
