/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.tool;

import org.objectweb.jalisto.se.api.remote.JalistoServer;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.factory.InternalRemoteFactory;
import org.objectweb.jalisto.se.impl.client.JalistoPropertiesClientImpl;

import java.io.PrintStream;

public class ServerLauncher {

    public static void main(String[] args) {
        try {
            parseArgs(args, System.out);
        } catch (Exception e) {
            printUsage(System.out);
            System.exit(0);
        }
        JalistoServer server = InternalRemoteFactory.getInstance().getJalistoServer(communicationFactoryClassName);
        server.launchServer(port);
    }

    private static void parseArgs(String[] args, PrintStream writer) {
        if (args.length == 0) {
            writer.println("no arguments. Uses default values");
        }
        communicationFactoryClassName = JalistoPropertiesClientImpl.COMMUNICATION_FACTORY_CLASS;
        port = Integer.parseInt(JalistoPropertiesClientImpl.PORT);
        for (int i = 0; i < args.length; i++) {
            String o = args[i];
            if (o.equalsIgnoreCase(CLASS_OPTION)) {
                communicationFactoryClassName = args[i + 1];

                i++;
            } else if (o.equalsIgnoreCase(PORT_OPTION)) {
                port = Integer.parseInt(args[i + 1]);
                i++;
            } else {
                throw new JalistoException("FileViewer : invalid argument format : " + o);
            }
        }
        writer.println("Use implementation class : " + communicationFactoryClassName);
        writer.println("Listen on port : " + port);
        writer.flush();
    }

    public static void printUsage(PrintStream writer) {
        writer.println(
                "java org.objectweb.jalisto.se.tool.ServerLauncher <option> : launch socket server for remote clients");
        writer.println("\t" + CLASS_OPTION + " <path> : server will use this implementation class");
        writer.println("\t" + PORT_OPTION + " <number> : server listen on this port");
        writer.flush();
    }

    private static String communicationFactoryClassName;
    private static int port;

    private static final String CLASS_OPTION = "-class";
    private static final String PORT_OPTION = "-port";
}
