/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.remote;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.test.data.Book;

import java.util.ArrayList;
import java.util.Iterator;

public class TestRemote {

    public static void main(String[] args) {
        Session session = JalistoFactory.getSession("jalisto-client.properties");
        session.openSession();
        session.defineClass(Book.getMetaDescription());

        for (int i = 0; i < 5; i++) {
            createAndReadBook(session);
        }

        for (int i = 0; i < 3; i++) {
            ArrayList oids = createBooks(session, 5);
            updateBooks(session, oids);
            deleteBooks(session, oids);
        }

        {
            ArrayList oids = createBooks(session, 20);
            updateBooks(session, oids);
            testExtent(session);
        }

        session.closeSession();
    }

    public static void createAndReadBook(Session session) {
        session.currentTransaction().begin();
        Object oid = session.createObject(Book.newBook().toArray(), Book.class);
        session.currentTransaction().commit();

        session.currentTransaction().begin();
        System.out.println(Book.toBook(session.readObjectByOid(oid)));
        session.currentTransaction().commit();

        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public static ArrayList createBooks(Session session, int nbrBooks) {
        ArrayList oids = new ArrayList();

        session.currentTransaction().begin();
        for (int i = 0; i < nbrBooks; i++) {
            Object oid = session.makeNewFileOid(Book.class);
            System.out.println("make oid : " + oid);
            oids.add(oid);
        }
        session.currentTransaction().commit();

        session.currentTransaction().begin();
        for (int i = 0; i < nbrBooks; i++) {
            Book book = Book.newBook();
            System.out.println("create : " + oids.get(i) + " " + book);
            session.createObject(oids.get(i), book.toArray());
        }
        session.currentTransaction().commit();

        return oids;
    }

    public static void updateBooks(Session session, ArrayList oids) {
        session.currentTransaction().begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            Book book = Book.toBook(session.readObjectByOid(oid));
            book.setAuthor(book.getAuthor() + "pouet");
            System.out.println("update : " + book);
            session.updateObjectByOid(oid, book.toArray());
        }
        session.currentTransaction().commit();
    }

    public static void deleteBooks(Session session, ArrayList oids) {
        session.currentTransaction().begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            System.out.println("delete : " + oid);
            session.deleteObjectByOid(oid);
        }
        session.currentTransaction().commit();
    }

    public static void testExtent(Session session) {
        session.currentTransaction().begin();
        Iterator extent = session.getExtent(Book.class).readFully().iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            Book book = Book.toBook(session.readObjectByOid(oid));
            System.out.println("readed book : " + String.valueOf(book));
        }
        session.currentTransaction().commit();
    }
}
