/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.remote.other;

import org.objectweb.jalisto.se.exception.JalistoException;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.Random;

public class TestSocketServer implements Runnable {

    public TestSocketServer(int port) {
        random = new Random();
        try {
            this.ss = new ServerSocket(port);
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
    }

    public void run() {
        System.out.println("Run server");
        while (true) {
            String socketInfos = null;
            try {
                socketInfos = connect();
                try {
                    while (true) {
                        Object message = readObject();
                        System.out.println("\nread : " + message.toString() + "\n");
                        sendReply();
                    }
                } catch (IOException ioe) {
                    if (ioe instanceof SocketException) {
                        throw ioe;
                    } else {
                        System.out.println(ioe.getMessage());
                    }
                } catch (ClassNotFoundException cnfe) {
                    cnfe.printStackTrace();
                    return;
                }
            } catch (SocketException se) {
                if (se.getMessage().equals("Connection reset")) {
                    System.out.println("socket " + socketInfos + " closed");
                } else {
                    throw new RuntimeException(se);
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
                return;
            }
        }
    }

    public void sendReply() throws IOException {
        int delay = random.nextInt(900) + 100;
        try {
            Thread.sleep(delay);
        } catch (InterruptedException e) {
        }
        toClient.writeObject("ok");
        toClient.flush();
    }

    public String connect() throws IOException {
        socket = ss.accept();
        System.out.println("Connect with " + socket.toString());
        fromClient = new ObjectInputStream(socket.getInputStream());
        toClient = new ObjectOutputStream(socket.getOutputStream());
        System.out.println("socket connected : " + socket.toString());
        return socket.toString();
    }

    public Object readObject() throws IOException, ClassNotFoundException {
        Object message = null;
        while (message == null) {
            message = fromClient.readObject();
        }
        return message;
    }


    private ServerSocket ss;
    private Socket socket;
    private ObjectInputStream fromClient;
    private ObjectOutputStream toClient;

    private Random random;

    public static void main(String[] args) {
        TestSocketServer server = new TestSocketServer(6000);
        server.run();
    }
}
