/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.single;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.io.DataOutput;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

public class BenchRemote {
    public static void main(String[] args) {
        String jalistoPropertiesFilename = args[0];
        int nbrAction = Integer.parseInt(args[1]);
        int nbrExec = Integer.parseInt(args[2]);
        int nbrMeasure = Integer.parseInt(args[3]);

        BenchRemote bench = new BenchRemote(jalistoPropertiesFilename, nbrAction, nbrExec, nbrMeasure);
        bench.runAll();
    }

    public BenchRemote(String propertiesPath, int nbrAction, int nbrExec, int nbrMeasure) {
        session = JalistoFactory.getSession(propertiesPath);
        session.openSession();
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
        oids = new ArrayList();
        random = new Random();
        createTimes = new float[nbrMeasure];
        deleteTimes = new float[nbrMeasure];
        readTimes = new float[nbrMeasure];
        this.nbrAction = nbrAction;
        this.nbrExec = nbrExec;
    }

    public void runAll() {
        readOids();
        for (int i = 0; i < createTimes.length; i++) {
            runCycle();
            JalistoTimer.summary();
            createTimes[i] = JalistoTimer.getAverage(timerNameCreate) / nbrAction;
            readTimes[i] = JalistoTimer.getAverage(timerNameRead) / nbrAction;
            deleteTimes[i] = JalistoTimer.getAverage(timerNameDelete) / nbrAction;
        }
        try {
            RandomAccessFile out = new RandomAccessFile("bench-out-" + System.currentTimeMillis() + ".txt", "rw");
            printExecutionInfos(out);
            printResults(out, "create", createTimes);
            printResults(out, "read", readTimes);
            printResults(out, "delete", deleteTimes);
        } catch (IOException e) {
            e.printStackTrace();
        }
        session.closeSession();
    }

    public void printResults(DataOutput out, String message, float[] datas) throws IOException {
        StringBuffer sb = new StringBuffer();
        sb.append(message).append(";");
        for (int i = 0; i < datas.length; i++) {
            sb.append(datas[i]);
            if (i != datas.length - 1) {
                sb.append(";");
            }
        }
        sb.append("\n");
        out.writeBytes(sb.toString());
    }

    public void printExecutionInfos(DataOutput out) throws IOException {
        JalistoProperties prop = session.getInternalSession().getProperties();
        StringBuffer sb = new StringBuffer();
        sb.append("nbrAction;").append(nbrAction).append("\n");
        sb.append("nbrExec;").append(nbrExec).append("\n");
        sb.append(JalistoProperties.PAGE_CACHE_SIZE_KEY).append(";");
        sb.append(prop.getPageCacheSize()).append("\n");
        sb.append(JalistoProperties.OBJECT_INST_PAGE_SIZE_KEY).append(";");
        sb.append(prop.getInstancePageSize()).append("\n");
        sb.append("\n");
        out.writeBytes(sb.toString());
    }

    public ArrayList getRandomSubCollection(ArrayList list, int card, boolean remove) {
        ArrayList subCollection = new ArrayList();
        int size = list.size();
        for (int i = 0; i < card; i++) {
            if (remove) {
                subCollection.add(list.remove(random.nextInt(list.size())));
            } else {
                subCollection.add(list.get(random.nextInt(size)));
            }
        }
        return subCollection;
    }


    public void readOids() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).readFully().iterator();
        while (extent.hasNext()) {
            oids.add(extent.next());
        }
        tx.commit();
    }

    public void runCycle() {
        System.out.println("");
        JalistoTimer.deleteAllTimers();

        System.out.println("oids collection size = " + oids.size());

        timerNameCreate = "execution cycle (" + nbrAction + "," + nbrExec + ") - create : ";
        timerNameRead = "execution cycle (" + nbrAction + "," + nbrExec + ") - read : ";
        timerNameDelete = "execution cycle (" + nbrAction + "," + nbrExec + ") - delete : ";
        JalistoTimer.createTimer(timerNameCreate);
        JalistoTimer.createTimer(timerNameRead);
        JalistoTimer.createTimer(timerNameDelete);

        for (int i = 0; i < nbrExec; i++) {
            JalistoTimer.timerStart(timerNameCreate);
            tx.begin();
            for (int j = 0; j < nbrAction; j++) {
                Object[] bookInArray = Book.newBook().toArray();
                Object oid = session.createObject(bookInArray, Book.class);
                oids.add(oid);
            }
            tx.commit();
            tx.begin();
            tx.commit();
            JalistoTimer.timerStop(timerNameCreate, false);

            ArrayList subCollectionRead = getRandomSubCollection(oids, nbrAction, false);
            JalistoTimer.timerStart(timerNameRead);
            tx.begin();
            session.readObjectsByOids(subCollectionRead);
            tx.commit();
            tx.begin();
            tx.commit();
            JalistoTimer.timerStop(timerNameRead, false);

            ArrayList subCollectionDelete = getRandomSubCollection(oids, nbrAction, true);
            JalistoTimer.timerStart(timerNameDelete);
            tx.begin();
            for (int j = 0; j < nbrAction; j++) {
                session.deleteObjectByOid(subCollectionDelete.get(j));
            }
            tx.commit();
            tx.begin();
            tx.commit();
            JalistoTimer.timerStop(timerNameDelete, false);
        }
    }


    private String timerNameCreate;
    private String timerNameRead;
    private String timerNameDelete;

    private int nbrAction;
    private int nbrExec;

    private float[] createTimes;
    private float[] readTimes;
    private float[] deleteTimes;

    private Session session;
    private Transaction tx;
    private ArrayList oids;

    private Random random;
}
