/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.single;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

public class OneClientTest {
    public static void main(String[] args) {
        String jalistoPropertiesFilename = args[0];
        int nbrIteration = Integer.parseInt(args[1]);
        int nbrBook = Integer.parseInt(args[2]);
        int nbrRead = Integer.parseInt(args[3]);
        int nbrCreate = Integer.parseInt(args[4]);
        int nbrDelete = Integer.parseInt(args[5]);

        OneClientTest client = new OneClientTest(jalistoPropertiesFilename, nbrBook, 0, nbrIteration);
        client.setWithTimer(true);
        client.setNbrRead(nbrRead);
        client.setNbrCreate(nbrCreate);
        client.setNbrDelete(nbrDelete);
        client.define(true);
        client.populate();

        client.stabilityTest();
    }

    public OneClientTest(String jalistoPropertiesFilename, int nbrBook, int nbrAction, int nbrIteration) {
        this.jalistoPropertiesFilename = jalistoPropertiesFilename;
        this.nbrBook = nbrBook;
        this.nbrCreate = nbrAction;
        this.nbrRead = nbrAction;
        this.nbrDelete = nbrAction;
        this.nbrIteration = nbrIteration;
        oids = new ArrayList();
    }

    public void define(boolean withDefineClass) {
        System.out.println("OneClientTest : define");
        session = JalistoFactory.getSession(jalistoPropertiesFilename);
        session.openSession();
        tx = session.currentTransaction();
        if (withDefineClass) {
            session.defineClass(Book.getMetaDescription());
        }
    }

    public void cleanUp() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).readFully().iterator();
        while (extent.hasNext()) {
            session.deleteObjectByOid(extent.next());
        }
        tx.commit();
        session.reorganize();
    }

    public void populate() {
        System.out.println("OneClientTest : populate " + nbrBook);
        tx.begin();
        if (nbrBook > 0) {
            for (int i = 0; i < nbrBook; i++) {
                oids.add(session.createObject(Book.newBook().toArray(), Book.class));
            }
        } else {
            oids.addAll(session.getExtent(Book.class).readFully().collection());
        }
        tx.commit();
    }

    public void stabilityTest() {
        System.out.println("OneClientTest ");

        JalistoFactory.launchJMXServer(session);

        String timerNameCreate = "BookClientBasic execution cycle - create " + nbrCreate;
        String timerNameRead = "BookClientBasic execution cycle - read " + nbrRead;
        String timerNameDelete = "BookClientBasic execution cycle - delete " + nbrDelete;
        if (withTimer) {
            JalistoTimer.createTimer(timerNameCreate);
            JalistoTimer.createTimer(timerNameDelete);
            JalistoTimer.createTimer(timerNameRead);
        }
        JalistoTimer.createTimer("get floid");
        JalistoTimer.createTimer("read");

        Random random = new Random();

        for (int c = 0; (nbrIteration < 0) || (c < nbrIteration); c++) {
            if (withTimer && (c != 0) && ((c % 50) == 0)) {
                System.out.println("");
                JalistoTimer.summary();
                JalistoTimer.clean(timerNameCreate);
                JalistoTimer.clean(timerNameRead);
                JalistoTimer.clean(timerNameDelete);
            }

            if (nbrCreate > 0) {
                if (withTimer) {
                    JalistoTimer.timerStart(timerNameCreate);
                }
                tx.begin();
                for (int i = 0; i < nbrCreate; i++) {
                    Object[] bookInArray = Book.newBook().toArray();
                    Object oid = session.createObject(bookInArray, Book.class);
                    oids.add(oid);
                }
                tx.commit();
                if (withTimer) {
                    JalistoTimer.timerStop(timerNameCreate, false);
                }
            }

            if (nbrRead > 0) {
                if (withTimer) {
                    JalistoTimer.timerStart(timerNameRead);
                }
                tx.begin();
                for (int i = 0; i < nbrRead; i++) {
                    int index = random.nextInt(oids.size());
                    Object oid = oids.get(index);
                    session.readObjectByOid(oid);
                }
                tx.commit();
                if (withTimer) {
                    JalistoTimer.timerStop(timerNameRead, false);
                }
            }

            if (nbrDelete > 0) {
                if (withTimer) {
                    JalistoTimer.timerStart(timerNameDelete);
                }
                tx.begin();
                for (int i = 0; i < nbrDelete; i++) {
                    int index = random.nextInt(oids.size());
                    Object oid = oids.get(index);
                    session.deleteObjectByOid(oid);
                    oids.remove(index);
                }
                tx.commit();
                if (withTimer) {
                    JalistoTimer.timerStop(timerNameDelete, false);
                }
            }
        }

        session.closeSession();
    }


    public void setWithTimer(boolean withTimer) {
        this.withTimer = withTimer;
    }

    public void setWithRead(boolean withRead) {
        this.withRead = withRead;
    }

    public void setNbrCreate(int nbrCreate) {
        this.nbrCreate = nbrCreate;
    }

    public void setNbrDelete(int nbrDelete) {
        this.nbrDelete = nbrDelete;
    }

    public void setNbrRead(int nbrRead) {
        this.nbrRead = nbrRead;
    }


    protected ArrayList oids;

    protected int nbrBook;
    protected int nbrIteration;
    protected int nbrCreate;
    protected int nbrRead;
    protected int nbrDelete;

    protected boolean withTimer = true;
    protected boolean withRead = false;

    protected String jalistoPropertiesFilename;
    protected Session session;
    protected Transaction tx;
}
