/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.internal.InternalMetaRepository;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.server.IdentityProvider;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;

import java.util.*;

public class BasicFunctionnalityTestCase extends JalistoTestCase {

    public BasicFunctionnalityTestCase() {
    }

    public BasicFunctionnalityTestCase(String name) {
        super(name);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        endTransaction();
        oids.clear();
        books.clear();
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        BasicFunctionnalityTestCase tc =
                (BasicFunctionnalityTestCase) newTestCase(suite, new BasicFunctionnalityTestCase());

        tc.define();

        tc.bookBaseTestWithDelete(10);
        tc.bookBaseTestGetOid(10);
        tc.testUpdate();
        tc.testWrongRead();
        tc.testReserveFloid();
        tc.testPointer();
        tc.testPointerCollection();
        tc.cleanUp();

        tc.testRemoveOverOidTableSize();
        tc.cleanUp();

        tc.testMakeOidExtentAndDelete();
        tc.cleanUp();

        tc.testBigExtent(20000);
        tc.cleanUp();

        tc.deleteOptiTest();
        tc.cleanUp();
        tc.makeOidOverInstancePageSize();

        tc.testNotSerialisable();

        tc.finishTests();

        return suite;
    }

    public void testNotSerialisable() {
        tx.begin();
        Object[] o = Book.newBook().toArray();
        o[0] = new NotSerialisableData("test");
        try {
            session.createObject(o, Book.class);
            assertTrue("should raise exception", false);
        } catch (JalistoException jalistoExc) {
            assertTrue("exception's messages must be equal",
                       jalistoExc.getMessage().startsWith("not serializabel class in data : "));
        }
        tx.commit();
    }

    public void bookBaseTestWithDelete(int num) {
        ArrayList oids = new ArrayList();
        tx.begin();
        for (int i = 0; i < num; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();

        HashMap books = new HashMap();
        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            books.put(oid, Book.toBook(session.readObjectByOid(oid)));
        }
        tx.commit();

        String r1 = "{0.2='Metes and Boundsfrom Andy Andrews, 3 pages, 20 E', 0.4='A Cold Case of Murderfrom John Sandford, 5 pages, 30 E', 0.9='Death Off Stagefrom Robert Dallek, 10 pages, 25 E', 0.8='The Jesterfrom John Sandford, 9 pages, 20 E', 0.6='Tipping the Velvetfrom Andy Andrews, 7 pages, 10 E', 0.1='Fingersmithfrom Robert Dallek, 2 pages, 15 E', 0.3='A Fountain Filled with Bloodfrom James McManus, 4 pages, 25 E', 0.7='Bittersweetfrom James McManus, 8 pages, 15 E', 0.5='Innocent Heartsfrom Robert Dallek, 6 pages, 5 E', 0.0='At the Edgefrom John Sandford, 1 pages, 10 E'}";
        assertEquals("should be equals", r1, String.valueOf(books));

        tx.begin();
        String r2 = "[0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]";
        assertEquals("should be equals", r2, String.valueOf(session.getExtent(Book.class).readFully()));
        tx.commit();

        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            session.deleteObjectByOid(oid);
        }
        tx.commit();

        tx.begin();
        assertTrue("should be empty", session.getExtent(Book.class).readFully().isEmpty());
        tx.commit();
    }

    public void bookBaseTestGetOid(int num) {
        tx.begin();
        for (int i = 0; i < num; i++) {
            oids.add(session.makeNewFileOid(Book.class));
        }
        tx.commit();

        tx.begin();
        for (int i = 0; i < num; i++) {
            session.createObject(oids.get(i), Book.newBook().toArray());
        }
        tx.commit();

        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            books.put(oid, Book.toBook(session.readObjectByOid(oid)));
        }
        tx.commit();

        String r1 = "{0.17='Bittersweetfrom Robert Dallek, 18 pages, 5 E', 0.15='Innocent Heartsfrom James McManus, 16 pages, 25 E', 0.13='A Fountain Filled with Bloodfrom Robert Dallek, 14 pages, 15 E', 0.19='Death Off Stagefrom James McManus, 20 pages, 15 E', 0.11='Fingersmithfrom James McManus, 12 pages, 5 E', 0.16='Tipping the Velvetfrom John Sandford, 17 pages, 30 E', 0.18='The Jesterfrom Andy Andrews, 19 pages, 10 E', 0.14='A Cold Case of Murderfrom Andy Andrews, 15 pages, 20 E', 0.10='At the Edgefrom Andy Andrews, 11 pages, 30 E', 0.12='Metes and Boundsfrom John Sandford, 13 pages, 10 E'}";
        assertEquals("should be equals", r1, String.valueOf(books));

        tx.begin();
        String r2 = "[0.10, 0.11, 0.12, 0.13, 0.14, 0.15, 0.16, 0.17, 0.18, 0.19]";
        assertEquals("should be equals", r2, String.valueOf(session.getExtent(Book.class).readFully()));
        tx.commit();
    }

    public void testUpdate() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).readFully().iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            Book book = Book.toBook(session.readObjectByOid(oid));
            book.setPrice(book.getPrice() + 100);
            session.updateObjectByOid(oid, book.toArray());
        }
        tx.commit();

        tx.begin();
        extent = session.getExtent(Book.class).readFully().iterator();
        ArrayList books = new ArrayList();
        while (extent.hasNext()) {
            Object oid = extent.next();
            books.add(Book.toBook(session.readObjectByOid(oid)));
        }
        tx.commit();

        String r1 = "['At the Edgefrom Andy Andrews, 11 pages, 130 E', 'Fingersmithfrom James McManus, 12 pages, 105 E', 'Metes and Boundsfrom John Sandford, 13 pages, 110 E', 'A Fountain Filled with Bloodfrom Robert Dallek, 14 pages, 115 E', 'A Cold Case of Murderfrom Andy Andrews, 15 pages, 120 E', 'Innocent Heartsfrom James McManus, 16 pages, 125 E', 'Tipping the Velvetfrom John Sandford, 17 pages, 130 E', 'Bittersweetfrom Robert Dallek, 18 pages, 105 E', 'The Jesterfrom Andy Andrews, 19 pages, 110 E', 'Death Off Stagefrom James McManus, 20 pages, 115 E']";
        assertEquals("should be equals", r1, String.valueOf(books));
    }

    public void testBigExtent(int nbrBook) {
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            session.createObject(Book.newBook().toArray(), Book.class);
        }
        tx.commit();

        tx.begin();
        Iterator extent = session.getExtent(Book.class).readFully().iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            session.deleteObjectByOid(oid);
        }
        tx.commit();
    }

    public void testRemoveOverOidTableSize() {
        int size = session.getInternalSession().getProperties().getOidTableSize() * 2;
        ArrayList oids = new ArrayList();
        tx.begin();
        for (int i = 0; i < size; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();

        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            session.readObjectByOid(oids.get(i));
        }
        tx.commit();
    }

    public void testMakeOidExtentAndDelete() {
        tx.begin();
        Object sessionId = session.getInternalSession().getSessionId();
        Object clid = ((InternalMetaRepository)session.getMetaRepository()).getClidFromClassName(Book.class.getName());
        Collection txOids = session.getInternalSession().getOidTable().getFloidsFromClid(sessionId, clid, true);
        int initialSize = txOids.size();
        tx.commit();

        tx.begin();
        for (int i = 0; i < 10; i++) {
            session.makeNewFileOid(Book.class);
        }
        tx.commit();

        tx.begin();
        assertEquals("should be equal", 0, session.getExtent(Book.class).readFully().size());
        tx.commit();

        tx.begin();
        txOids = session.getInternalSession().getOidTable().getFloidsFromClid(sessionId, clid, true);
        assertEquals("should be equal", 10 + initialSize, txOids.size());
        tx.commit();

        tx.begin();
        Iterator toRemove = txOids.iterator();
        while(toRemove.hasNext()) {
            session.deleteObjectByOid(toRemove.next());
        }
        tx.commit();
    }

    public void testReserveFloid() {
        if (session.getInternalSession().isRemoteSession()) {
            System.out.println("can't execute this test with remote client");
            return;
        }
        int number = 25;
        String bookClassName = Book.getMetaDescription().getClassName();
        InternalMetaRepository metaRepository = ((InternalMetaRepository)session.getMetaRepository());
        short bookClid = ((Short)metaRepository.getClidFromClassName(bookClassName)).shortValue();
        tx.begin();
        IdentityProvider identityProvider = session.getInternalSession().getOidTable().getIdentityProvider();
        long currentFloidCounter = identityProvider.getCurrentFloidCounter(bookClid);
        long firstReservedFloidValue = session.getInternalSession().reserveFloids(bookClid, number).longValue();
        assertEquals("floid's initial value must be equal", firstReservedFloidValue, currentFloidCounter);
        long afterReserveLongValue = identityProvider.getCurrentFloidCounter(bookClid);
        assertEquals("must be equal", currentFloidCounter + number, afterReserveLongValue);
        tx.commit();

        tx.begin();
        LogicalOid floid01 = (LogicalOid) session.createObject(Book.newBook().toArray(), bookClassName);
        assertEquals("floids must be equals", afterReserveLongValue, floid01.getFloidAsLong());

        LogicalOid floid02 = (LogicalOid) session.getInternalSession().makeNewFileOid(new LogicalOid(firstReservedFloidValue));
        session.createObject(floid02, Book.newBook().toArray());
        assertEquals("floids must be equals", firstReservedFloidValue, floid02.getFloidAsLong());
        tx.commit();
    }

    public void testWrongRead() {
        tx.begin();
        LogicalOid readOid = (LogicalOid) session.makeNewFileOid(Book.class);
        LogicalOid wrongOid = new LogicalOid(readOid.getFloidAsLong()+1);
        tx.commit();

        tx.begin();
        try {
            session.readObjectByOid(wrongOid);
            assertTrue("should raise exception", false);
        } catch (IdentityException jalistoIdentityExc) {
            assertEquals("exception's messages must be equal",
                         "the given oid 0.21 doesn't exist in this Jalisto datastore",
                         jalistoIdentityExc.getMessage());
        }
        try {
            Object[] result = session.readObjectByOid(readOid);
            assertTrue("should raise exception", false);
        } catch (IdentityException jalistoIdentityExc) {
            assertEquals("exception's messages must be equal",
                         "the given oid 0.20 doesn't exist in this Jalisto datastore",
                         jalistoIdentityExc.getMessage());
        }
        tx.commit();
    }

    public void testPointer() {
        Object[] book = Book.newBook().toArray();

        tx.begin();
        LogicalOid bookOid = (LogicalOid) session.createObject(book, Book.getMetaDescription().getClassName());
        Object newValue = new Integer(456);
        book[0] = newValue;
        Object[] bookInDatastore = session.readObjectByOid(bookOid);
        tx.commit();

        assertTrue("must not be equal", book != bookInDatastore);
        assertTrue("must not be equal", !newValue.equals(bookInDatastore[0]));
        // ok parce que le tableau est dupliqu
    }

    public void testPointerCollection() {
        Object[] book = Book.newBook().toArray();
        ArrayList list = new ArrayList();
        list.add("tralala");
        list.add("trilili");
        book[0] = list;

        tx.begin();
        LogicalOid bookOid = (LogicalOid) session.createObject(book, Book.getMetaDescription().getClassName());
        list.add("trololo");
        Object[] bookInDatastore = session.readObjectByOid(bookOid);
        tx.commit();

        System.out.println(bookInDatastore[0]);

        assertTrue(" : EXPECTED FAILURE ON THIS TEST : must not be equal", list != bookInDatastore[0]);
        // pas ok parce que les objets du tableau ne sont pas dupliqus !!!
        assertEquals("size must be equals to 2", 2, ((ArrayList)bookInDatastore[0]).size());
        assertTrue("must not contains", ((ArrayList)bookInDatastore[0]).contains("trololo"));
    }

    public void deleteOptiTest() {
        tx.begin();
        Object oid = session.createObject(Book.newBook().toArray(), Book.class);
        session.deleteObjectByOid(oid);
        try {
            tx.commit();         // TODO : ne doit pas lancer d'exception !!!
        } catch (Exception e) {
            if (tx.isActive()) {
                tx.rollback();
            }
            assertTrue(" : MINOR FAILURE TO FIX SOON : "+e.getMessage(), false);
        }
    }

    public void makeOidOverInstancePageSize() {
        int size = session.getInternalSession().getProperties().getInstancePageSize() * 2;
        tx.begin();
        for (int i = 0; i < size; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();

        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            session.deleteObjectByOid(oids.get(i));
        }
        tx.commit();
    }

    private ArrayList oids = new ArrayList();
    private HashMap books = new HashMap();

    /**
     * **************************************************************************************
     */

    public void define() {
        super.initSession(true);
        super.define(Book.getMetaDescription());
        Book.counter = 0;
    }

    public void cleanUp() {
        super.cleanUp(Book.class);
        oids.clear();
        books.clear();
    }

    public void finishTests() {
        super.finishTests();
    }
}
