/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.Extent;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.exception.TransactionException;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;

import java.util.ArrayList;

public class ExtentTestCase extends JalistoTestCase {

    private static final int number_of_book = 20;

    public ExtentTestCase() {
    }

    public ExtentTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        ExtentTestCase tc = (ExtentTestCase) newTestCase(suite, new ExtentTestCase());

        tc.define();
        tc.cleanUp();
        tc.populate(number_of_book);

        tc.testBasicExtent01(number_of_book);
        tc.testBasicExtent02(number_of_book);
        tc.testBasicExtent03(number_of_book);

        tc.testCursorExtent01(number_of_book);
        tc.testCursorExtent02(number_of_book);
        tc.testCursorExtent03(number_of_book);

        tc.testTransactionnalExtent01(number_of_book);
        tc.testTransactionnalExtent02(number_of_book);
        tc.testTransactionnalExtent03(number_of_book);

        tc.finishTests();

        return suite;
    }

    public void testBasicExtent01(int nbr) {
        try {
            session.getExtent(Book.class);
            assertTrue("must raise exception", false);
        } catch (TransactionException jalistoTransactionExc) {
            assertEquals("must be equal", "Transaction must be active during getExtent", jalistoTransactionExc.getMessage());
        }
    }

    public void testBasicExtent02(int nbr) {
        tx.begin();
        Extent extent = session.getExtent(Book.class).readFully();
        assertEquals("must be equal", nbr, extent.size());
        tx.commit();
    }

    public void testBasicExtent03(int nbr) {
        tx.begin();
        Extent extent = session.getExtent(Book.class);
        assertEquals("must be equal", 0, extent.size());
        tx.commit();
        try {
            extent.readFully();
            assertTrue("must raise exception", false);
        } catch (TransactionException jalistoTransactionExc) {
            assertEquals("must be equal", "Transaction must be active during readFully", jalistoTransactionExc.getMessage());
        }
        tx.begin();
        extent.readFully();
        tx.commit();
        assertEquals("must be equal", nbr, extent.size());
    }

    public void testCursorExtent01(int nbr) {
        if (session.getInternalSession().isRemoteSession()) {
            System.out.println("can't execute this test with remote client");
            return;
        }

        tx.begin();
        Extent extent = session.getExtent(Book.class).readNextFloids(nbr/2);
        assertEquals("must be equal", nbr/2, extent.size());
        tx.commit();

        ArrayList oids1 = new ArrayList(extent.collection());
        extent.cleanFloids();
        assertEquals("must be equal", 0, extent.size());

        tx.begin();
        extent.readNextFloids(10);
        assertEquals("must be equal", nbr/2, extent.size());
        ArrayList oids2 = new ArrayList(extent.collection());
        tx.commit();

        int errors = 0;
        for (int i = 0; i < oids1.size(); i++) {
            if (oids2.contains(oids1.get(i))) {
                errors++;
            }
        }
        assertEquals("have errors", 0, errors);
    }

    public void testCursorExtent02(int nbr) {
        if (session.getInternalSession().isRemoteSession()) {
            System.out.println("can't execute this test with remote client");
            return;
        }

        tx.begin();
        Extent extent = session.getExtent(Book.class).readNextFloids(nbr/2);
        assertEquals("must be equal", nbr/2, extent.size());
        tx.commit();

        try {
            extent.readNextFloids(nbr/2);
            assertTrue("must raise exception", false);
        } catch (TransactionException jalistoTransactionExc) {
            assertEquals("must be equal", "Transaction must be active during readNextFloids", jalistoTransactionExc.getMessage());
        }

        tx.begin();
        extent.readNextFloids(10);
        assertEquals("must be equal", nbr, extent.size());
        tx.commit();
    }

    public void testCursorExtent03(int nbr) {
        if (session.getInternalSession().isRemoteSession()) {
            System.out.println("can't execute this test with remote client");
            return;
        }

        tx.begin();
        Extent extent = session.getExtent(Book.class).readNextFloids(nbr+5);
        assertEquals("must be equal", nbr, extent.size());
        tx.commit();
    }

    public void testTransactionnalExtent01(int nbr) {
        tx.begin();
        ArrayList oids = new ArrayList(session.getExtent(Book.class).readFully().collection());
        tx.commit();

        assertEquals("must be equal", nbr, oids.size());
        LogicalOid oid = (LogicalOid) oids.get(0);

        tx.begin();
        session.deleteObjectByOid(oid);
        Extent extent = session.getExtent(Book.class).readFully();
        assertEquals("must be equal", nbr-1, extent.size());
        assertTrue("must not contains value", !extent.contains(oid));
        tx.commit();

        populate(1);
    }

    public void testTransactionnalExtent02(int nbr) {
        tx.begin();
        ArrayList oids = new ArrayList(session.getExtent(Book.class).readFully().collection());
        tx.commit();

        assertEquals("must be equal", nbr, oids.size());

        tx.begin();
        LogicalOid oid01 = (LogicalOid) session.createObject(Book.newBook().toArray(), Book.class);
        LogicalOid oid02 = (LogicalOid) session.createObject(Book.newBook().toArray(), Book.class);
        Extent extent = session.getExtent(Book.class).readFully();
        assertEquals("must be equal", nbr+2, extent.size());
        assertTrue("must contains value", extent.contains(oid01));
        assertTrue("must contains value", extent.contains(oid02));
        // TODO : BUG opti qd on laisse le delete ici !!!!!!!!
//        session.deleteObjectByOid(oid01);
//        session.deleteObjectByOid(oid02);
        tx.commit();

        tx.begin();
        session.deleteObjectByOid(oid01);
        session.deleteObjectByOid(oid02);
        tx.commit();
    }

    public void testTransactionnalExtent03(int nbr) {
        tx.begin();
        ArrayList oids = new ArrayList(session.getExtent(Book.class).readFully().collection());
        tx.commit();

        assertEquals("must be equal", nbr, oids.size());

        {
            tx.begin();
            LogicalOid oid01 = (LogicalOid) session.makeNewFileOid(Book.class);
            Extent extent = session.getExtent(Book.class).readFully();
            assertEquals("must be equal", nbr, extent.size());
            assertTrue("must not contains value", !extent.contains(oid01));
            tx.rollback();
        }

        {
            tx.begin();
            LogicalOid oid01 = (LogicalOid) session.makeNewFileOid(Book.class);
            tx.commit();
            tx.begin();
            Extent extent = session.getExtent(Book.class).readFully();
            assertEquals("must be equal", nbr, extent.size());
            assertTrue("must not contains value", !extent.contains(oid01));
            tx.commit();
            tx.begin();
            session.deleteObjectByOid(oid01);
            tx.commit();
        }
    }

    /**
     * **************************************************************************************
     */

    public void define() {
        super.initSession(false);
        super.define(Book.getMetaDescription());
    }

    public void cleanUp() {
        super.cleanUp(Book.class);
    }

    public void populate(int nbr) {
        super.populate(nbr);
    }

    public void finishTests() {
        super.finishTests();
    }
}
