/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.api.Extent;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.JalistoFactory;

public class ReadOnlyTestCase extends JalistoTestCase {

    private final static int number_of_books = 50;

    public ReadOnlyTestCase() {
    }

    public ReadOnlyTestCase(String name) {
        super(name);
    }

    protected void tearDownOnce() throws Exception {
        super.tearDownOnce();
        System.out.println("tearDownOnce for ReadOnly test");
        if ((roTx != null) && (roTx.isActive())) {
            roTx.rollback();
        }
        if ((roSession != null) && (roSession.isOpen())) {
            roSession.closeSession();
        }
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        ReadOnlyTestCase tc = (ReadOnlyTestCase) newTestCase(suite, new ReadOnlyTestCase());

        tc.initSession();

        tc.populate(number_of_books);

        tc.initRoSession();
        tc.extentTest(number_of_books);

        tc.finishTests();

        return suite;
    }

    public void initRoSession() {
        endTransaction();
        closeSession();
        JalistoFactory.getInternalFactory().cleanFactory();
        roSession = JalistoFactory.getSession("jalisto-ro.properties");
        roSession.openSession();
        roTx = roSession.currentTransaction();
    }

    public void extentTest(int number) {
        roTx.begin();
        Extent extent = roSession.getExtent(Book.class).readFully();
        assertEquals("must be equal", number, extent.size());
        roTx.commit();
    }

    private Transaction roTx;
    private Session roSession;

    /*****************************************************************************************************/

    public void initSession() {
        super.initSession(true);
        define(Book.getMetaDescription());
    }

    public void cleanUp(Class aClass) {
        super.cleanUp(aClass);
    }

    public void finishTests() {
        super.finishTests();
    }

    public void populate(int i) {
        super.populate(i);
    }
}
