/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

public class RestructureTestCase extends JalistoTestCase {

    public RestructureTestCase() {
    }

    public RestructureTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        RestructureTestCase tc = (RestructureTestCase) newTestCase(suite, new RestructureTestCase());

        tc.newSession();
        tc.define();

        tc.cleanUp();

        tc.populate(500);

        tc.stabilityTest(50, 100);
        tc.reorganize();

        tc.cleanUp();
        tc.reorganize();

        tc.finishTests();

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void newSession() {
        System.gc();
        session = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session.openSession();
        tx = session.currentTransaction();
    }

    public void define() {
        session.defineClass(Book.getMetaDescription());
    }

    public void reorganize() {
        if (session.getInternalSession().getProperties().allowsSpecialFunctionnalities()) {
            session.closeSession();
            session.reorganize();
            session.openSession();
        } else {
            System.out.println("not allows reorganize functionnality");
        }
    }

    public void cleanUp() {
        oids = new ArrayList();
        tx.begin();
        Iterator extent = session.getExtent(Book.class).readFully().iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            session.deleteObjectByOid(oid);
        }
        tx.commit();
    }

    public void populate(int nbrBook) {
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();
    }

    public void stabilityTest(int nbrCreate, int nbrIt) {
        Random random = new Random();

        for (int c = 0; c < nbrIt; c++) {
            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                Object[] bookInArray = Book.newBook().toArray();
                Object oid = session.createObject(bookInArray, Book.class);
                oids.add(oid);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.readObjectByOid(oid);
            }
            tx.commit();
        }
    }

    public void finishTests() {
        if ((session != null) && (session.isOpen())) {
            session.closeSession();
        }
    }

    ArrayList oids = null;
}
