/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.exception.remote.RemoteException;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;

import java.util.ArrayList;
import java.util.Iterator;

public class RollbackTestCase extends JalistoTestCase {

    public RollbackTestCase() {
    }

    public RollbackTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        RollbackTestCase tc = (RollbackTestCase) newTestCase(suite, new RollbackTestCase());

        tc.define();
        tc.cleanUp();

        tc.populateRollback(700);
        tc.checkCard(0);

        tc.populate(1000);
        tc.checkCard(1000);

        tc.deleteRollback(500);
        tc.checkCard(1000);

        tc.delete(500);
        tc.checkCard(500);

        tc.cleanUp();
        tc.populateRollback(3000);
        tc.checkCard(0);

        tc.populate(3000);
        tc.checkCard(3000);

        tc.cleanUp();
        tc.testMakeOidRollback(1000);

        tc.cleanUp();
        tc.createOverInstancePageSize();

        tc.cleanUp();
        tc.finishTests();

        return suite;
    }

    public void populateRollback(int nbr) {
        tx.begin();
        for (int i = 0; i < nbr; i++) {
            session.createObject(Book.newBook().toArray(), Book.class);
        }
        tx.rollback();
    }

    public void checkCard(int card) {
        tx.begin();
        assertEquals("should contains " + card + " books", card, session.getExtent(Book.class).readFully().size());
        tx.commit();
    }

    public void deleteRollback(int card) {
        tx.begin();
        Iterator extentIt = session.getExtent(Book.class).readFully().iterator();
        int i = 0;
        while (extentIt.hasNext() && (i < card)) {
            session.deleteObjectByOid(extentIt.next());
            i++;
        }
        tx.rollback();
    }

    public void delete(int card) {
        tx.begin();
        Iterator extentIt = session.getExtent(Book.class).readFully().iterator();
        int i = 0;
        while (extentIt.hasNext() && (i < card)) {
            session.deleteObjectByOid(extentIt.next());
            i++;
        }
        tx.commit();
    }

    public void makeFloidRollback(int nbr) {
        tx.begin();
        for (int i = 0; i < nbr; i++) {
            session.makeNewFileOid(Book.class);
        }
        tx.rollback();
    }

    public void testMakeOidRollback(int nbr) {
        ArrayList oids = new ArrayList();
        tx.begin();
        for (int i = 0; i < nbr; i++) {
            oids.add(session.makeNewFileOid(Book.class));
        }
        tx.rollback();

        tx.begin();
        assertEquals("should be equal", 0, session.getExtent(Book.class).readFully().size());
        tx.commit();

        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            try {
                session.deleteObjectByOid(oid);
                if (!session.getInternalSession().isRemoteSession()) {
                    assertTrue("must raise exception", false);
                }
            } catch (IdentityException jalistoIdentityExc) {
                assertEquals("should be equal", "the given oid " + oid + " doesn't exist in this Jalisto datastore",
                             jalistoIdentityExc.getMessage());
            }
        }
        try {
            tx.commit();
        } catch (RemoteException jalistoRemoteExc) {
            if (session.getInternalSession().isRemoteSession()) {
                assertTrue("should be equal", jalistoRemoteExc.getMessage().startsWith("the given oid ") &&
                                              jalistoRemoteExc.getMessage().endsWith(" doesn't exist in this Jalisto datastore"));
                tx.rollback();
            }
        }
    }

    public void createOverInstancePageSize() {
        int size = session.getInternalSession().getProperties().getInstancePageSize() * 2;
        ArrayList oids = new ArrayList();
        tx.begin();
        for (int i = 0; i < size; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.rollback();

        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            try {
                session.deleteObjectByOid(oid);
            } catch (IdentityException jalistoIdentityExc) {
                assertEquals("should be equal", "the given oid " + oid + " doesn't exist in this Jalisto datastore",
                             jalistoIdentityExc.getMessage());
            }
        }
        tx.commit();
        oids.clear();

        tx.begin();
        for (int i = 0; i < size; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();

        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            session.deleteObjectByOid(oids.get(i));
        }
        tx.commit();
    }


    /**
     * ***********************************************************************************
     */

    public void define() {
        super.initSession(false);
        super.define(Book.getMetaDescription());
    }

    public void cleanUp() {
        super.cleanUp(Book.class);
    }

    public void populate(int nbr) {
        super.populate(nbr);
    }

    public void finishTests() {
        super.finishTests();
    }
}
