/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.query.QueryManager;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.exception.JcaException;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.impl.util.JalistoUtils;

import javax.resource.ResourceException;
import javax.resource.cci.*;
import javax.resource.spi.ManagedConnection;
import javax.transaction.xa.XAException;
import java.util.Collection;

public class JalistoConnection implements javax.resource.cci.Connection, Session {

    public JalistoConnection(JalistoManagedConnection jalistoMc, Trace trace) {
        this.trace = trace;
        this.jalistoMc = jalistoMc;
    }

    public void setMc(ManagedConnection mc) {
        if (mc instanceof JalistoManagedConnection) {
            this.jalistoMc = (JalistoManagedConnection) mc;
        } else {
            throw new JcaException("Cannot manage connection of this class : " + mc.getClass().toString());
        }
    }

    public void open(JalistoManagedConnection jalistoMc) throws ResourceException {
        this.jalistoMc = jalistoMc;
        isValide = true;
        isClosed = false;
    }

    public void invalidate() throws ResourceException {
        jalistoMc = null;
        isValide = false;
    }

    public boolean isClosed() {
        return isClosed;
    }

    private final Session getSession() {
        try {
            checkClosedOrNotValid();
        } catch (ResourceException e) {
            new JcaException(CLOSED_EXCEPTION_MSG);
        }
        Session result = jalistoMc.getSession();
        if (result == null) {
            throw new JcaException("PM null in connection... You may be outside of a transaction");
        }
        return result;
    }

    private void checkClosedOrNotValid() throws ResourceException {
        if ((isClosed || (!isValide))) {
            throw new ResourceException(CLOSED_EXCEPTION_MSG);
        }
    }

    private static String addressOf(Object o) {
        String s = String.valueOf(o);
        int index = s.indexOf('@');
        if (index == -1)
            return s;
        return JalistoUtils.substring(s, index);
    }


    public String toString() {
        return "JalistoConnection" + addressOf(super.toString()) +
               " ManagedConnection" + addressOf(jalistoMc) + " ManagedPM" +
               addressOf(jalistoMc == null ? null : jalistoMc.getSession());
    }

    /*************************** Connection implementation   **************************************/

    public LocalTransaction getLocalTransaction() throws ResourceException {
        checkClosedOrNotValid();
        return (javax.resource.cci.LocalTransaction) jalistoMc.getLocalTransaction();
    }

    public synchronized void close() {
        trace.println(Trace.JCA, TRACE_ID + " closing connection");
        try {
            checkClosedOrNotValid();
        } catch (ResourceException e) {
            new JcaException(CLOSED_EXCEPTION_MSG);
        }
        try {
            jalistoMc.close(this);
        } catch (XAException e) {
            e.printStackTrace();
        }
        isClosed = true;
        trace.println(Trace.JCA, TRACE_ID + " connection closed...");
    }

    public Interaction createInteraction() throws ResourceException {
        checkClosedOrNotValid();
        return null;
    }

    public ConnectionMetaData getMetaData() throws ResourceException {
        checkClosedOrNotValid();
        return null;
    }

    public ResultSetInfo getResultSetInfo() throws ResourceException {
        checkClosedOrNotValid();
        return null;
    }

    public void setAutoCommit(boolean b) throws ResourceException {
        checkClosedOrNotValid();
    }

    public boolean getAutoCommit() throws ResourceException {
        checkClosedOrNotValid();
        return false;
    }

    /***************************   Session Implementation ***************************************/

    public void closeSession() {
        getSession().closeSession();
    }

    public void openSession() {
        getSession().openSession();
    }

    public boolean isOpen() {
        return getSession().isOpen();
    }

    public boolean contains(Object oid) {
        return getSession().contains(oid);
    }

    public Object createObject(Object[] objectToCreate, Class objectClass) {
        return getSession().createObject(objectToCreate, objectClass);
    }

    public Object createObject(Object[] objectToCreate, String objectClassName) {
        return getSession().createObject(objectToCreate, objectClassName);
    }

    public Object createObject(Object oid, Object[] objectToCreate) {
        return getSession().createObject(oid, objectToCreate);
    }

    public Transaction currentTransaction() {
        return getSession().currentTransaction();
    }

    public void defineClass(ClassDescription classDescription) {
        getSession().defineClass(classDescription);
    }

    public void deleteObjectByOid(Object oid) {
        getSession().deleteObjectByOid(oid);
    }

    public String getClassNameFor(Object oid) {
        return null;
    }

    public Extent getExtent(String fullCLassName) {
        return getSession().getExtent(fullCLassName);
    }

    public Extent getExtent(Class theClass) {
        return getSession().getExtent(theClass);
    }

    public SessionInternal getInternalSession() {
        return getSession().getInternalSession();
    }

    public QueryManager getQueryManager() {
        return getSession().getQueryManager();
    }

    public MetaRepository getMetaRepository() {
        return getSession().getMetaRepository();
    }

    public boolean isClassDefined(String fullQualifiedClassName) {
        return getSession().isClassDefined(fullQualifiedClassName);
    }

    public boolean isNewBase() {
        return getSession().isNewBase();
    }

    public Object makeNewFileOid(Class objectClass) {
        return getSession().makeNewFileOid(objectClass);
    }

    public Object makeNewFileOid(String objectClassName) {
        return getSession().makeNewFileOid(objectClassName);
    }

    public Object[] readObjectByOid(Object oid) {
        return getSession().readObjectByOid(oid);
    }

    public void removeClass(String fullClassName) {
        getSession().removeClass(fullClassName);
    }

    public void reorganize() {
        getSession().reorganize();
    }

    public Object updateObjectByOid(Object oid, Object[] objectToUpdate) {
        return getSession().updateObjectByOid(oid, objectToUpdate);
    }

    public Collection readObjectsByOids(Collection oids) {
        return getSession().readObjectsByOids(oids);
    }

    public Object[] refreshObjectByOid(Object oid) {
        return getSession().refreshObjectByOid(oid);
    }

    public void eraseStorage() {
        getSession().eraseStorage();
    }


    private static final String TRACE_ID = "[JalistoConnection]";
    private static final String CLOSED_EXCEPTION_MSG = "Connection already closed or invalidated...";
    private JalistoManagedConnection jalistoMc;
    private Trace trace;
    private boolean isClosed = false;
    private boolean isValide = true;
}
