/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.exception.JcaException;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.impl.trace.TraceNull;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.jca.data.JalistoManagedConnectionMetaData;
import org.objectweb.jalisto.se.jca.session.ManagedSession;

import javax.resource.ResourceException;
import javax.resource.spi.*;
import javax.security.auth.Subject;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

public class JalistoManagedConnection implements ManagedConnection {

    public JalistoManagedConnection(String connectionURL, ConnectionRequestInfo info, Trace trace) {
        this.connectionURL = connectionURL;
        this.trace = trace;
        this.info = info;
        metadata = new JalistoManagedConnectionMetaData();
        try {
            getXAResource();
        } catch (ResourceException e) {
            throw new JcaException("Can't initialize the JalistoManagedConnection", e);
        }
    }

    public void setManagedJalistoSession(ManagedSession managedJalistoSession) {
        this.managedJalistoSession = managedJalistoSession;
    }

    public boolean isAvalaible() {
        return (isAvalaible && (managedJalistoSession == null) && jdoxarWrapper.isAvailable());
    }

    public void setNotAvalaible() {
        isAvalaible = false;
    }

    public void setCfIdentification(String id) {
        cfIdentification = id;
    }

    public String getCfIdentification() {
        return cfIdentification;
    }

    public String getConnectionURL() {
        return connectionURL;
    }


    public void close(JalistoConnection c) throws XAException {
        trace("close(c)");
        if (!c.equals(activeJalistoConnection)) {
            throw new XAException("Cannot close a not active connection...");
        }
        for (int i = 0; i < eventListeners.size(); i++) {
            ConnectionEventListener cel = (ConnectionEventListener) eventListeners.get(i);
            ConnectionEvent ce = new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
            ce.setConnectionHandle(c);
            cel.connectionClosed(ce);
        }
    }

    public Session getSession() {
        return managedJalistoSession;
    }

    public boolean matchConnectionRequestInfo(ConnectionRequestInfo requestInfo) {
        if (info == null) {
            return (requestInfo == null);
        }
        // info != null
        return info.equals(requestInfo);
    }

    private void trace(Object o) {
        String message = TRACE_ID + " " + String.valueOf(o);
        trace.println(Trace.JCA, message);
    }


    public boolean equals(Object o) {
        boolean result = false;
        if (o instanceof JalistoManagedConnection) {
            JalistoManagedConnection candidat = (JalistoManagedConnection) o;
            result = (this.hashCode() == candidat.hashCode());
        }
        return result;
    }

    public int hashCode() {
        if (hashCode == 0) {
            hashCode = super.hashCode();
        }
        return hashCode;
    }


    /*****************************    ManagedConnection implementation ****************************/

    // pas de connection sharing pour l'instant (p40 et 71 de la spec)
    public Object getConnection(Subject subject, ConnectionRequestInfo info) throws ResourceException {
        trace("getConnection");
        if (activeJalistoConnection == null) {
            activeJalistoConnection = new JalistoConnection(this, trace);
        } else {
            if (activeJalistoConnection.isClosed()) {
                activeJalistoConnection.open(this);
            }
            //throw new JDOUserException("not the first call to getConnection, but don't support connection sharing");
        }
        return activeJalistoConnection;
    }

    public void associateConnection(Object c) throws ResourceException {
        trace("associateConnection " + c.getClass().toString());

        if (c == null) {
            throw new ResourceException("Cannot associate a null connection");
        }

        if (activeJalistoConnection == null) {
            activeJalistoConnection = (JalistoConnection) c;
            return;
        }

        if (activeJalistoConnection.equals(c)) {
            return;
        }

        if (c instanceof JalistoConnection) {
            JalistoConnection jdoJalistoConnection = (JalistoConnection) c;
            jdoJalistoConnection.setMc(this);
            activeJalistoConnection.invalidate();
            activeJalistoConnection = jdoJalistoConnection;
        } else {
            throw new ResourceException("The managedConnection cannot manage this connection : not a JDOConnection");
        }
    }

    public XAResource getXAResource() throws ResourceException {
        trace("getXAResource");
        if (jdoxarWrapper == null) {
            trace("getXAResource : new ressource");
            jdoxarWrapper = new JalistoXAResourceWrapper(this, trace);
        }
        return jdoxarWrapper;
    }

    public void addConnectionEventListener(ConnectionEventListener listener) {
        trace("addConnectionEventListener");
        eventListeners.add(listener);
    }

    public void removeConnectionEventListener(ConnectionEventListener listener) {
        trace("removeConnectionEventListener");
        eventListeners.remove(listener);
    }

    public void cleanup() throws ResourceException {
        trace("cleanup");
        if (activeJalistoConnection != null) {
            activeJalistoConnection.invalidate();
            activeJalistoConnection = null;
        }
        jdoxarWrapper.cleanUp();
        managedJalistoSession = null;
        isAvalaible = true;
    }

    public void destroy() throws ResourceException {
        trace("destroy");
        jdoxarWrapper = null;
        isAvalaible = false;
        eventListeners.clear();
        logger = null;
    }

    // must be spi.LocalTransaction
    public LocalTransaction getLocalTransaction() throws ResourceException {
        throw new RuntimeException("not implemented yet...");
    }

    public ManagedConnectionMetaData getMetaData() throws ResourceException {
        trace("getMetaData : " + metadata.toString());
        return metadata;
    }

    public void setLogWriter(PrintWriter writer) throws ResourceException {
        trace("setLogWriter");
        logger = writer;
    }

    public PrintWriter getLogWriter() throws ResourceException {
        trace("getLogWriter");
        return logger;
    }


    private ManagedSession managedJalistoSession;
    private JalistoConnection activeJalistoConnection = null;
    private JalistoXAResourceWrapper jdoxarWrapper;

    private String connectionURL;
    private String cfIdentification;
    private ConnectionRequestInfo info;
    private JalistoManagedConnectionMetaData metadata;

    private List eventListeners = new ArrayList();

    private PrintWriter logger;
    private Trace trace = new TraceNull();


    private boolean isAvalaible = true;
    private int hashCode = 0;

    private static final String TRACE_ID = "[JalistoManagedConnection]";
}
