/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.impl.trace.TraceNull;
import org.objectweb.jalisto.se.jca.session.*;

import javax.transaction.xa.Xid;
import java.util.*;

public class JalistoXARPool {

    public JalistoXARPool() {
    }

    public static synchronized JalistoXAResource getBoundXAResource(Xid xid, JalistoManagedConnection mc) {
        String connectionId = mc.getCfIdentification();
        trace.println(Trace.JCAPOOL, TRACE_ID + " getBoundXAResource with {0}", connectionId);
        ConnectionFactoryRecord record = getRecord(connectionId);
        Map cfMap = record.getAssociatedXAResources();
        JalistoXAResource result = (JalistoXAResource) cfMap.get(xid);
        trace.println(Trace.JCAPOOL, TRACE_ID + " return result : {0}", result);
        return result;
    }

    public static void setJdoConnectionFactory(String key, JalistoConnectionFactory jalistoConnectionFactory) {
        ConnectionFactoryRecord record = new ConnectionFactoryRecord(jalistoConnectionFactory);
        identifier2connectionFactoryRecordMap.put(key, record);
    }

    public static synchronized JalistoXAResource getOrBindJalistoXAResource(Xid xid, JalistoManagedConnection mc) {
        String mcId = mc.getCfIdentification();
        JalistoXAResource xaResource;

        trace.println(Trace.JCAPOOL, TRACE_ID + " getOrBindJalistoXAResource");
        trace.println(Trace.JCAPOOL,
                      TRACE_ID + " available XA resources in pool: {0}",
                      new Integer(freeXars.size()));
        trace.println(Trace.JCAPOOL,
                      TRACE_ID + " connection factory identification : {0}",
                      mcId);
        trace.println(Trace.JCAPOOL, TRACE_ID + " XID : {0}", xid);

        ConnectionFactoryRecord record = getRecord(mcId);
        Map xid2resourceMap = record.associatedXAResources;
        xaResource = (JalistoXAResource) xid2resourceMap.get(xid);
        if (xaResource != null) {
            trace.println(Trace.JCAPOOL, TRACE_ID + " found a XAR for XID : {0}", xid);
        } else {
            if (freeXars.isEmpty()) {
                xaResource = new JalistoXAResource(getOrBindManagedJalistoSession(mcId, mc.getConnectionURL()), trace);
            } else {
                xaResource = (JalistoXAResource) freeXars.getFirst();
                xaResource.setManagedJalistoSession(getOrBindManagedJalistoSession(mcId, mc.getConnectionURL()));
                freeXars.removeFirst();
            }
            trace.println(Trace.JCAPOOL, TRACE_ID + " bound new XAR to XID : {0}", xid);
            xid2resourceMap.put(xid, xaResource);
            xaResource.setCurrentXid(xid);
            trace.println(Trace.JCAPOOL, TRACE_ID + " bound xar : {0}", identifier2connectionFactoryRecordMap);
        }
        return xaResource;
    }

    public static void setTrace(Trace t) {
        trace = t;
        trace.println(Trace.JCAPOOL, TRACE_ID + " setTrace");
    }

    public static synchronized void releaseXAResource(JalistoXAResource jdoxar) {
        trace.println(Trace.JCAPOOL, TRACE_ID + " releaseXAResource");
        try {
            Iterator keys = identifier2connectionFactoryRecordMap.values().iterator();
            while (keys.hasNext()) {
                ConnectionFactoryRecord record = (ConnectionFactoryRecord) keys.next();
                Map cfMap = record.getAssociatedXAResources();
                Object xidKey = jdoxar.getCurrentXid();
                if (cfMap.containsKey(xidKey)) {
                    JalistoXAResource xar = (JalistoXAResource) cfMap.get(xidKey);
                    if (jdoxar.equals(xar)) { // same transaction AND same ressource
                        trace.println(Trace.JCAPOOL, TRACE_ID + " jalistoxar in Map");
                        ManagedSession mpm = xar.getManagedJalistoSession();
                        if (mpm != null) {
                            record.getAssociatedManagedSession().add(mpm);
                        }
                        cfMap.remove(xidKey);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (!freeXars.contains(jdoxar)) {
            freeXars.addLast(jdoxar);
        }
    }

    private static final ConnectionFactoryRecord getRecord(String connectionId) {
        return (ConnectionFactoryRecord) identifier2connectionFactoryRecordMap.get(connectionId);
    }

    private static synchronized ManagedSession getOrBindManagedJalistoSession(String connectionId, String connectionURL) {
        trace.println(Trace.JCAPOOL, TRACE_ID + " getOrBindManagedJalistoSession({0})", connectionId);
        ConnectionFactoryRecord record = getRecord(connectionId);
        List mpmList = record.getAssociatedManagedSession();
        ManagedSession result;
        if (!mpmList.isEmpty()) {
            result = (ManagedSession) mpmList.remove(0);
        } else {
            JalistoConnectionFactory jalistoCF = record.getConnectionFactory();
            if (jalistoCF == null) {
                throw new RuntimeException(TRACE_ID + " jalistoCF null in JalistoXARPool during binding...");
            }
            trace.println(Trace.JCAPOOL, TRACE_ID + " getOrBindManagedJalistoSession : create new ManagedJalistoSession");
            JalistoProperties prop = JalistoFactory.getInternalFactory().getProperties(connectionURL);
            ManagedSession mpm;
            if (prop.isMonoImplementation()) {
                mpm = new ManagedSessionMonoImpl(prop);
            } else if (prop.isReadOnlyImplementation()) {
                mpm = new ManagedSessionReadOnlyImpl(prop);
            } else {
                mpm = new ManagedSessionMultiImpl(prop);
            }
            result = mpm;
        }
        return result;
    }

    private static class ConnectionFactoryRecord {
        private JalistoConnectionFactory connectionFactory;
        private List associatedManagedSession;
        private Map associatedXAResources;

        public ConnectionFactoryRecord(JalistoConnectionFactory connectionFactory) {
            this.connectionFactory = connectionFactory;
            this.associatedManagedSession = new ArrayList();
            this.associatedXAResources = new HashMap();
        }

        public final List getAssociatedManagedSession() {
            return associatedManagedSession;
        }

        public final Map getAssociatedXAResources() {
            return associatedXAResources;
        }

        public final JalistoConnectionFactory getConnectionFactory() {
            return connectionFactory;
        }
    }


    private static final Map identifier2connectionFactoryRecordMap = new HashMap();
    private static LinkedList freeXars = new LinkedList();
    private static Trace trace = new TraceNull();

    private static final String TRACE_ID = "[JalistoXARPool]";
}
