/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.impl.trace.Trace;

import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;
import java.io.PrintStream;
import java.io.PrintWriter;

public class JalistoXAResourceWrapper implements XAResource {

    public JalistoXAResourceWrapper(JalistoManagedConnection mc, Trace trace) {
        this.trace = trace;
        this.mc = mc;
    }

    public boolean isAvailable() {
        return (currentJdoxar == null);
    }

    public boolean isSameRM(XAResource resource) throws XAException {
        try {
            JalistoXAResourceWrapper candidate = (JalistoXAResourceWrapper) resource;
            if (currentJdoxar == null) {
                return (candidate.currentJdoxar == null);
            }
            return currentJdoxar.isSameJDOXAR(candidate.currentJdoxar);
        } catch (ClassCastException cce) {
            return false;
        }
    }

    public boolean setTransactionTimeout(int i) throws XAException {
        this.timeout = i;
        return true;
    }

    public int getTransactionTimeout() throws XAException {
        return timeout;
    }

    public void cleanUp() {
        out("cleanUpJalisto()");
        currentJdoxar = null;
    }

    public void commit(Xid xid, boolean b) throws XAException {
        JalistoXAResource res = JalistoXARPool.getBoundXAResource(xid, mc);
        if (res != null) {
            res.commit(xid, b);
        } else {
            // throw new XAException("No resource binded with xid "+String.valueOf(xid));
        }
    }

    public void end(Xid xid, int i) throws XAException {
        out("end");
    }

    public void forget(Xid xid) throws XAException {
        out("forget");
    }

    public int prepare(Xid xid) throws XAException {
        out("prepare");
        // No 2PC
        return XA_OK;
    }

    public Xid[] recover(int i) throws XAException {
        out("recover");
        return new Xid[0];
    }

    public void rollback(Xid xid) throws XAException {
        JalistoXAResource res = JalistoXARPool.getBoundXAResource(xid, mc);
        if (res != null) {
            res.rollback(xid);
        } else {
            // throw new XAException("No resource binded with xid "+String.valueOf(xid));
        }
    }

    public void start(Xid xid, int i) throws XAException {
        try {
            currentJdoxar = JalistoXARPool.getOrBindJalistoXAResource(xid, mc);
            currentJdoxar.start(xid, mc);
        } catch (Throwable t) {
            t.printStackTrace();
            throw new JalistoXAException("ERROR during IES association : ", t);
        }
        out("start");
    }

    public void boundUnderlyingXaResource(Xid xid) throws XAException {
        try {
            currentJdoxar = JalistoXARPool.getOrBindJalistoXAResource(xid, mc);
            mc.setManagedJalistoSession(currentJdoxar.getManagedJalistoSession());
        } catch (Throwable t) {
            t.printStackTrace();
            throw new JalistoXAException("ERROR during IES association : ", t);
        }
        out("boundUnderlyingXaResource");
    }

    private void out(Object o) {
        if (trace != null) {
            trace.println(Trace.JCA, TRACE_ID + " {0}", o);
        }
    }

    public class JalistoXAException extends XAException {
        private Throwable cause;

        public JalistoXAException(String message, Throwable throwable) {
            super(message);
            this.cause = throwable;
        }

        public JalistoXAException(Throwable throwable) {
            super(throwable.getClass().getName() + ": " + throwable.getMessage());
            this.cause = throwable;
        }

        public void printStackTrace() {
            this.printStackTrace(System.err);
        }

        public void printStackTrace(PrintStream s) {
            super.printStackTrace(s);
            if (cause != null) {
                s.println("CAUSED BY:");
                cause.printStackTrace(s);
            }
        }

        public void printStackTrace(PrintWriter s) {
            super.printStackTrace(s);
            if (cause != null) {
                s.println("CAUSED BY:");
                cause.printStackTrace(s);
            }
        }
    }


    private JalistoManagedConnection mc;
    private JalistoXAResource currentJdoxar = null;
    private int timeout;

    private Trace trace;


    private static final String TRACE_ID = "[JalistoXAResourceWrapper]";

}
