/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca.session;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.impl.mono.SessionMonoImpl;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.JalistoFactory;

import javax.transaction.xa.Xid;

public class ManagedSessionMonoImpl extends SessionMonoImpl implements ManagedSession {

    public ManagedSessionMonoImpl(JalistoProperties prop) {
        super(prop);
    }

    public Xid getCurrentXid() {
        return currentXid;
    }

    protected Transaction createTransaction() {
        return new ManagedTransactionImpl(this, trace);
    }


    public synchronized void open() {
        trace("open");
        isClosed = false;
    }

    public synchronized void close() {
        trace("close");
        isClosed = true;
    }


    public void startTransaction(Xid xid) {
        currentXid = xid;
        this.open();
    }

    public void finishTransaction() {
        currentXid = null;
    }


    private void trace(Object message) {
        if (trace == null) {
            trace = JalistoFactory.getInternalFactory().getTracer(props);
        }
        if (trace != null) {
            trace.println(Trace.JCA, TRACE_ID + " " + message);
        }
    }


    private Xid currentXid;
    private boolean isClosed = true;

    private static final String TRACE_ID = "[ManagedJalistoSession]";
}
