/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jmx;

import org.objectweb.jalisto.se.impl.server.oid.OidTableImpl;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.cache.JalistoCache;

import javax.management.*;
import java.util.Map;

public class CacheAdmin extends JalistoAbstractDynamicMBean {

    public CacheAdmin(String propertiesPath) {
        this.propertiesPath = propertiesPath;
        this.provider = SessionProvider.getInstance();
    }


    public int getOidTableCurrentSize() throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            OidTableImpl oidTable = (OidTableImpl) session.getOidTable();
            return oidTable.getTransactionalSize(session.getSessionId());
        } catch (Exception e) {
            throw new MBeanException(e, "failed to get OidTableCurrentSize");
        }
    }

    public int getOidTableMaxSize() throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            OidTableImpl oidTable = (OidTableImpl) session.getOidTable();
            return oidTable.getOidTableSize();
        } catch (Exception e) {
            throw new MBeanException(e, "failed to get OidTableMaxSize");
        }
    }

    public int getPageCacheCurrentSize() throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            return session.getFileAccess().getPhysicalAccess().getBaseImage().getReadObjects().size();
        } catch (Exception e) {
            throw new MBeanException(e, "failed to get PageCacheCurrentSize");
        }
    }

    public int getPageCacheMaxSize() throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            Map map = session.getFileAccess().getPhysicalAccess().getBaseImage().getReadObjects();
            if (map instanceof JalistoCache) {
                return ((JalistoCache) map).getMaxSize();
            } else {
                return -1;
            }
        } catch (Exception e) {
            throw new MBeanException(e, "failed to get OidTableMaxSize");
        }
    }

    public void setOidTableMaxSize(int size) throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            OidTableImpl oidTable = (OidTableImpl) session.getOidTable();
            oidTable.setOidTableSize(size);
        } catch (Exception e) {
            throw new MBeanException(e, "failed to set OidTableMaxSize");
        }
    }

    public void setPageCacheMaxSize(int size) throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            Map map = session.getFileAccess().getPhysicalAccess().getBaseImage().getReadObjects();
            if (map instanceof JalistoCache) {
                ((JalistoCache) map).setMaxSize(size);
            }
        } catch (Exception e) {
            throw new MBeanException(e, "failed to set PageCacheMaxSize");
        }
    }

    public boolean getKeepInMemory() throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            return session.getFileAccess().getPhysicalAccess().getBaseImage().isKeepInMemory();
        } catch (Exception e) {
            throw new MBeanException(e, "failed to get KeepInMemory");
        }
    }

    public void setKeepInMemory(boolean keep) throws MBeanException {
        try {
            SessionInternal session = provider.getSessionForMbean(propertiesPath);
            session.getFileAccess().getPhysicalAccess().getBaseImage().setKeepInMemory(keep);
        } catch(Exception e) {
            throw new MBeanException(e, "failed to set KeepInMemory");
        }
    }


    /*****************************   DYNAMIC MBEAN IMPLEMENTATION   *****************************/

    public MBeanInfo getMBeanInfo() {
        if (infos == null) {
            MBeanAttributeInfo[] attributesInfos = new MBeanAttributeInfo[5];
            attributesInfos[0] = new MBeanAttributeInfo(
                    "OidTableCurrentSize",
                    "java.lang.Integer",
                    "Number of page used in oid table's cache.",
                    true,
                    false,
                    false);
            attributesInfos[1] = new MBeanAttributeInfo(
                    "OidTableMaxSize",
                    "java.lang.Integer",
                    "Maximum size of oid table's cache.",
                    true,
                    true,
                    false);
            attributesInfos[2] = new MBeanAttributeInfo(
                    "PageCacheCurrentSize",
                    "java.lang.Integer",
                    "Number of page used in page cache.",
                    true,
                    false,
                    false);
            attributesInfos[3] = new MBeanAttributeInfo(
                    "PageCacheMaxSize",
                    "java.lang.Integer",
                    "Maximum size of page cache.",
                    true,
                    true,
                    false);
            attributesInfos[4] = new MBeanAttributeInfo(
                    "KeepInMemory",
                    "java.lang.Boolean",
                    "Tell if readed page are keep in memory after commit or rollback.",
                    true,
                    true,
                    false);

            infos = new MBeanInfo(this.getClass().getName(),
                                  "Administration MBean to watch caches used by Jalisto",
                                  attributesInfos,
                                  new MBeanConstructorInfo[0],
                                  new MBeanOperationInfo[0],
                                  new MBeanNotificationInfo[0]);
        }
        return infos;
    }

    protected Object internalGetAttribute(String attributeName)
            throws AttributeNotFoundException, MBeanException, ReflectionException {
        if (attributeName.equals("OidTableCurrentSize")) {
            return new Integer(getOidTableCurrentSize());
        }
        if (attributeName.equals("OidTableMaxSize")) {
            return new Integer(getOidTableMaxSize());
        }
        if (attributeName.equals("PageCacheCurrentSize")) {
            return new Integer(getPageCacheCurrentSize());
        }
        if (attributeName.equals("PageCacheMaxSize")) {
            return new Integer(getPageCacheMaxSize());
        }
        if (attributeName.equals("KeepInMemory")) {
            return new Boolean(getKeepInMemory());
        }

        throw(new AttributeNotFoundException(
                "Cannot find " + attributeName + " attribute in " + this.getClass().getName()));
    }

    protected boolean internalSetAttribute(String name, Object value)
            throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        if (name.equals("OidTableCurrentSize")) {
            throw(new AttributeNotFoundException(
                    "Cannot set attribute " + name + " because it is read-only"));
        } else if (name.equals("OidTableMaxSize")) {
            setOidTableMaxSize(((Integer)value).intValue());
            return true;
        } else if (name.equals("PageCacheCurrentSize")) {
            throw(new AttributeNotFoundException(
                    "Cannot set attribute " + name + " because it is read-only"));
        } else if (name.equals("PageCacheMaxSize")) {
            setPageCacheMaxSize(((Integer)value).intValue());
            return true;
        } else if (name.equals("KeepInMemory")) {
            setKeepInMemory(((Boolean)value).booleanValue());
            return true;
        }
        return false;
    }

    protected Object internalInvoke(String operationName, Object[] params, String[] signature)
            throws MBeanException, ReflectionException {
        throw new ReflectionException(new NoSuchMethodException(operationName),
                                      "Cannot find the operation " + operationName +
                                      " in " + this.getClass().getName());
    }


    private MBeanInfo infos;
    private String propertiesPath;
    private SessionProvider provider;
}
