/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jmx;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.MetaRepository;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.impl.meta.type.MetaTypeFactory;
import org.objectweb.jalisto.se.jmx.exception.MBeanAdminException;

import javax.management.*;
import java.util.Iterator;

public class ClassDescriptionAdmin extends JalistoAbstractDynamicMBean {

    public ClassDescriptionAdmin(String propertiesPath) {
        this.propertiesPath = propertiesPath;
        this.provider = SessionProvider.getInstance();
    }

    public String getClassDescription(String filter) {
        try {
            ClassDescription meta = internalGetClassDescription(filter);
            return meta.toStringFull(0);
        } catch (MBeanAdminException JalistoAdminExc) {
            return JalistoAdminExc.getMessage();
        }
    }

    public int getNumberOfPersistentClass() throws MBeanException {
        Session session = null;
        try {
            session = provider.getSessionForMbean(propertiesPath);
            session.openSession();
            return session.getMetaRepository().getAllClassNames().size();
        } catch(Exception e) {
            e.printStackTrace();
            throw new MBeanException(e, "failed to get NumberOfPersitentClass");
        } finally {
            if (session != null ) session.closeSession();
        }
    }

    public String getPersistentClassNames() throws MBeanException {
        Session session = null;
        try {
            session = provider.getSessionForMbean(propertiesPath);
            session.openSession();
            return session.getMetaRepository().getAllClassNames().toString();
        } catch(Exception e) {
            e.printStackTrace();
            throw new MBeanException(e, "failed to get PersitentClassNames");
        } finally {
            if (session != null ) session.closeSession();
        }
    }

    public String defineClass(String className) {
        try {
            internalGetClassDescription(className);
        } catch (MBeanAdminException JalistoAdminExc) {
            if (JalistoAdminExc.getMessage().startsWith("No persistent class available")) {
                Session session = provider.getSessionForMbean(propertiesPath);
                session.getMetaRepository().defineClass(session, JalistoFactory.createClassDescription(className));
                return session.getMetaRepository().getMetaData(className).toStringFull(0);
            } else {
                throw JalistoAdminExc;
            }
        }
        throw new MBeanAdminException("Class "+className+" is already define !");
    }

    public String removeClass(String classNameFilter) {
        ClassDescription meta = internalGetClassDescription(classNameFilter);
        Session session = provider.getSessionForMbean(propertiesPath);
        session.getMetaRepository().removeClass(session, meta.getClassName());
        return "Class "+meta.getClassName()+" has been removed";
    }

    public String addField(String classNameFilter, String fieldName, String fieldType, Short index) {
        ClassDescription meta = internalGetClassDescription(classNameFilter);
        FieldDescription fieldDescription = JalistoFactory.createFieldDescription(fieldName, MetaTypeFactory.getMetaType(fieldType));
        fieldDescription.setIndex(index.shortValue());
        Session session = provider.getSessionForMbean(propertiesPath);
        session.getMetaRepository().addField(session, meta.getClassName(), fieldDescription);
        return session.getMetaRepository().getMetaData(meta.getClassName()).toStringFull(0);
    }

    public String removeField(String classNameFilter, String fieldName) {
        ClassDescription meta = internalGetClassDescription(classNameFilter);
        Session session = provider.getSessionForMbean(propertiesPath);
        session.getMetaRepository().removeField(session, meta.getClassName(), "");
        return session.getMetaRepository().getMetaData(meta.getClassName()).toStringFull(0);
    }

    private ClassDescription internalGetClassDescription(String filter) {
        MetaRepository repository = provider.getSessionForMbean(propertiesPath).getMetaRepository();
        Iterator names = repository.getAllClassNames().iterator();
        ClassDescription meta = null;
        while (names.hasNext()) {
            String name = (String) names.next();
            if (name.indexOf(filter) != -1) {
                if (meta == null) {
                    meta = repository.getMetaData(name);
                } else {
                    throw new MBeanAdminException("More than one persistent class available with filter "+filter);
                }
            }
        }
        if (meta == null) {
            throw new MBeanAdminException("No persistent class available with filter "+filter);
        }
        return meta;
    }


    /*****************************   DYNAMIC MBEAN IMPLEMENTATION   *****************************/

    public MBeanInfo getMBeanInfo() {
        if (infos == null) {
            MBeanAttributeInfo[] attributesInfos = new MBeanAttributeInfo[2];
            attributesInfos[0] = new MBeanAttributeInfo(
                    "NumberOfPersistentClass",
                    "java.lang.Integer",
                    "Number of persistent class defined in Jalisto instance.",
                    true,
                    false,
                    false);
            attributesInfos[1] = new MBeanAttributeInfo(
                    "PersistentClassNames",
                    "java.lang.String",
                    "Persistent class names defined in Jalisto instance.",
                    true,
                    false,
                    false);

            MBeanOperationInfo[] operationsInfos = new MBeanOperationInfo[5];
            operationsInfos[0] = new MBeanOperationInfo(
                    "getClassDescription",
                    "Get String representation of the persistent class's metadata.",
                    new MBeanParameterInfo[]{new MBeanParameterInfo("classNameFilter", "java.lang.String", null)},
                    "java.lang.String",
                    MBeanOperationInfo.INFO);
            operationsInfos[1] = new MBeanOperationInfo(
                    "defineClass",
                    "Define a new persistent class in base, only with the class name.",
                    new MBeanParameterInfo[]{new MBeanParameterInfo("fullClassName", "java.lang.String", null)},
                    "java.lang.String",
                    MBeanOperationInfo.ACTION);
            operationsInfos[2] = new MBeanOperationInfo(
                    "removeClass",
                    "Remove a persistent class from base.",
                    new MBeanParameterInfo[]{new MBeanParameterInfo("classNameFilter", "java.lang.String", null)},
                    "java.lang.String",
                    MBeanOperationInfo.ACTION);
            operationsInfos[3] = new MBeanOperationInfo(
                    "addField",
                    "Add a field to a persistent class.",
                    new MBeanParameterInfo[]{
                        new MBeanParameterInfo("classNameFilter", "java.lang.String", null),
                        new MBeanParameterInfo("fieldName", "java.lang.String", null),
                        new MBeanParameterInfo("fieldType", "java.lang.String", null),
                        new MBeanParameterInfo("index", "java.lang.Short", null)},
                    "java.lang.String",
                    MBeanOperationInfo.ACTION);
            operationsInfos[4] = new MBeanOperationInfo(
                    "removeField",
                    "remove a field from a persistent class.",
                    new MBeanParameterInfo[]{
                        new MBeanParameterInfo("classNameFilter", "java.lang.String", null),
                        new MBeanParameterInfo("fieldName", "java.lang.String", null)},
                    "java.lang.String",
                    MBeanOperationInfo.ACTION);

            infos = new MBeanInfo(this.getClass().getName(),
                                  "Administration MBean for the Jalisto persistent class metadatas",
                                  attributesInfos,
                                  new MBeanConstructorInfo[0],
                                  operationsInfos,
                                  new MBeanNotificationInfo[0]);
        }
        return infos;
    }

    protected Object internalGetAttribute(String attributeName)
            throws AttributeNotFoundException, MBeanException, ReflectionException {
        if (attributeName.equals("NumberOfPersistentClass")) {
            return new Integer(getNumberOfPersistentClass());
        }
        if (attributeName.equals("PersistentClassNames")) {
            return getPersistentClassNames();
        }

        throw(new AttributeNotFoundException(
                "Cannot find " + attributeName + " attribute in " + this.getClass().getName()));
    }

    protected boolean internalSetAttribute(String name, Object value)
            throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        if (name.equals("NumberOfPersistentClass")) {
            throw(new AttributeNotFoundException(
                    "Cannot set attribute " + name + " because it is read-only"));
        } else if (name.equals("PersistentClassNames")) {
            throw(new AttributeNotFoundException(
                    "Cannot set attribute " + name + " because it is read-only"));
        }
        return false;
    }

    protected Object internalInvoke(String operationName, Object[] params, String[] signature)
            throws MBeanException, ReflectionException {
        if (operationName.equals("getClassDescription")) {
            return getClassDescription((String) params[0]);
        } else if (operationName.equals("defineClass")) {
            return defineClass((String) params[0]);
        } else if (operationName.equals("removeClass")) {
            return removeClass((String) params[0]);
        } else if (operationName.equals("addField")) {
            return addField((String) params[0],
                            (String) params[1],
                            (String) params[2],
                            (Short) params[3]);
        } else if (operationName.equals("removeField")) {
            return removeField((String) params[0],
                               (String) params[1]);
        } else {
            throw new ReflectionException(new NoSuchMethodException(operationName),
                                          "Cannot find the operation " + operationName +
                                          " in " + this.getClass().getName());
        }
    }


    private String propertiesPath;
    private SessionProvider provider;
    private MBeanInfo infos;
}
