/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.constraint;

import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.query.FielComparator;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.query.exception.QueryEngineException;
import org.objectweb.jalisto.se.query.execution.ExecutionTree;
import org.objectweb.jalisto.se.query.parameter.SubQuery;
import org.objectweb.jalisto.se.query.result.QueryResultWrapper;

import java.util.ArrayList;

public class SubQueryConstraintImpl extends ConstraintImpl {
    public SubQueryConstraintImpl(SubQuery subQuery, String className, ArrayList subFieldNames) {
        this.className = className;
        this.subQuery = subQuery;
        this.fieldNames = new String[subFieldNames.size()];
        for (int i = 0; i < fieldNames.length; i++) {
            fieldNames[i] = (String) subFieldNames.get(i);
        }
    }

    public Object getObject() {
        return subQuery;
    }

    public Query getQuery() {
        return subQuery.getQuery();
    }

    public ExecutionTree getTree() {
        return subQuery.getTree(null);
    }

    public String getFieldName() {
        return fieldNames[0];
    }

    public String[] getFieldNames() {
        return fieldNames;
    }

    public Constraint contains() {
        throw new QueryEngineException("cannot do this operation on a subQuery");
    }

    public Constraint equal() {
        throw new QueryEngineException("cannot do this operation on a subQuery");
    }

    public Constraint greater() {
        throw new QueryEngineException("cannot do this operation on a subQuery");
    }

    public Constraint identity() {
        throw new QueryEngineException("cannot do this operation on a subQuery");
    }

    public Constraint like() {
        throw new QueryEngineException("cannot do this operation on a subQuery");
    }

    public Constraint not() {
        not = !not;
        return this;
    }

    public Constraint smaller() {
        throw new QueryEngineException("cannot do this operation on a subQuery");
    }

    public boolean execute(Session session, FielComparator comparator, Object candidate) {
        return subQuery.getTree(null).resolveOnOneElement(
                (SessionInternal) session,
                (QueryResultWrapper) candidate);
    }

    public String getClassName() {
        return className;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("SubQueryConstraint");
        return sb.toString();
    }

    private SubQuery subQuery;
    private String className;
    private String[] fieldNames;
    private boolean not = false;
}
