/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.execution;

import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.MetaRepository;
import org.objectweb.jalisto.se.api.query.IndexManager;
import org.objectweb.jalisto.se.query.constraint.BinaryBooleanOperatorConstraint;
import org.objectweb.jalisto.se.query.result.QueryResultWrapper;
import org.objectweb.jalisto.se.query.result.WrapperSet;

import java.util.List;

public class ExecutionNode extends ExecutionElement {

    public ExecutionNode(ExecutionTree tree, ExecutionElement father,
                         BinaryBooleanOperatorConstraint binaryConstraint, int depth) {
        super(tree, father, depth);
        this.andOp = binaryConstraint.isAndOperator();
    }

    public void setLeft(ExecutionElement left) {
        this.left = left;
    }

    public void setRight(ExecutionElement right) {
        this.right = right;
    }

    public boolean onlyOneToResolve() {
        if (left.isResolved()) {
            return !right.isResolved();
        }
        return right.isResolved();
    }

    public boolean isAndNode() {
        return andOp;
    }

    public boolean isInOrBranch(IndexManager indexManager) {
        return ((!andOp) && (left.isInOrBranch(indexManager) || right.isInOrBranch(indexManager)));
    }

    public void cleanSubExecutionTree() {
        super.cleanSubExecutionTree();
        left.cleanSubExecutionTree();
        right.cleanSubExecutionTree();
    }

    public void cleanAll() {
        super.cleanAll();
        left.cleanAll();
        right.cleanAll();
    }

    public void defineMeta(MetaRepository repository) {
        left.defineMeta(repository);
        right.defineMeta(repository);
    }

    /**
     * ******************************   RESOLVE   *******************************************
     */

    public ExecutionNode resolveBack() {
        if (isResolved()) {
            return getFather();
        }

        if (left.isResolved() && right.isResolved()) {
            WrapperSet setLeft = left.getResolvedValues();
            WrapperSet setRight = right.getResolvedValues();
            if (andOp) {
                doIntersection(resolvedValues, setLeft, setRight);
                tree.getToResolveAndNodes().remove(this);
            } else {
                resolvedValues.addAll(setLeft);
                resolvedValues.addAll(setRight);
            }
            isResolved = true;
            if (andOp) {
                tree.getToResolveAndNodes().remove(this);
            }
            return getFather();
        }

        return null;
    }

    public void resolveBeforeAndNodeNearRoot(Session session, WrapperSet candidates, List toResolve) {
        // one is resolved
        ExecutionElement resolved;
        ExecutionElement unresolved;
        if (left.isResolved()) {
            resolved = left;
            unresolved = right;
        } else {
            resolved = right;
            unresolved = left;
        }

        if (candidates != null) {
            unresolved.resolveBeforeAndNodeNearRoot(
                    session,
                    doIntersection(new WrapperSet(), candidates, resolved.getResolvedValues()),
                    toResolve);
        } else {
            unresolved.resolveBeforeAndNodeNearRoot(session, resolved.getResolvedValues(), toResolve);
        }

        toResolve.remove(this);
    }

    public boolean resolveOnElement(Session session, QueryResultWrapper wrapper) {
        if (andOp) {
            return left.resolveOnElement(session, wrapper) && right.resolveOnElement(session, wrapper);
        } else {
            return left.resolveOnElement(session, wrapper) || right.resolveOnElement(session, wrapper);
        }
    }

    public WrapperSet resolveWithCandidates(Session session, WrapperSet candidates) {
        if (isResolved()) {
            return doIntersection(new WrapperSet(), candidates, resolvedValues);
        }

        // one is resolved
        ExecutionElement resolved;
        ExecutionElement unresolved;
        if (left.isResolved()) {
            resolved = left;
            unresolved = right;
        } else {
            resolved = right;
            unresolved = left;
        }

        WrapperSet wSet = doIntersection(new WrapperSet(), candidates, resolved.getResolvedValues());
        resolvedValues = unresolved.resolveWithCandidates(session, wSet);
        isResolved = true;

        if (andOp) {
            tree.getToResolveAndNodes().remove(this);        // really needed ?
        }

        return resolvedValues;
    }

    public String toString() {
        if (andOp) {
            return "Enode&&";
        } else {
            return "Enode||";
        }
    }


    private boolean andOp;
    private ExecutionElement left;
    private ExecutionElement right;

}
