/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.execution;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.MetaRepository;
import org.objectweb.jalisto.se.api.query.Index;
import org.objectweb.jalisto.se.api.query.IndexManager;
import org.objectweb.jalisto.se.query.exception.QueryEngineException;
import org.objectweb.jalisto.se.query.NavigationInfo;
import org.objectweb.jalisto.se.query.constraint.ValueConstraintImpl;
import org.objectweb.jalisto.se.query.result.QueryResultWrapper;
import org.objectweb.jalisto.se.query.result.WrapperSet;

import java.util.Iterator;

public class ValueExecutionLeaf extends ExecutionElement {

    public ValueExecutionLeaf(ExecutionTree tree, ExecutionElement father,
                              ValueConstraintImpl valueConstraint, int depth) {
        super(tree, father, depth);
        this.valueConstraint = valueConstraint;
        setToLeaf(0);
    }

    // not an index => "Or branch" i.e. not resolvable branch
    public boolean isInOrBranch(IndexManager indexManager) {
        return (navigationInfos.getIndexIfa() == null);
    }

    public boolean isResolved() {
        return (super.isResolved() || valueConstraint.isResolved());
    }

    public WrapperSet getResolvedValues() {
        if (resolvedValues.isEmpty() && valueConstraint.isResolved()) {
            return valueConstraint.getResolvedValues();
        }
        return resolvedValues;
    }

    public void cleanAll() {
        super.cleanAll();
        valueConstraint.clean();
    }

    public void defineMeta(MetaRepository repository) {
        ClassDescription meta = repository.getMetaData(valueConstraint.getClassName());
        navigationInfos = new NavigationInfo(meta, valueConstraint.getFieldNames());
    }

    /**
     * *****************************   RESOLVE   ******************************************
     * resolve this leaf if field has an index
     */

    // resolve this leaf if field has an index
    public void resolveLeafOnIndex(Session session, IndexManager indexManager) {
        if (!navigationInfos.isIndexDefined()) {
            return;
        }
        if (!valueConstraint.isResolved()) {
            WrapperSet result = valueConstraint.getResolvedValues();     // get empty collection to fill it
            Index indexOnValues = navigationInfos.getIndex(indexManager);
            Iterator keys = indexOnValues.keySet().iterator();
            while (keys.hasNext()) {
                Object value = keys.next();
                if (valueConstraint.execute(session, navigationInfos.getComparator(), value)) {
                    Iterator oids = indexOnValues.get(value).iterator();
                    while (oids.hasNext()) {
                        LogicalOid oid = (LogicalOid) oids.next();
                        result.addToUnread(new QueryResultWrapper(oid, null));
                    }
                }
            }
            valueConstraint.setResolved(true);
        }
    }

    public boolean resolveOnElement(Session session, QueryResultWrapper wrapper) {
        Object value = wrapper.getValue()[navigationInfos.getIndex()];
        return valueConstraint.execute(session, navigationInfos.getComparator(), value);
    }

    // not on a resolved leaf
    public WrapperSet resolveWithCandidates(Session session, WrapperSet candidates) {
        if (valueConstraint.isResolved()) {
            throw new QueryEngineException("resolveWithCandidates on a resolved leaf");
        }
        candidates.readAll(session, tree.getReadedValues());
        Iterator wrappers = candidates.iterator();
        while (wrappers.hasNext()) {
            QueryResultWrapper wrapper = (QueryResultWrapper) wrappers.next();
            if (resolveOnElement(session, wrapper)) {
                resolvedValues.addToRead(wrapper);
            }
        }
        return resolvedValues;
    }

    public String toString() {
        return "Eleaf:" + valueConstraint.toString();
    }

    private ValueConstraintImpl valueConstraint;
    private NavigationInfo navigationInfos;
}
