/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.index;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.FielComparator;
import org.objectweb.jalisto.se.api.query.Index;
import org.objectweb.jalisto.se.api.query.IndexManager;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.query.IndexManagerImpl;
import org.objectweb.jalisto.se.query.btree.BTree;

import java.util.Set;

public class IndexBTreeImpl implements Index {

    public IndexBTreeImpl(InFileAddress ifa, int fieldIndex,
                             ClassDescription meta, IndexManagerImpl indexManager) {
//        System.out.println("Build a BTree index");
        this.tree = new BTree(this, fieldIndex, meta, indexManager);
        this.ifa = ifa;
        this.className = meta.getClassName();
        this.fieldName = meta.getFieldNames()[fieldIndex];
    }

    public Set get(Object key) {
        return tree.getOidsFor(key);
    }

    public FielComparator getComparator() {
        return tree.getComparator();
    }

    public Set keySet() {
        return tree.getKeys();
    }

    public void put(Object key, LogicalOid value) {
        tree.addOid(key, value);
    }

    public void remove(Object key, LogicalOid logicalOid) {
        tree.removeOid(key, logicalOid);
    }

    public InFileAddress getIfa() {
        return ifa;
    }

    public void setIndexManager(IndexManager indexManager) {
        tree.setIndexManager((IndexManagerImpl) indexManager);
    }

    public void deleteIndex() {
        tree.deleteTree();
    }

    public String toFullString() {
        StringBuffer sb = new StringBuffer();
        sb.append("IndexBTreeImpl on class ").append(className);
        sb.append(" and field ").append(fieldName);
        sb.append(":\n").append(tree);
        return sb.toString();
    }

    public BTree getTree() {
        return tree;
    }

    public String toString() {
        return "IndexBTreeImpl(" + className + "," + fieldName + ")";
    }

    private InFileAddress ifa;
    private BTree tree;
    private String className;
    private String fieldName;

    public static final short TYPE = 1;
}
