/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.result;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.Session;

import java.util.*;

public class WrapperSet {

    public WrapperSet() {
        this.readedWrappers = new HashMap();
        this.noneReadedWrappers = new HashMap();
    }

    public int size() {
        return readedWrappers.size() + noneReadedWrappers.size();
    }

    public boolean isEmpty() {
        return readedWrappers.isEmpty() && noneReadedWrappers.isEmpty();
    }

    public boolean contains(QueryResultWrapper w) {
        return readedWrappers.containsKey(w.getLogicalOid()) || noneReadedWrappers.containsKey(w.getLogicalOid());
    }

    // return -1 si dans readed
    // return 1 si dans non readed
    // return 0 sinon
    public int containsIn(QueryResultWrapper w) {
        if (readedWrappers.containsKey(w.getLogicalOid()))
            return -1;
        if (noneReadedWrappers.containsKey(w.getLogicalOid()))
            return 1;
        return 0;
    }

    public Iterator iterator() {
        ArrayList list = new ArrayList();
        list.addAll(readedWrappers.values());
        list.addAll(noneReadedWrappers.values());
        return list.iterator();
    }

    public void add(QueryResultWrapper w) {
        if (w.getValue() == null) {
            noneReadedWrappers.put(w.getLogicalOid(), w);
        } else {
            readedWrappers.put(w.getLogicalOid(), w);
        }
    }

    public void addToUnread(QueryResultWrapper w) {
        noneReadedWrappers.put(w.getLogicalOid(), w);
    }

    public void addToRead(QueryResultWrapper w) {
        readedWrappers.put(w.getLogicalOid(), w);
    }

    public void addAll(Collection collection) {
        Iterator iterator = collection.iterator();
        while (iterator.hasNext()) {
            add((QueryResultWrapper) iterator.next());
        }
    }

    public void addAll(WrapperSet wSet) {
        readedWrappers.putAll(wSet.readedWrappers);
        noneReadedWrappers.putAll(wSet.noneReadedWrappers);
    }

    public QueryResultWrapper get(Object key) {
        if (readedWrappers.containsKey(key)) {
            return (QueryResultWrapper) readedWrappers.get(key);
        } else {
            return (QueryResultWrapper) noneReadedWrappers.get(key);
        }
    }

    public Set getCopyInSet() {
        HashSet result = new HashSet();
        result.addAll(readedWrappers.values());
        result.addAll(noneReadedWrappers.values());
        return result;
    }

    public void remove(QueryResultWrapper w) {
        Object loid = w.getLogicalOid();
        if (readedWrappers.remove(loid) == null) {
            noneReadedWrappers.remove(loid);
        }
    }

    public void clear() {
        readedWrappers.clear();
        noneReadedWrappers.clear();
    }

    public void readAll(Session session, Map valueMap) {
        Iterator none = noneReadedWrappers.values().iterator();
        while (none.hasNext()) {
            QueryResultWrapper wrapper = (QueryResultWrapper) none.next();
            LogicalOid loid = wrapper.getLogicalOid();
            if (valueMap.containsKey(loid)) {
                wrapper.setValue((Object[]) valueMap.get(loid));
            } else {
                Object[] value = session.readObjectByOid(loid);
                wrapper.setValue(value);
                valueMap.put(loid, value);
            }
            readedWrappers.put(loid, wrapper);
        }
        noneReadedWrappers.clear();
    }

    private Map readedWrappers;
    private Map noneReadedWrappers;
}
