/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.query.parameter.QueryParameter;
import org.objectweb.jalisto.se.query.result.ObjectSetImpl;
import org.objectweb.jalisto.se.test.data.*;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

public class LinkQueryTestCase extends JalistoQueryTestCase {

    public LinkQueryTestCase() {
    }

    public LinkQueryTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        LinkQueryTestCase tc = (LinkQueryTestCase) newTestCase(suite, new LinkQueryTestCase());

        tc.prepare();

        tc.testQueryWithLink("without index and parameters", false, numberOfQueryExecute);
        tc.testQueryWithLink("without index, with parameters", true, numberOfQueryExecute);

        tc.buildIndex(BookWithFormat.class, "title");

        tc.testQueryWithLink("with index, without parameters", false, numberOfQueryExecute);
        tc.testQueryWithLink("with index, with parameters", true, numberOfQueryExecute);

        tc.deleteIndex(BookWithFormat.class, "title");

        tc.finishTests();

        return suite;
    }

    /**
     * ************************* need to overwrite for suite() method ********************************************
     */

    public void prepare() {
        initSession();
        define(BookWithAuthor.getMetaDescription());
        define(BookWithFormat.getMetaDescription());
        define(Author.getMetaDescription());

        resetFloidValue();
        BookWithFormat.counter = -1;
        Author.counter = -1;
        Format.counter = -1;

        deleteIndex(BookWithFormat.class, "title");
        cleanUp(BookWithFormat.class);
        cleanUp(Author.class);

        populateBookWithFormat(1000);
    }

    public void testQueryWithLink(String message, boolean withParameters, int nbr) {
        String timerName = String.valueOf(nbr) + " execution " + message;

        Query query;
        if (withParameters) {
            query = getQueryWithParameters();
        } else {
            query = getQueryWithoutParameters();
        }

        int resultSize = -1;
        ObjectSetImpl resultSet = null;

        JalistoTimer.timerStart(timerName);
        for (int i = 0; i < nbr; i++) {
            tx.begin();
            resultSet = (ObjectSetImpl) query.execute();
            resultSize = resultSet.size();
            tx.commit();
        }
        JalistoTimer.timerStop(timerName, false);

        assertEquals("shoulb be equal", 8, resultSize);
        resultSet.sort();
        tx.begin();
        String r1 = "((1.0,[Death Off Stage,0,5,[James,McManus],18x13],(1.140,[Death Off Stage,140,15,[James,McManus],18x13],(1.280,[Death Off Stage,280,25,[James,McManus],18x13],(1.420,[Death Off Stage,420,5,[James,McManus],18x13],(1.560,[Death Off Stage,560,15,[James,McManus],18x13],(1.700,[Death Off Stage,700,25,[James,McManus],18x13],(1.840,[Death Off Stage,840,5,[James,McManus],18x13],(1.980,[Death Off Stage,980,15,[James,McManus],18x13])";
        assertEquals("should be equal", r1, resultSet.toStringFull(session));
        tx.commit();

        JalistoTimer.summary(timerName);
        JalistoTimer.clean(timerName);
    }

    private Query getQueryWithoutParameters() {
        String title1 = BookWithFormat.titles[0];
        String authorLastName = Author.lastNames[0];
        Format format1 = Format.availableFormat[0];

        Query query = queryManager.query();
        query.constrain(BookWithFormat.class);
        Constraint c1 = query.descend("title").constrain(title1).equal();
        Constraint c2 = query.descend("author").descend("lastName").constrain(authorLastName).equal();
        Constraint c3 = query.descend("format").constrain(format1).equal();
        c1.and(c2).and(c3);

        return query;
    }

    private Query getQueryWithParameters() {
        String title1 = BookWithFormat.titles[0];
        String authorLastName = Author.lastNames[0];
        Format format1 = Format.availableFormat[0];

        QueryParameter titleParameter = new QueryParameter("title");
        QueryParameter lastNameParameter = new QueryParameter("lastName");

        Query query = queryManager.query();
        query.constrain(BookWithFormat.class);
        Constraint c1 = query.descend("title").constrain(titleParameter).equal();
        Constraint c2 = query.descend("author").descend("lastName").constrain(lastNameParameter).equal();
        Constraint c3 = query.descend("format").constrain(format1).equal();
        c1.and(c2).and(c3);

        query.bind("title", title1);
        query.bind("lastName", authorLastName);

        return query;
    }

    public void resetFloidValue() {
        tx.begin();
        session.getInternalSession().getOidTable().getIdentityProvider().resetFloidCounters();
        tx.commit();
    }


    /**
     * ********************************************************************************
     */

    public void initSession() {
        super.initSession(false);
    }

    public void define(ClassDescription classDescription) {
        super.define(classDescription);
    }

    public void cleanUp(Class aClass) {
        super.cleanUp(aClass);
    }

    public void finishTests() {
        super.finishTests();
    }

    public void populateBookWithFormat(int nbr) {
        tx.begin();
        for (int i = 0; i < nbr; i++) {
            Object authorOid = session.createObject(Author.newAuthor().toArray(), Author.class);
            Object[] book = BookWithFormat.newBook().toArray();
            book[3] = authorOid;
            session.createObject(book, BookWithFormat.class);
        }
        tx.commit();
    }

    public void buildIndex(Class aClass, String fieldName) {
        super.buildIndex(aClass, fieldName);
    }

    public void deleteIndex(Class aClass, String fieldName) {
        super.deleteIndex(aClass, fieldName);
    }

    public void printIndex(Class aClass, String fieldName) {
        super.printIndex(aClass, fieldName);
    }
}
