/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query.bench;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.test.data.BookWithAuthor;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

public class EvolutionTestCase extends JalistoTestCase {
    public EvolutionTestCase() {
    }

    public EvolutionTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        EvolutionTestCase tc = (EvolutionTestCase) newTestCase(suite, new EvolutionTestCase());

        tc.initSession();
        tc.define(BookWithAuthor.getMetaDescription());

        tc.evolutionTest(50);

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void evolutionTest(int nbrExecution) {
        int[] intValues = {1000, 5000, 10000, 20000};
        String title1 = BookWithAuthor.titles[0];
        String title2 = BookWithAuthor.titles[1];

        Query query1 = queryManager.query();
        query1.constrain(BookWithAuthor.class);
        query1.descend("title").constrain(title1).equal();

        Query query2 = queryManager.query();
        query2.constrain(BookWithAuthor.class);
        Constraint c1 = query2.descend("title").constrain(title1).equal();
        Constraint c2 = query2.descend("title").constrain(title2).equal();
        c1.or(c2);

        Query[] queries = {query1, query2};

        float[][] resultTimes = new float[intValues.length][queries.length];

        for (int i = 0; i < intValues.length; i++) {
            for (int j = 0; j < queries.length; j++) {
                cleanUp(BookWithAuthor.class);
                populateBookWithAuthor(intValues[i]);

                for (int k = 0; k < nbrExecution; k++) {
                    JalistoTimer.timerStart("evolutionTest");
                    tx.begin();
                    queries[j].execute();
                    tx.commit();
                    JalistoTimer.timerStop("evolutionTest", false);
                }
                resultTimes[i][j] = JalistoTimer.getAverage("evolutionTest");
                JalistoTimer.clean("evolutionTest");
            }
        }

        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < resultTimes.length; i++) {
            sb.append(intValues[i]).append(" : ");
            float[] f = resultTimes[i];
            for (int j = 0; j < f.length; j++) {
                sb.append(String.valueOf(f[j]) + " \t");
            }
            sb.append("\n");
        }

        System.out.println(sb.toString());

        cleanUp(BookWithAuthor.class);
    }


    /**
     * ********************************************************************************
     */

    public void initSession() {
        super.initSession(false);
    }

    public void define(ClassDescription classDescription) {
        super.define(classDescription);
    }
}
