/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.samples.basic;

import org.objectweb.jalisto.samples.basic.data.Book;
import org.objectweb.jalisto.se.impl.meta.type.MetaTypeFactory;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.JalistoFactory;

public class JalistoDemo {

    private Session session;
    private Transaction tx;

    private Object oid;

    public static void main(String[] args) {
        JalistoDemo demo = new JalistoDemo();

        demo.initJalisto("samples.properties");
        demo.defineData();
        demo.createBook();
        demo.updateBook();
        demo.deleteBook();
        demo.closeJalisto();
    }

    public void initJalisto(String propertiesFile) {
        System.out.println("JalistoDemo.initJalisto : "+propertiesFile);

        // get jalisto working session
        session = JalistoFactory.getSession(propertiesFile);
        session.openSession();
        // get the transaction associated to this session
        tx = session.currentTransaction();
    }

    public void closeJalisto() {
        System.out.println("JalistoDemo.closeJalisto");

        session.closeSession();
    }

    public void defineData() {
        System.out.println("JalistoDemo.defineData");

        // define structure of persistent data
        ClassDescription meta = JalistoFactory.createClassDescription(Book.class.getName());
        meta.addField(JalistoFactory.createFieldDescription("title", MetaTypeFactory.StringType));
        meta.addField(JalistoFactory.createFieldDescription("author", MetaTypeFactory.StringType));
        meta.addField(JalistoFactory.createFieldDescription("price", MetaTypeFactory.IntegerType));

        // define this structure in jalisto
        session.defineClass(meta);
    }

    public void createBook() {
        System.out.println("JalistoDemo.createBook");

        tx.begin();
        Book book = new Book("The Bible", "God", 777);
        Object[] array = book.toArray();
        oid = session.createObject(array, Book.class);
        tx.commit();
    }

    public void updateBook() {
        System.out.println("JalistoDemo.updateBook");

        tx.begin();
        Object[] o = session.readObjectByOid(oid);
        Book book = Book.toBook(o);
        book.setPrice(8888);
        session.updateObjectByOid(oid, book.toArray());
        tx.commit();
    }

    public void deleteBook() {
        System.out.println("JalistoDemo.deleteBook");

        tx.begin();
        session.deleteObjectByOid(oid);
        tx.commit();
    }
}
