/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.samples.basic2;

import org.objectweb.jalisto.se.impl.meta.type.MetaTypeFactory;
import org.objectweb.jalisto.se.impl.*;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.samples.basic2.data.Book;

import java.util.Collection;
import java.util.Iterator;

public class JalistoBookDemo {

    public static void main(String[] args) {
        String jalistoPropertiesFilename = "jalisto.properties";
        int nbrBook = 5000;

        try {
            jalistoPropertiesFilename = args[0];
            nbrBook = Integer.parseInt(args[1]);
        } catch (Exception e) {
        }

        JalistoBookDemo demo = new JalistoBookDemo();
        demo.initialise(jalistoPropertiesFilename);
        demo.run(nbrBook);
    }

    public void initialise(String jalistoPropertiesFilename) {
        // get the Jalisto working session
        session = null;
        // get the transaction associated with this session
        tx = session.currentTransaction();

        // define structure of persistent data
        ClassDescription meta = JalistoFactory.createClassDescription(Book.class.getName());
        meta.addField(JalistoFactory.createFieldDescription("title", MetaTypeFactory.StringType));
        meta.addField(JalistoFactory.createFieldDescription("pages", MetaTypeFactory.IntegerType));
        meta.addField(JalistoFactory.createFieldDescription("comments", MetaTypeFactory.StringType));
        // define this structure in jalisto
        session.defineClass(meta);
    }

    public void run(int nbrBook) {
        long startTime;
        long execTime;
        Iterator iterator;

        System.out.println("JalistoBookDemo is running :");

        System.out.println("Create " + nbrBook + " books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            Book book = Book.newBook();
            Object[] o = book.toArray();
            Object oid = session.createObject(o, Book.class);
        }
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes " + execTime + " ms.\n");


        System.out.println("Get oids of all books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        Collection oids = session.getExtent(Book.class).readFully().collection();
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes " + execTime + " ms.\n");


        System.out.println("Add a comment to all books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        iterator = oids.iterator();
        int i = 0;
        while (iterator.hasNext()) {
            Object oid = iterator.next();
            Object[] o = session.readObjectByOid(oid);
            Book book = Book.toBook(o);
            book.setComments("comments number " + i);
            session.updateObjectByOid(oid, book.toArray());
            i++;
        }
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes " + execTime + " ms.\n");


        System.out.println("Delete all books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        iterator = oids.iterator();
        while (iterator.hasNext()) {
            session.deleteObjectByOid(iterator.next());
        }
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes " + execTime + " ms.\n");

        session.reorganize();
    }

    private Session session;
    private Transaction tx;
}
